@@ -4,13 +4,11 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: ops.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #include "pset.h"
 #include <stdlib.h>
 
-#define PRIVATE				static
 #define POINTER				__pset_pointer
 
 
@@ -49,7 +47,7 @@ void pset_compact( register pset_h pset )
 /*
  * Apply a function to all pointers of a pset
  */
-void pset_apply( register pset_h pset, register void (*func)(), register void *arg )
+void pset_apply( register pset_h pset, void (*func)(), register void *arg )
 {
 	register unsigned u ;
 
