@@ -1,16 +1,11 @@
 /*
  * (c) Copyright 1992, 1993 by Panagiotis Tsirigotis
+ * (c) Portions Copyright 1998-2001 by Rob Braun
  * All rights reserved.  The file named COPYRIGHT specifies the terms 
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: sio.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 #include "config.h"
-#ifdef SIO_VERSION
-static char sio_version[] = SIO_VERSION ;
-#endif
-
-
 #ifndef linux
 #include <sys/types.h>
 #endif
@@ -21,15 +16,10 @@ static char sio_version[] = SIO_VERSION ;
 
 #include <stdlib.h>
 #include <unistd.h>
-#include <sys/stat.h>
 
 #include "sio.h"
 #include "impl.h"
 
-#ifdef EVENTS
-#include "events.h"
-#endif
-
 /*
  * SIO WRITE FUNCTIONS: Swrite, Sputc
  */
@@ -37,24 +27,19 @@ static char sio_version[] = SIO_VERSION ;
 /*
  * Stream write call: arguments same as those of write(2)
  */
-int Swrite( fd, addr, nbytes )
-	int fd ;
-	register char *addr ;
-	register int nbytes ;
+int Swrite( int fd, const char *addr, int nbytes )
 {
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
-	register __sio_od_t *odp = ODP( dp ) ;
-	register int b_transferred ;
-	register int b_avail ;
+	__sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
+	__sio_od_t *odp ;
+	int b_transferred ;
+	int b_avail ;
 	int total_b_transferred ;
 	int b_written ;
 	int b_in_buffer ;
 
-#ifdef EVENTS
-	EVENT( fd, EV_SWRITE ) ;
-#endif
-
-	IO_SETUP( fd, dp, __SIO_OUTPUT_STREAM, SIO_ERR ) ;
+	if( sio_setup( fd, dp, __SIO_OUTPUT_STREAM ) == SIO_ERR )
+		return( SIO_ERR );
+	odp = ODP( dp ) ;
 	ASSERT( odp->start <= odp->nextb && odp->nextb <= odp->buf_end ) ;
 
 	b_avail = odp->buf_end - odp->nextb ;
@@ -117,18 +102,14 @@ int Swrite( fd, addr, nbytes )
 /*
  * Add a character to a file
  */
-int Sputc( fd, c )
-	int fd ;
-	char c ;
+int Sputc( int fd, char c )
 {
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
-	register __sio_od_t *odp = ODP( dp ) ;
+	__sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
+	__sio_od_t *odp ;
 
-#ifdef EVENTS
-	EVENT( fd, EV_SPUTC ) ;
-#endif
-	
-	IO_SETUP( fd, dp, __SIO_OUTPUT_STREAM, SIO_ERR ) ;
+	if( sio_setup( fd, dp, __SIO_OUTPUT_STREAM) == SIO_ERR )
+		return( SIO_ERR );
+	odp = ODP( dp ) ;
 	ASSERT( odp->start <= odp->nextb && odp->nextb <= odp->buf_end ) ;
 
 	/*
@@ -162,85 +143,23 @@ int Sputc( fd, c )
  * SIO READ FUNCTIONS
  */
 
-/*
- * Stream read call: arguments same as those of read(2)
- */
-int Sread( fd, addr, nbytes )
-	int fd ;
-	char *addr ;
-	int nbytes ;
-{
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
-	register __sio_id_t *idp = IDP( dp ) ;
-	register int b_transferred ;
-	int b_read ;
-	int total_b_transferred ;
-	int b_left ;
-
-#ifdef EVENTS
-	EVENT( fd, EV_SREAD ) ;
-#endif
-
-	IO_SETUP( fd, dp, __SIO_INPUT_STREAM, SIO_ERR ) ;
-	ASSERT( idp->start <= idp->nextb && idp->nextb <= idp->end ) ;
-
-	b_left = idp->end - idp->nextb ;
-	b_transferred = MIN( b_left, nbytes ) ;
-	sio_memcopy( idp->nextb, addr, b_transferred ) ;
-	idp->nextb += b_transferred ;
-	if ( b_transferred == nbytes )
-		return( b_transferred ) ;
-	
-	nbytes -= b_transferred ;
-	total_b_transferred = b_transferred ;
-	addr += b_transferred ;
-
-	do
-	{
-		b_read = __sio_readf( idp, fd ) ;
-		switch ( b_read )
-		{
-			case SIO_ERR:
-				if ( total_b_transferred == 0 )
-					return( SIO_ERR ) ;
-				/* FALL THROUGH */
-			
-			case 0:
-				return( total_b_transferred ) ;
-		}
-			
-		b_transferred = MIN( b_read, nbytes ) ;
-		sio_memcopy( idp->nextb, addr, b_transferred ) ;
-		addr += b_transferred ;
-		idp->nextb += b_transferred ;
-		total_b_transferred += b_transferred ;
-		nbytes -= b_transferred ;
-	}
-	while ( nbytes && b_read == idp->buffer_size ) ;
-	return( total_b_transferred ) ;
-}
-
-
 
 /*
  * Read a line from a file
  * Returns a pointer to the beginning of the line or NULL
  */
-char *Srdline( fd )
-	int fd ;
+char *Srdline( int fd )
 {
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
-	register __sio_id_t *idp = IDP( dp ) ;
-	register char *cp ;
-	register char *line_start ;
-	register int b_left ;
-	register int extension ;
+	__sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
+	__sio_id_t *idp ;
+	char *cp ;
+	char *line_start ;
+	int b_left ;
+	int extension ;
 
-#ifdef EVENTS
-	EVENT( fd, EV_SRDLINE ) ;
-#endif
-
-	IO_SETUP( fd, dp, __SIO_INPUT_STREAM, NULL ) ;
+	if( sio_setup( fd, dp, __SIO_INPUT_STREAM ) == SIO_ERR )
+		return( NULL );
+	idp = IDP( dp ) ;
 	ASSERT( idp->start <= idp->nextb && idp->nextb <= idp->end ) ;
 
 #ifdef HAVE_MMAP
@@ -332,122 +251,11 @@ char *Srdline( fd )
 }
 
 
-/*
- * Get a character from a file
- */
-int Sgetc( fd )
-	int fd ;
-{
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
-	register __sio_id_t *idp = IDP( dp ) ;
-
-#ifdef EVENTS
-	EVENT( fd, EV_SGETC ) ;
-#endif
-
-	IO_SETUP( fd, dp, __SIO_INPUT_STREAM, SIO_ERR ) ;
-	ASSERT( idp->start <= idp->nextb && idp->nextb <= idp->end ) ;
-	if ( idp->nextb >= idp->end )
-	{
-		register int b_read = __sio_readf( idp, fd ) ;
-
-		if ( b_read == 0 )
-			return( SIO_EOF ) ;
-		else if ( b_read == SIO_ERR )
-			return( SIO_ERR ) ;
-	}
-	return( (int) *idp->nextb++ ) ;
-}
-
-
-char *Sfetch( fd, lenp )
-	int fd ;
-	long *lenp ;
-{
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
-	register __sio_id_t *idp = IDP( dp ) ;
-	register int b_read ;
-	register char *p ;
-
-#ifdef EVENTS
-	EVENT( fd, EV_SFETCH ) ;
-#endif
-
-	IO_SETUP( fd, dp, __SIO_INPUT_STREAM, NULL ) ;
-	ASSERT( idp->start <= idp->nextb && idp->nextb <= idp->end ) ;
-	if ( idp->nextb >= idp->end )
-	{
-		b_read = __sio_readf( idp, fd ) ;
-		if ( b_read == SIO_ERR )
-			return( NULL ) ;
-		if ( b_read == 0 )
-		{
-			errno = 0 ;
-			return( NULL ) ;
-		}
-	}
-	*lenp = idp->end - idp->nextb ;
-	p = idp->nextb ;
-	idp->nextb = idp->end ;
-	return( p ) ;
-}
-
-
 
 /*
  * SIO CONTROL FUNCTIONS
  */
 
-/*
- * Undo the last Srdline or Sgetc
- */
-int Sundo( fd, type )
-	int fd ;
-	int type ;
-{
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ; 
-	register __sio_id_t *idp = IDP( dp ) ;
-	int retval = 0 ;
-
-#ifdef EVENTS
-	EVENT( fd, EV_SUNDO ) ;
-#endif
-
-	CONTROL_SETUP( dp, __SIO_INPUT_STREAM, SIO_ERR ) ;
-
-	/*
-	 * Undo works only for fd's used for input
-	 */
-	if ( dp->stream_type != __SIO_INPUT_STREAM )
-		return( SIO_ERR ) ;
-
-	/*
-	 * Check if the operation makes sense; if so, do it, otherwise ignore it
-	 */
-	switch ( type )
-	{
-		case SIO_UNDO_LINE:
-			if ( idp->nextb - idp->line_length > idp->start )
-			{
-				*--idp->nextb = '\n' ;
-				idp->nextb -= idp->line_length ;
-			}
-			ASSERT( idp->start <= idp->nextb && idp->nextb <= idp->end ) ;
-			break ;
-		
-		case SIO_UNDO_CHAR:
-			if ( idp->nextb > idp->start )
-				idp->nextb-- ;
-			ASSERT( idp->start <= idp->nextb && idp->nextb <= idp->end ) ;
-			break ;
-		
-		default:
-			retval = SIO_ERR ;
-			break ;
-	}
-	return( retval ) ;
-}
-
 
 /*
  * Flush the buffer associated with the given file descriptor
@@ -460,20 +268,15 @@ int Sundo( fd, type )
  *							or the descriptor is not initialized or it is not 
  *								an output descriptor
  */
-int Sflush( fd )
-	int fd ;
+int Sflush( int fd )
 {
-   register __sio_descriptor_t *dp ;
+   __sio_descriptor_t *dp ;
    int b_in_buffer ;
 
-#ifdef EVENTS
-	EVENT( fd, EV_SFLUSH ) ;
-#endif
-
    if ( fd == SIO_FLUSH_ALL )
    {
       for ( fd = 0, dp = __sio_descriptors ;
-				fd < N_SIO_DESCRIPTORS ;
+				fd < __sio_n_descriptors ;
 				dp++, fd++ )
          if ( DESCRIPTOR_INITIALIZED( dp ) &&
 							dp->stream_type == __SIO_OUTPUT_STREAM )
@@ -482,9 +285,17 @@ int Sflush( fd )
    }
    else
    {
+      if ( fd >= __sio_n_descriptors ) {
+         errno = EBADF ;
+         return( SIO_ERR ) ;
+      }
+
       dp = &__sio_descriptors[ fd ] ;
 
-		CONTROL_SETUP( dp, __SIO_OUTPUT_STREAM, SIO_ERR ) ;
+      if( (! DESCRIPTOR_INITIALIZED( dp )) || 
+            (dp->stream_type != __SIO_OUTPUT_STREAM) )
+         return( SIO_ERR );
+
       b_in_buffer = ODP( dp )->nextb - ODP( dp )->start ;
       if ( __sio_writef( ODP( dp ), fd ) != b_in_buffer )
          return( SIO_ERR ) ;
@@ -509,13 +320,8 @@ int Sflush( fd )
  *
  * There is no need to invoke SETUP; Sdone will do it.
  */
-int Sclose( fd )
-	int fd ;
+int Sclose( int fd )
 {
-#ifdef EVENTS
-	EVENT( fd, EV_SCLOSE ) ;
-#endif
-
 	if ( __SIO_FD_INITIALIZED( fd ) )
 		if ( Sdone( fd ) == SIO_ERR )
 			return( SIO_ERR ) ;
@@ -523,132 +329,13 @@ int Sclose( fd )
 }
 
 
-
-/*
- * Tie the file descriptor in_fd to the file descriptor out_fd
- * This means that whenever a read(2) is done on in_fd, it is
- * preceded by a write(2) on out_fd.
- * Why this is nice to have:
- * 	1) When used in filters it maximizes concurrency
- *		2) When the program prompts the user for something it forces
- *			the prompt string to be displayed even if it does not
- *			end with a '\n' (which would cause a flush).
- * In this implementation, out_fd cannot be a regular file.
- * This is done to avoid non-block-size write's.
- * The file descriptors are initialized if that has not been done
- * already. If any of them is initialized, it must be for the appropriate
- * stream type (input or output).
- *
- * NOTE: the code handles correctly the case when in_fd == out_fd
- */
-int Stie( in_fd, out_fd )
-	int in_fd, out_fd ;
-{
-	struct stat st ;
-	register __sio_descriptor_t *dp ;
-	int was_initialized ;
-	boolean_e failed = NO ;
-
-#ifdef EVENTS
-	EVENT( in_fd, EV_STIE ) ;
-#endif
-
-	/*
-	 * Check if the out_fd is open
-	 */
-	if ( fstat( out_fd, &st ) == -1 )
-		return( SIO_ERR ) ;
-
-	/*
-	 * If the out_fd refers to a regular file, the request is silently ignored
-	 */
-	if ( ( st.st_mode & S_IFMT ) == S_IFREG )
-		return( 0 ) ;
-	
-	dp = &__sio_descriptors[ in_fd ] ;
-	was_initialized = dp->initialized ;		/* remember if it was initialized */
-	IO_SETUP( in_fd, dp, __SIO_INPUT_STREAM, SIO_ERR ) ;
-
-	/*
-	 * Perform manual initialization of out_fd to avoid leaving in_fd
-	 * initialized if the initialization of out_fd fails.
-	 * If out_fd is initialized, check if it is used for output.
-	 * If it is not initialized, initialize it for output.
-	 */
-	dp = &__sio_descriptors[ out_fd ] ;
-	if ( DESCRIPTOR_INITIALIZED( dp ) )
-	{
-		if ( dp->stream_type != __SIO_OUTPUT_STREAM )
-		{
-			failed = YES ;
-			errno = EBADF ;
-		}
-	}
-	else
-		if ( __sio_init( dp, out_fd, __SIO_OUTPUT_STREAM ) == SIO_ERR )
-			failed = YES ;
-
-	if ( failed == NO )
-	{
-		__SIO_ID( in_fd ).tied_fd = out_fd ;
-		return( 0 ) ;
-	}
-	else
-	{
-		/*
-		 * We failed. If we did any initialization, undo it
-		 */
-		if ( ! was_initialized )
-		{
-			int save_errno = errno ;
-
-			(void) Sdone( in_fd ) ;
-			errno = save_errno ;
-		}
-		return( SIO_ERR ) ;
-	}
-}
-
-
-/*
- * Untie a file descriptor
- */
-int Suntie( fd )
-	int fd ;
-{
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
-
-#ifdef EVENTS
-	EVENT( fd, EV_SUNTIE ) ;
-#endif
-
-	CONTROL_SETUP( dp, __SIO_INPUT_STREAM, SIO_ERR ) ;
-	
-	if ( IDP( dp )->tied_fd != SIO_NO_TIED_FD )
-	{
-		IDP( dp )->tied_fd = SIO_NO_TIED_FD ;
-		return( 0 ) ;
-	}
-	else
-	{
-		errno = EBADF ;
-		return( SIO_ERR ) ;
-	}
-}
-
-
 /*
  * Changes the type of buffering on the specified descriptor.
  * As a side-effect, it initializes the descriptor as an output stream.
  */
-int Sbuftype( fd, type )
-	int fd, type ;
+int Sbuftype( int fd, int type )
 {
-	register __sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
-
-#ifdef EVENTS
-	EVENT( fd, EV_SBUFTYPE ) ;
-#endif
+	__sio_descriptor_t *dp = &__sio_descriptors[ fd ] ;
 
 	/*
 	 * Check for a valid type
@@ -659,7 +346,8 @@ int Sbuftype( fd, type )
 		return( SIO_ERR ) ;
 	}
 
-	IO_SETUP( fd, dp, __SIO_OUTPUT_STREAM, SIO_ERR ) ;
+	if( sio_setup( fd, dp, __SIO_OUTPUT_STREAM ) == SIO_ERR )
+		return( SIO_ERR );
 	ODP( dp )->buftype = type ;
 	return( 0 ) ;
 }
@@ -667,13 +355,10 @@ int Sbuftype( fd, type )
 
 #ifndef sio_memscan
 
-PRIVATE char *sio_memscan( from, how_many, ch )
-   char *from ;
-   int how_many ;
-   register char ch ;
+static char *sio_memscan( const char *from, int how_many, char ch )
 {
-   register char *p ;
-   register char *last = from + how_many ;
+   char *p ;
+   char *last = from + how_many ;
 
    for ( p = from ; p < last ; p++ )
       if ( *p == ch )
@@ -686,9 +371,7 @@ PRIVATE char *sio_memscan( from, how_many, ch )
 
 #ifdef NEED_MEMCOPY
 
-void __sio_memcopy( from, to, nbytes )
-   register char *from, *to ;
-   register int nbytes ;
+void __sio_memcopy( const char *from, char *to, int nbytes )
 {
    while ( nbytes-- )
       *to++ = *from++ ;
@@ -696,4 +379,54 @@ void __sio_memcopy( from, to, nbytes )
 
 #endif /* NEED_MEMCOPY */
 
+int sio_setup(int fd, __sio_descriptor_t *dp, int type)
+{
+   if ( fd >= __sio_n_descriptors ) {
+      errno = EBADF;
+      return(SIO_ERR);
+   }
+
+   if( DESCRIPTOR_INITIALIZED( dp ) ) {
+      if( dp->stream_type != type ) {
+         errno = EBADF;
+         return( SIO_ERR );
+      }
+   } else if( __sio_init(dp, fd, type) == SIO_ERR )
+      return( SIO_ERR );
+
+   return( 0 );
+}
+
+/*
+ * The Sputchar function depends on the fact that the fields
+ *                              nextb, buf_end, end
+ * are 0 if a stream descriptor is not being used or has not yet been
+ * initialized.
+ * This is true initially because of the static allocation of the
+ * descriptor array, and Sdone must make sure that it is true
+ * after I/O on a descriptor is over.
+ */
+int Sputchar( int fd, char c )
+{
+   int ret = 0;
+
+   if( __SIO_OD( fd ).nextb < __SIO_OD( fd ).buf_end ) {
+      if( __SIO_OD(fd).buftype == SIO_FULLBUF ) {
+         ret = *(__SIO_OD(fd).nextb)++ = (unsigned char) (c);
+      } else if ( __SIO_OD(fd).buftype == SIO_LINEBUF ) {
+         if( (*(__SIO_OD(fd).nextb) = (unsigned char)(c)) != '\n' ) {
+            ret = *(__SIO_OD(fd).nextb)++;
+         } else {
+            ret = Sputc( fd, *(__SIO_OD(fd).nextb) );
+         }
+      } else {
+         ret = Sputc( fd, c );
+      }
+   } else {
+      ret = Sputc( fd, c );
+   }
+
+   return ret;
+}
+
 
