@@ -58,7 +58,6 @@
 #if !defined( SIO_MMAP ) || !defined( SIO_MUNMAP ) || !defined( SIO_MNEED )
 #include <sys/types.h>
 #include <sys/mman.h>
-#undef PRIVATE
 #endif
 
 #ifndef SIO_MMAP
@@ -88,8 +87,12 @@
  */
 #ifndef N_SIO_DESCRIPTORS
 #include <sys/param.h>
+#ifdef OPEN_MAX
+#define N_SIO_DESCRIPTORS		OPEN_MAX
+#else
 #define N_SIO_DESCRIPTORS		NOFILE
 #endif
+#endif
 
 
 
