@@ -4,24 +4,28 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: strparse.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 #include "config.h"
-#ifdef STR_VERSION
-static char version[] = STR_VERSION ;
-#endif
-
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 
 #include "str.h"
 #include "strparse.h"
-#include "sio.h"
-
-int str_errno ;
 
 
-char *new_string( register char *s )
+static int str_errno ;
+
+#define HANDLE_ERROR( flags, retval, errp, errval, msg )    \
+	if ( flags & STR_RETURN_ERROR )                 \
+	{                                               \
+               *errp = errval ;                         \
+               return( retval ) ;                       \
+        }                                               \
+        else                                            \
+               terminate( msg );
+
+
+char *new_string( const char *s )
 {
    if ( s )
       return strdup( s ) ;
@@ -30,6 +34,14 @@ char *new_string( register char *s )
 }
 
 
+static void terminate(const char *msg)
+{
+      (void) write( 2, msg, strlen( msg ) ) ;
+      (void) abort() ;
+      _exit( 1 ) ;	/* NOT REACHED */
+}
+
+
 str_h str_parse( register char *str, char *separ, int flags, int *errnop )
 {
 	register struct str_handle *hp ;
@@ -37,13 +49,13 @@ str_h str_parse( register char *str, char *separ, int flags, int *errnop )
 
 	if ( separ == NULL ) {
 		HANDLE_ERROR( flags, NULL, errp, STR_ENULLSEPAR,
-								"STR str_parse: NULL separator\n" ) ;
+			"STR str_parse: NULL separator\n" ) ;
 	}
 
 	hp = (struct str_handle *) malloc( sizeof( struct str_handle ) ) ;
 	if ( hp == NULL ) {
 		HANDLE_ERROR( flags, NULL, errp, STR_ENOMEM,
-								"STR str_parse: malloc failed\n" ) ;
+			"STR str_parse: malloc failed\n" ) ;
 	}
 
 	hp->string = str ;
@@ -56,7 +68,7 @@ str_h str_parse( register char *str, char *separ, int flags, int *errnop )
 			*errp = STR_ENOMEM ;
 			return( NULL ) ;
 		} else {
-			TERMINATE( "STR str_parse: malloc failed\n" ) ;
+			terminate( "STR str_parse: malloc failed\n" ) ;
 		}
 	}
 	
@@ -85,7 +97,7 @@ int str_setstr( str_h handle, char *newstr )
 	
 	if ( newstr == NULL ) {
 		HANDLE_ERROR( hp->flags, STR_ERR, hp->errnop, STR_ENULLSTRING,
-								"STR str_setstr: NULL string\n" ) ;
+			"STR str_setstr: NULL string\n" ) ;
 	}
 	
 	hp->string = newstr ;
@@ -95,42 +107,6 @@ int str_setstr( str_h handle, char *newstr )
 }
 
 
-
-/*
- * Change the separator
- */
-int str_separator( str_h handle, char *separator )
-{
-	register struct str_handle *hp = (struct str_handle *) handle ;
-	char *new_separator ;
-
-	if ( separator == NULL ) {
-		HANDLE_ERROR( hp->flags, STR_ERR, hp->errnop, STR_ENULLSEPAR,
-								"STR str_separator: NULL separator\n" ) ;
-	}
-	new_separator = new_string( separator ) ;
-	if ( new_separator == NULL ) {
-		HANDLE_ERROR( hp->flags, STR_ERR, hp->errnop, STR_ENOMEM,
-			"STR str_separator: malloc failed\n" ) ;
-	}
-
-	free( hp->separator ) ;
-	hp->separator = new_separator ;
-	return( STR_OK ) ;
-}
-
-
-char *str_nextpos( str_h handle )
-{
-	struct str_handle *hp = (struct str_handle *) handle ;
-
-	if ( hp->no_more || *hp->pos == '\0' )
-		return( NULL ) ;
-	else
-		return( hp->pos ) ;
-}
-
-
 char *str_component( str_h handle )
 {
 	register char			*start ;
