@@ -4,8 +4,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: strutil.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
-
 
 #include <ctype.h>
 #include "config.h"
@@ -116,7 +114,7 @@ char *str_casefind( register char *str, char *sstr )
  * look for an instance of s2 in s1
  * Returns a pointer to the beginning of s2 in s1
  */
-char *str_find( char *s1, char *s2 )
+char *str_find( const char *s1, const char *s2 )
 {
    int i ;
    int l1 = strlen( s1 ) ;
@@ -129,7 +127,7 @@ char *str_find( char *s1, char *s2 )
 }
 
 
-char *str_casefind( char *s1, char *s2 )
+char *str_casefind( const char *s1, const char *s2 )
 {
    int i ;
    int l1 = strlen( s1 ) ;
@@ -147,37 +145,7 @@ char *str_casefind( char *s1, char *s2 )
 /*
  * Fill string s with character c
  */
-void str_fill( s, c )
-	register char *s ;
-	register char c ;
+void str_fill( register char *s, register char c )
 {
 	while ( *s ) *s++ = c ;
 }
-
-
-char *str_lower( s )
-	char *s ;
-{
-	register char *p ;
-	register int offset = 'a' - 'A' ;
-
-	for ( p = s ; *p ; p++ )
-		if ( isascii( *p ) && isupper( *p ) )
-			*p += offset ;
-	return( s ) ;
-}
-
-
-char *str_upper( s )
-	char *s ;
-{
-	register char *p ;
-	register int offset = 'a' - 'A' ;
-
-	for ( p = s ; *p ; p++ )
-		if ( isascii( *p ) && islower( *p ) )
-			*p -= offset ;
-	return( s ) ;
-}
-
-
