@@ -12,6 +12,8 @@
 #ifndef __FILELOG_H
 #define __FILELOG_H
 
+#include "impl.h"
+
 /*
  * The file can be either open or closed.
  * When the file is closed, the state is always FL_CLOSED.
@@ -26,7 +28,6 @@ struct filelog
 {
 	int					fl_fd ;
 	filelog_state_e	fl_state ;
-	bool_int 			fl_close_on_exec ;
 	int 					fl_error ;				/* error code when in FL_ERROR		*/
 	bool_int				fl_size_control ;		/* enabled or not							*/
 	bool_int				fl_issued_warning ;	/* when the soft limit was exceeded	*/
@@ -35,9 +36,9 @@ struct filelog
 	unsigned				fl_hard_limit ;
 } ;
 
-#define FILELOG_ENABLE_SIZE_CONTROL( flp )		(flp)->fl_size_control = TRUE
-#define FILELOG_DISABLE_SIZE_CONTROL( flp )		(flp)->fl_size_control = FALSE
-#define FILELOG_SIZE_CONTROL( flp )    			( (flp)->fl_size_control )
+#define FILELOG_ENABLE_SIZE_CONTROL( flp )	(flp)->fl_size_control = TRUE
+#define FILELOG_DISABLE_SIZE_CONTROL( flp )	(flp)->fl_size_control = FALSE
+#define FILELOG_SIZE_CONTROL( flp )    		( (flp)->fl_size_control )
 
 #define FILELOG( xp )			((struct filelog *)xp->xl_data)
 
