@@ -4,7 +4,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: slog.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #ifndef NO_SYSLOG
@@ -20,14 +19,14 @@ static char RCSid[] = "$Id: slog.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 #define MSGBUFSIZE         2048
 
 
-PRIVATE int syslog_init(xlog_s *, va_list) ;
-PRIVATE void syslog_fini(xlog_s *) ;
-PRIVATE int syslog_control(xlog_s *, xlog_cmd_e, va_list ) ;
-PRIVATE int syslog_write(xlog_s *, char buf[], int , int , va_list ) ;
-PRIVATE int syslog_parms(xlog_e, va_list) ;
+static int syslog_init(xlog_s *, va_list) ;
+static void syslog_fini(xlog_s *) ;
+static int syslog_control(xlog_s *, xlog_cmd_e, va_list ) ;
+static int syslog_write(xlog_s *, const char buf[], int , int , va_list ) ;
+static int syslog_parms(xlog_e, va_list) ;
 
 static struct syslog_parms parms =
-   {
+{
       0,
 #ifndef NO_SYSLOG
       LOG_PID + LOG_NOWAIT,
@@ -37,8 +36,7 @@ static struct syslog_parms parms =
       0,
 #endif
       "XLOG",
-      FALSE
-   } ;
+} ;
 
 
 struct xlog_ops __xlog_syslog_ops =
@@ -58,15 +56,15 @@ struct xlog_ops __xlog_syslog_ops =
  * them so that we don't have any unresolved references; and we define 
  * them without any arguments.
  */
-PRIVATE void syslog()
+static void syslog()
 {
 }
 
-PRIVATE void openlog()
+static void openlog()
 {
 }
 
-PRIVATE void closelog()
+static void closelog()
 {
 }
 
@@ -77,7 +75,7 @@ PRIVATE void closelog()
  * Expected arguments:
  *      facility, level
  */
-PRIVATE int syslog_init( xlog_s *xp, va_list ap )
+static int syslog_init( xlog_s *xp, va_list ap )
 {
    struct syslog_parms   *slp = &parms ;
    struct syslog                  *sp ;
@@ -94,16 +92,16 @@ PRIVATE int syslog_init( xlog_s *xp, va_list ap )
 }
 
 
-PRIVATE void syslog_fini( xlog_s *xp )
+static void syslog_fini( xlog_s *xp )
 {
    if ( --parms.slp_n_xlogs == 0 )
       closelog() ;
-   FREE( SYSLOG( xp ) ) ;
+   free( SYSLOG( xp ) ) ;
    xp->xl_data = NULL ;
 }
 
 
-PRIVATE int syslog_control( xlog_s *xp, xlog_cmd_e cmd, va_list ap )
+static int syslog_control( xlog_s *xp, xlog_cmd_e cmd, va_list ap )
 {
    switch ( cmd )
    {
@@ -123,12 +121,22 @@ PRIVATE int syslog_control( xlog_s *xp, xlog_cmd_e cmd, va_list ap )
          if ( parms.slp_n_xlogs )
             openlog( parms.slp_ident, parms.slp_logopts, parms.slp_facility ) ;
          break ;
+
+      /* These fall through ? */
+      case XLOG_LINK:
+      case XLOG_CALLBACK:
+      case XLOG_GETFLAG:
+      case XLOG_SETFLAG: 
+      case XLOG_SIZECHECK:
+      case XLOG_GETFD:
+      case XLOG_LIMITS:
+         break ;
    }
    return( XLOG_ENOERROR ) ;
 }
 
 
-PRIVATE int syslog_write( xlog_s *xp, char buf[], int len, int flags, va_list ap )
+static int syslog_write( xlog_s *xp, const char buf[], int len, int flags, va_list ap )
 {
    int   level ;
    int   syslog_arg ;
@@ -136,7 +144,7 @@ PRIVATE int syslog_write( xlog_s *xp, char buf[], int len, int flags, va_list ap
    int   prefix_size = sizeof( prefix ) ;
    int   prefix_len = 0 ;
    int   cc ;
-   char   *percent_m_pos ;
+   int   percent_m_pos ;
    int   action_flags = ( flags | xp->xl_flags ) ;
 
    if ( flags & XLOG_SET_LEVEL )
@@ -154,12 +162,10 @@ PRIVATE int syslog_write( xlog_s *xp, char buf[], int len, int flags, va_list ap
    }
 
    if ( ( action_flags & XLOG_NO_ERRNO ) ||
-                  ( percent_m_pos = __xlog_add_errno( buf, len ) ) == NULL )
+                  ( percent_m_pos = __xlog_add_errno( buf, len ) ) == -1 )
       syslog( syslog_arg, "%.*s%.*s", prefix_len, prefix, len, buf ) ;
    else
    {
-      int cc_before_errno = percent_m_pos - buf ;
-      int cc_after_errno = len - cc_before_errno - 2 ;
       char *ep ;
       char errno_buf[ 100 ] ;
       unsigned size = sizeof( errno_buf ) ;
@@ -167,15 +173,15 @@ PRIVATE int syslog_write( xlog_s *xp, char buf[], int len, int flags, va_list ap
       ep = __xlog_explain_errno( errno_buf, &size ) ;
       syslog( syslog_arg, "%.*s%.*s%.*s%.*s",
             prefix_len, prefix,
-               cc_before_errno, buf,
+               percent_m_pos, buf,
                   (int)size, ep,
-                     cc_after_errno, &percent_m_pos[ 2 ] ) ;
+                     len - percent_m_pos - 2, buf + percent_m_pos + 2 ) ;
    }
    return( XLOG_ENOERROR ) ;
 }
 
 
-PRIVATE int syslog_parms( xlog_e type, va_list ap )
+static int syslog_parms( xlog_e type, va_list ap )
 {
    char *id = NULL;
 
