@@ -20,11 +20,11 @@ struct syslog
 
 struct syslog_parms
 {
-   int 		slp_n_xlogs ;              /* # of xlogs using syslog */
-   int 		slp_logopts ;              /* used at openlog */
+   int 		slp_n_xlogs ;           /* # of xlogs using syslog */
+   int 		slp_logopts ;           /* used at openlog */
    int 		slp_facility ;
-   char		*slp_ident ;					/* used at openlog */
-   bool_int slp_ident_is_malloced ;
+   char		*slp_ident ;		/* used at openlog */
+   /* bool_int slp_ident_is_malloced ; */
 } ;
 
 #define SYSLOG( xp )         ((struct syslog *)xp->xl_data)
