@@ -5,7 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: builtins.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
@@ -17,36 +16,41 @@ static char RCSid[] = "$Id: builtins.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp
 #include <syslog.h>
 #include <stdlib.h>
 #include <unistd.h>
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
 
 #include "str.h"
-
-#include "xconfig.h"
-#include "state.h"
-#include "defs.h"
+#include "pset.h"
+#include "sio.h"
 #include "builtins.h"
-#include "sconf.h"
+#include "msg.h"
+#include "connection.h"
 #include "server.h"
 #include "service.h"
-#include "access.h"
-#include "addr.h"
-#include "log.h"
+#include "sconf.h"
+#include "main.h"
+#include "util.h"
+#include "xconfig.h"
+#include "state.h"
+#include "libportable.h"
 
 #define BUFFER_SIZE               1024
 
-PRIVATE void stream_echo(struct server *) ;
-PRIVATE void dgram_echo(struct server *) ;
-PRIVATE void stream_discard(struct server *) ;
-PRIVATE void dgram_discard(struct server *) ;
-PRIVATE void stream_time(struct server *) ;
-PRIVATE void dgram_time(struct server *) ;
-PRIVATE void stream_daytime(struct server *) ;
-PRIVATE void dgram_daytime(struct server *) ;
-PRIVATE void stream_chargen(struct server *) ;
-PRIVATE void dgram_chargen(struct server *) ;
-PRIVATE void stream_servers(struct server *) ;
-PRIVATE void stream_services(struct server *) ;
-PRIVATE void xadmin_handler(struct server *) ;
-PRIVATE int bad_port_check(struct sockaddr *);
+static void stream_echo(const struct server *) ;
+static void dgram_echo(const struct server *) ;
+static void stream_discard(const struct server *) ;
+static void dgram_discard(const struct server *) ;
+static void stream_time(const struct server *) ;
+static void dgram_time(const struct server *) ;
+static void stream_daytime(const struct server *) ;
+static void dgram_daytime(const struct server *) ;
+static void stream_chargen(const struct server *) ;
+static void dgram_chargen(const struct server *) ;
+static void stream_servers(const struct server *) ;
+static void stream_services(const struct server *) ;
+static void xadmin_handler(const struct server *) ;
+static int bad_port_check(const union xsockaddr *, const char *);
 
 /*
  * SG - This is the call sequence to get to a built-in service
@@ -61,7 +65,7 @@ PRIVATE int bad_port_check(struct sockaddr *);
  *        sc_builtin -- index into builtin_services   builtins.c
  */
 
-static struct builtin_service builtin_services[] =
+static const struct builtin_service builtin_services[] =
    {
       { "echo",      SOCK_STREAM,   { stream_echo,     FORK    } },
       { "echo",      SOCK_DGRAM,    { dgram_echo,      NO_FORK } },
@@ -82,10 +86,10 @@ static struct builtin_service builtin_services[] =
    } ;
 
 
-builtin_s *builtin_find( char *service_name, int type )
+const builtin_s *builtin_find( const char *service_name, int type )
 {
-   builtin_s   *bsp ;
-   char        *func = "builtin_find" ;
+   const builtin_s   *bsp ;
+   const char        *func = "builtin_find" ;
    
    if ( (bsp = builtin_lookup( builtin_services, service_name, type )) )
       return( bsp ) ;
@@ -95,11 +99,11 @@ builtin_s *builtin_find( char *service_name, int type )
 }
 
 
-builtin_s *builtin_lookup( struct builtin_service services[], 
-                           char *service_name, 
+const builtin_s *builtin_lookup( const struct builtin_service services[], 
+                           const char *service_name, 
                            int type )
 {
-   struct builtin_service *bsp ;
+   const struct builtin_service *bsp ;
    
    for ( bsp = services ; bsp->bs_name != NULL ; bsp++ )
       if ( EQ( bsp->bs_name, service_name ) && bsp->bs_socket_type == type )
@@ -114,19 +118,21 @@ builtin_s *builtin_lookup( struct builtin_service services[],
  */
 
 
-PRIVATE void stream_echo( struct server *serp )
+static void stream_echo( const struct server *serp )
 {
    char   buf[ BUFFER_SIZE ] ;
    int    cc ;
    int    descriptor = SERVER_FD( serp ) ;
-   struct service *svc = SERVER_SERVICE( serp ) ;;
+   const struct service *svc = SERVER_SERVICE( serp ) ;;
 
-   if( SVC_WAITS( svc ) )
+   if( SVC_WAITS( svc ) ) {
       descriptor = accept(descriptor, NULL, NULL);
+      if ( descriptor == -1 ) return;
+   }
 
    for ( ;; )
    {
-      cc = read( descriptor, buf, BUFFER_SIZE ) ;
+      cc = read( descriptor, buf, sizeof( buf ) ) ;
       if ( cc == 0 )
          break ;
       if ( cc == -1 ) {
@@ -143,69 +149,57 @@ PRIVATE void stream_echo( struct server *serp )
       close(descriptor);
 }
 
-/* For internal UDP services, make sure we don't respond to
- * ports our ports on other servers.  This can cause looping.
+/* For internal UDP services, make sure we don't respond to our ports
+ * on other servers and to low ports of other services (such as DNS).
+ * This can cause looping.
  */
-PRIVATE int bad_port_check(struct sockaddr *sa)
+static int bad_port_check( const union xsockaddr *sa, const char *func )
 {
-   uint16_t bad_ports[] = { 7, 9, 13, 19, 37, 0 };
    uint16_t port = 0;
-   int i;
 
-   if( ((struct sockaddr_in *)sa)->sin_family == AF_INET )
-      port = ntohs(((struct sockaddr_in *)sa)->sin_port);
-#ifdef INET6
-   else if( ((struct sockaddr_in6 *)sa)->sin6_family == AF_INET6 )
-      port = ntohs(((struct sockaddr_in6 *)sa)->sin6_port);
-#endif
-   else
-      return(-1);
+   port = ntohs( xaddrport( sa ) );
 
-   for( i=0; bad_ports[i] != 0; i++ ) {
-      if( port == bad_ports[i] ) {
-         return (-1);
-      }
+   if ( port < 1024 ) {
+      msg(LOG_WARNING, func,
+         "Possible Denial of Service attack from %s %d", xaddrname(sa), port);
+      return (-1);
    }
 
    return (0);
 }
 
-PRIVATE void dgram_echo( struct server *serp )
+static void dgram_echo( const struct server *serp )
 {
    char            buf[ DATAGRAM_SIZE ] ;
-#ifdef INET6
-   struct sockaddr_in6 lsin ;
-#else
-   struct sockaddr_in lsin ;
-#endif
+   union xsockaddr lsin;
    int             cc ;
-   int             sin_len = sizeof( lsin ) ;
+   int             sin_len = 0;
    int             descriptor = SERVER_FD( serp ) ;
-   char            *func = "dgram_echo";
+   const char     *func = "dgram_echo";
+
+   if( SC_IPV4( SVC_CONF( SERVER_SERVICE( serp ) ) ) )
+      sin_len = sizeof( struct sockaddr_in );
+   if( SC_IPV6( SVC_CONF( SERVER_SERVICE( serp ) ) ) )
+      sin_len = sizeof( struct sockaddr_in6 );
 
    cc = recvfrom( descriptor, buf, sizeof( buf ), 0, SA( &lsin ), &sin_len ) ;
    if ( cc != -1 ) {
-      if( bad_port_check(SA(&lsin)) != 0 ) {
-         if( ((struct sockaddr_in *)&lsin)->sin_family == AF_INET)
-            msg(LOG_WARNING, func, 
-            "Possible Denial of Service attack from %s %d",
-            xntoa(SA(&lsin)), 
-            ntohs(((struct sockaddr_in *)&lsin)->sin_port));
-         return;
-      }
+      if( bad_port_check(&lsin, func) != 0 ) return;
       (void) sendto( descriptor, buf, cc, 0, SA( &lsin ), sizeof( lsin ) ) ;
    }
 }
 
-PRIVATE void stream_discard( struct server *serp )
+static void stream_discard( const struct server *serp )
 {
    char  buf[ BUFFER_SIZE ] ;
    int   cc ;
    int    descriptor = SERVER_FD( serp ) ;
-   struct service *svc = SERVER_SERVICE( serp ) ;;
+   const struct service *svc = SERVER_SERVICE( serp ) ;;
 
-   if( SVC_WAITS( svc ) )
+   if( SVC_WAITS( svc ) ) {
       descriptor = accept(descriptor, NULL, NULL);
+      if ( descriptor == -1 ) return;
+   }
    for ( ;; )
    {
       cc = read( descriptor, buf, sizeof( buf ) ) ;
@@ -217,7 +211,7 @@ PRIVATE void stream_discard( struct server *serp )
 }
 
 
-PRIVATE void dgram_discard( struct server *serp )
+static void dgram_discard( const struct server *serp )
 {
    char buf[ 1 ] ;
 
@@ -233,9 +227,9 @@ PRIVATE void dgram_discard( struct server *serp )
  * buflen is a value-result parameter. It indicates the size of
  * buf and on exit it has the length of the string placed in buf.
  */
-PRIVATE void daytime_protocol( char *buf, int *buflen )
+static void daytime_protocol( char *buf, int *buflen )
 {
-   static char *month_name[] =
+   static const char *month_name[] =
       {
          "JAN", "FEB", "MAR", "APR", "MAY", "JUN",
          "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"
@@ -267,47 +261,43 @@ PRIVATE void daytime_protocol( char *buf, int *buflen )
 }
 
 
-PRIVATE void stream_daytime( struct server *serp )
+static void stream_daytime( const struct server *serp )
 {
    char  time_buf[ BUFFER_SIZE ] ;
    int   buflen = sizeof( time_buf ) ;
    int    descriptor = SERVER_FD( serp ) ;
-   struct service *svc = SERVER_SERVICE( serp ) ;;
+   const struct service *svc = SERVER_SERVICE( serp ) ;;
 
-   if( SVC_WAITS( svc ) )
+   if( SVC_WAITS( svc ) ) {
       descriptor = accept(descriptor, NULL, NULL);
+      if ( descriptor == -1 ) return;
+   }
    daytime_protocol( time_buf, &buflen ) ;
-   (void) write_buf( SERVER_FD( serp ), time_buf, buflen ) ;
+   (void) write_buf( descriptor, time_buf, buflen ) ;
    if( SVC_WAITS( svc ) )
       close(descriptor);
 }
 
 
-PRIVATE void dgram_daytime( struct server *serp )
+static void dgram_daytime( const struct server *serp )
 {
    char            time_buf[ BUFFER_SIZE ] ;
-#ifdef INET6
-   struct sockaddr_in6 lsin ;
-#else
-   struct sockaddr_in lsin ;
-#endif
-   int             sin_len     = sizeof( lsin ) ;
+   union xsockaddr lsin ;
+   int             sin_len     = 0 ;
    int             buflen      = sizeof( time_buf ) ;
    int             descriptor  = SERVER_FD( serp ) ;
-   char            *func       = "dgram_daytime";
+   const char     *func       = "dgram_daytime";
+
+   if ( SC_IPV4( SVC_CONF( SERVER_SERVICE( serp ) ) ) ) 
+      sin_len = sizeof( struct sockaddr_in );
+   if ( SC_IPV6( SVC_CONF( SERVER_SERVICE( serp ) ) ) ) 
+      sin_len = sizeof( struct sockaddr_in6 );
 
    if ( recvfrom( descriptor, time_buf, sizeof( time_buf ), 0,
             SA( &lsin ), &sin_len ) == -1 )
       return ;
 
-   if( bad_port_check(SA(&lsin)) != 0 ) {
-      if( ((struct sockaddr_in *)&lsin)->sin_family == AF_INET )
-         msg(LOG_WARNING, func, 
-            "Possible Denial of Service attack from %s %d", 
-            xntoa(SA(&lsin)), 
-            ntohs(((struct sockaddr_in *)&lsin)->sin_port));
-      return;
-   }
+   if( bad_port_check(&lsin, func) != 0 ) return;
 
    daytime_protocol( time_buf, &buflen ) ;
    
@@ -318,62 +308,61 @@ PRIVATE void dgram_daytime( struct server *serp )
 #define TIME_OFFSET         2208988800UL
 
 /*
- * We always report the time using network-byte-order
+ * We always report the time as 32 bits in network-byte-order
  */
-PRIVATE void time_protocol( time_t *timep )
+static void time_protocol( unsigned char *timep )
 {
    time_t now ;
+   unsigned long base1900;
 
    (void) time( &now ) ;
-   *timep = now + TIME_OFFSET ;
-   *timep = htonl( *timep ) ;
+   base1900 = (unsigned long)now + TIME_OFFSET ;
+   timep[0] = base1900 >> 24;
+   timep[1] = base1900 >> 16;
+   timep[2] = base1900 >> 8;
+   timep[3] = base1900;
 }
 
 
-PRIVATE void stream_time( struct server *serp )
+static void stream_time( const struct server *serp )
 {
-   time_t now ;
+   unsigned char time_buf[4];
    int descriptor = SERVER_FD( serp );
-   struct service *svc = SERVER_SERVICE( serp );
+   const struct service *svc = SERVER_SERVICE( serp );
 
-   if( SVC_WAITS( svc ) )
+   if( SVC_WAITS( svc ) ) {
       descriptor = accept(descriptor, NULL, NULL);
+      if ( descriptor == -1 ) return;
+   }
 
-   time_protocol( &now ) ;
-   (void) write_buf( descriptor, (char *) &now, sizeof( now ) ) ;
+   time_protocol( time_buf ) ;
+   (void) write_buf( descriptor, (char *) time_buf, 4 ) ;
 
    if( SVC_WAITS( svc ) )
       close(descriptor);
 }
 
 
-PRIVATE void dgram_time( struct server *serp )
+static void dgram_time( const struct server *serp )
 {
    char     buf[ 1 ] ;
-   time_t   now ;
-#ifdef INET6
-   struct sockaddr_in6 lsin ;
-#else
-   struct sockaddr_in lsin ;
-#endif
+   unsigned char time_buf[4];
+   union xsockaddr lsin ;
    int             sin_len = sizeof( lsin ) ;
    int             fd      = SERVER_FD( serp ) ;
-   char            *func   = "dgram_daytime";
+   const char     *func    = "dgram_daytime";
+
+   if ( SC_IPV4( SVC_CONF( SERVER_SERVICE( serp ) ) ) ) 
+      sin_len = sizeof( struct sockaddr_in );
+   if ( SC_IPV6( SVC_CONF( SERVER_SERVICE( serp ) ) ) ) 
+      sin_len = sizeof( struct sockaddr_in6 );
 
    if ( recvfrom( fd, buf, sizeof( buf ), 0, SA( &lsin ), &sin_len ) == -1 )
       return ;
+   if( bad_port_check(&lsin, func) != 0 ) return;
 
-   if( bad_port_check(SA(&lsin)) != 0 ) {
-      if( ((struct sockaddr_in *)&lsin)->sin_family == AF_INET )
-         msg(LOG_WARNING, func, 
-            "Possible Denial of Service attack from %s %d", 
-            xntoa(SA(&lsin)), 
-            ntohs(((struct sockaddr_in *)&lsin)->sin_port));
-      return;
-   }
-
-   time_protocol( &now ) ;
-   (void) sendto( fd, (char *) &now, sizeof( now ), 0, SA( &lsin ), sin_len ) ;
+   time_protocol( time_buf ) ;
+   (void) sendto( fd, (char *) time_buf, 4, 0, SA( &lsin ), sin_len ) ;
 }
 
 
@@ -382,7 +371,7 @@ PRIVATE void dgram_time( struct server *serp )
 
 #define RING_BUF_SIZE             ASCII_PRINTABLE_CHARS + LINE_LENGTH
 
-static char ring_buf[ RING_BUF_SIZE ] ;
+static char *ring_buf = NULL ;
 static char *ring ;
 
 
@@ -391,20 +380,26 @@ static char *ring ;
 
 #define min( a, b )          ((a)<(b) ? (a) : (b))
 
-PRIVATE char *generate_line( char *buf, int len )
+static char *generate_line( char *buf, int len )
 {
    int line_len = min( LINE_LENGTH, len-2 ) ;
 
    if ( line_len < 0 )
       return( NULL ) ;
 
+   /* This never gets freed.  That's ok, because the reference to it is
+    * always kept for future reference.
+    */
+   if ( (ring_buf == NULL) && ((ring_buf = malloc(RING_BUF_SIZE)) == NULL) ) 
+      return(NULL);
+
    if ( ring == NULL )
    {
       char ch ;
       char *p ;
 
       for ( p = ring_buf, ch = ASCII_START ;
-            p < &ring_buf[ RING_BUF_SIZE ] ; p++ )
+            p <= &ring_buf[ RING_BUF_SIZE - 1 ] ; p++ )
       {
          *p = ch++ ;
          if ( ch == ASCII_END )
@@ -417,20 +412,22 @@ PRIVATE char *generate_line( char *buf, int len )
    buf[ line_len+1 ] = '\n' ;
 
    ring++ ;
-   if ( &ring_buf[ RING_BUF_SIZE ] - ring < LINE_LENGTH )
+   if ( &ring_buf[ RING_BUF_SIZE - 1 ] - ring + 1 < LINE_LENGTH )
       ring = ring_buf ;
    return( buf ) ;
 }
 
 
-PRIVATE void stream_chargen( struct server *serp )
+static void stream_chargen( const struct server *serp )
 {
    char   line_buf[ LINE_LENGTH+2 ] ;
    int    descriptor = SERVER_FD( serp ) ;
-   struct service *svc = SERVER_SERVICE( serp );
+   const struct service *svc = SERVER_SERVICE( serp );
 
-   if( SVC_WAITS( svc ) )
+   if( SVC_WAITS( svc ) ) {
       descriptor = accept(descriptor, NULL, NULL);
+      if ( descriptor == -1 ) return;
+   }
 
    (void) shutdown( descriptor, 0 ) ;
    for ( ;; )
@@ -445,20 +442,21 @@ PRIVATE void stream_chargen( struct server *serp )
 }
 
 
-PRIVATE void dgram_chargen( struct server *serp )
+static void dgram_chargen( const struct server *serp )
 {
    char            buf[ BUFFER_SIZE ] ;
    char            *p ;
    int             len ;
-#ifdef INET6
-   struct sockaddr_in6 lsin ;
-#else
-   struct sockaddr_in lsin ;
-#endif
+   union xsockaddr lsin ;
    int             sin_len = sizeof( lsin ) ;
    int             fd      = SERVER_FD( serp ) ;
    int             left    = sizeof( buf ) ;
-   char            *func   = "dgram_chargen";
+   const char     *func    = "dgram_chargen";
+
+   if ( SC_IPV4( SVC_CONF( SERVER_SERVICE( serp ) ) ) ) 
+      sin_len = sizeof( struct sockaddr_in );
+   if ( SC_IPV6( SVC_CONF( SERVER_SERVICE( serp ) ) ) ) 
+      sin_len = sizeof( struct sockaddr_in6 );
 
    if ( recvfrom( fd, buf, sizeof( buf ), 0, SA( &lsin ), &sin_len ) == -1 )
       return ;
@@ -467,14 +465,7 @@ PRIVATE void dgram_chargen( struct server *serp )
    bad_variable = 1 ;      /* this will cause a compilation error */
 #endif
 
-   if( bad_port_check(SA(&lsin)) != 0 ) {
-      if( ((struct sockaddr_in *)&lsin)->sin_family == AF_INET )
-         msg(LOG_WARNING, func, 
-            "Possible Denial of Service attack from %s %d", 
-            xntoa(SA(&lsin)), 
-            ntohs(((struct sockaddr_in *)&lsin)->sin_port));
-      return;
-   }
+   if( bad_port_check(&lsin, func) != 0 ) return;
 
    for ( p = buf ; left > 2 ; left -= len, p += len )
    {
@@ -486,14 +477,14 @@ PRIVATE void dgram_chargen( struct server *serp )
 }
 
 
-PRIVATE void stream_servers( struct server *this_serp )
+static void stream_servers( const struct server *this_serp )
 {
    unsigned  u ;
    int       descriptor = SERVER_FD( this_serp ) ;
 
    for ( u = 0 ; u < pset_count( SERVERS( ps ) ) ; u++ )
    {
-      struct server *serp = SERP( pset_pointer( SERVERS( ps ), u ) ) ;
+      const struct server *serp = SERP( pset_pointer( SERVERS( ps ), u ) ) ;
 
       /*
        * We cannot report any useful information about this server because
@@ -507,7 +498,7 @@ PRIVATE void stream_servers( struct server *this_serp )
 }
 
 
-PRIVATE void stream_services( struct server *serp )
+static void stream_services( const struct server *serp )
 {
    unsigned u ;
    int      fd = SERVER_FD( serp ) ;
@@ -516,7 +507,7 @@ PRIVATE void stream_services( struct server *serp )
    {
       int cc ;
       char buf[ BUFFER_SIZE ] ;
-      struct service_config *scp ;
+      const struct service_config *scp ;
       
       scp = SVC_CONF( SP( pset_pointer( SERVICES( ps ), u ) ) ) ;
 
@@ -532,11 +523,12 @@ PRIVATE void stream_services( struct server *serp )
 
 /* Do the redirection of a service */
 /* This function gets called from child.c after we have been forked */
-PRIVATE void xadmin_handler( struct server *serp )
+static void xadmin_handler( const struct server *serp )
 {
    int  descriptor = SERVER_FD( serp );
    char cmd[MAX_CMD]; 
-   int  red = 0, u = 0;
+   int  red = 0;
+   unsigned  u = 0;
    
    while(1)
    {
@@ -614,45 +606,46 @@ PRIVATE void xadmin_handler( struct server *serp )
             for( u = 0 ; u < pset_count( SERVICES( ps ) ) ; u++ )
             {
                struct service *sp=NULL;
-#ifdef INET6
-               struct sockaddr_in6 baddr;
-               struct sockaddr_in6 raddr;
-#else
-               struct sockaddr_in baddr;
-               struct sockaddr_in raddr;
-#endif
+               char bname[NI_MAXHOST];
+               char rname[NI_MAXHOST];
+               int length = 0;
 
-               bzero(&baddr, sizeof(baddr));
-               bzero(&raddr, sizeof(raddr));
+               memset(bname, 0, sizeof(bname));
+               memset(rname, 0, sizeof(rname));
 
                sp = SP( pset_pointer( SERVICES( ps ), u ) );
 
-               if( SVC_CONF(sp)->sc_bind_addr != NULL )
-#ifdef INET6
-               memcpy(&baddr.sin6_addr, SVC_CONF(sp)->sc_bind_addr, sizeof(baddr.sin6_addr));
-#else
-               memcpy(&baddr.sin_addr, SVC_CONF(sp)->sc_bind_addr, sizeof(baddr.sin_addr));
-#endif
+               if( SVC_CONF(sp)->sc_bind_addr != NULL ) {
+                  if( SVC_CONF(sp)->sc_bind_addr->sa.sa_family == AF_INET )
+                     length = sizeof(struct sockaddr_in);
+                  else if( SVC_CONF(sp)->sc_bind_addr->sa.sa_family == AF_INET6)
+                     length = sizeof(struct sockaddr_in6);
+                  if( getnameinfo(&SVC_CONF(sp)->sc_bind_addr->sa, length, 
+                        bname, NI_MAXHOST, NULL, 0, 0) )
+                     strcpy(bname, "unknown");
+               }
   
                if( SVC_CONF(sp)->sc_redir_addr != NULL )
                {
-#ifdef INET6
-                  memcpy(&raddr.sin6_addr, SVC_CONF(sp)->sc_bind_addr, sizeof(raddr.sin6_addr));
-#else
-                  memcpy(&raddr.sin_addr, SVC_CONF(sp)->sc_bind_addr, sizeof(raddr.sin_addr));
-#endif
+                  if( SVC_CONF(sp)->sc_redir_addr->sa.sa_family == AF_INET )
+                     length = sizeof(struct sockaddr_in);
+                  else if(SVC_CONF(sp)->sc_redir_addr->sa.sa_family == AF_INET6)
+                     length = sizeof(struct sockaddr_in6);
+                  if( getnameinfo(&SVC_CONF(sp)->sc_redir_addr->sa, length, 
+                        rname, NI_MAXHOST, NULL, 0, 0) )
+                     strcpy(rname, "unknown");
                   Sprint(descriptor, "%-10s ", SC_NAME( SVC_CONF( sp ) ) );
                   Sprint(descriptor, "%-6d ", SC_PORT( SVC_CONF( sp ) ) );
-                  Sprint(descriptor, "%-16s ", xntoa( SA(&baddr) ) );
+                  Sprint(descriptor, "%-16s ", bname );
                   Sprint(descriptor, "%-6d ", SVC_CONF(sp)->sc_uid );
-                  Sprint(descriptor, "%-16s ", xntoa(SA(&raddr)) );
-                  Sprint(descriptor, "%-6d\n", SVC_CONF(sp)->sc_redir_port );
+                  Sprint(descriptor, "%-16s ", rname );
+                  Sprint(descriptor, "%-6d\n", xaddrport(SVC_CONF(sp)->sc_redir_addr) );
                }
                else
                {
                   Sprint(descriptor, "%-10s ", SC_NAME( SVC_CONF( sp ) ) );
                   Sprint(descriptor, "%-6d ", SC_PORT( SVC_CONF( sp ) ) );
-                  Sprint(descriptor, "%-16s ", xntoa( SA(&baddr) ) );
+                  Sprint(descriptor, "%-16s ", bname );
                   Sprint(descriptor, "%-6d\n", SVC_CONF(sp)->sc_uid );
                }
             }
@@ -665,6 +658,9 @@ PRIVATE void xadmin_handler( struct server *serp )
          }
       }
    }
+   /* XXX: NOTREACHED (the compiler agrees) */
+#if 0
    Sprint(descriptor, "exiting\n");
    Sflush(descriptor);
+#endif
 }
