@@ -14,6 +14,7 @@
 
 #include "defs.h"
 
+
 #define FORK            YES
 #define NO_FORK         NO
 
@@ -39,8 +40,7 @@ struct builtin_service
    builtin_s   bs_handle ;
 } ;
 
-builtin_s *builtin_find(char *, int) ;
-builtin_s *builtin_lookup(struct builtin_service services[], char *, int) ;
+const builtin_s *builtin_find(const char *service_name,int type);
+const builtin_s *builtin_lookup(const struct builtin_service services[],const char *service_name,int type);
 
 #endif   /* BUILTIN_H */
-
