@@ -5,13 +5,14 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: child.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+#ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
+#endif
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <netinet/in.h>
@@ -23,37 +24,37 @@ static char RCSid[] = "$Id: child.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $"
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <string.h>
 #if defined (HAVE_GRP_H)
 #include <grp.h>
 #endif
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
 
 #include "str.h"
-#include "pset.h"
-
-#include "options.h"
-
-#include "attr.h"
-#include "server.h"
-#include "state.h"
-#include "sconst.h"
+#include "child.h"
+#include "sconf.h"
+#include "msg.h"
+#include "main.h"
 #include "xconfig.h"
-#include "log.h"
-#include "servicedefs.h"
+#include "ident.h"
+#include "sconst.h"
 #include "signals.h"
-
-extern void redir_handler(struct server *serp);
+#include "options.h"
+#include "redirect.h"
 
 /*
  * This function is running in the new process
  */
-PRIVATE void exec_server( struct server *serp )
+static void exec_server( const struct server *serp )
 {
-   struct service_config *scp = SVC_CONF( SERVER_SERVICE( serp ) ) ;
+   const struct service_config *scp = SVC_CONF( SERVER_SERVICE( serp ) ) ;
    struct rlimit rl ;
    int fd ;
    int descriptor = SERVER_FD( serp ) ;
-   char *server = SC_SERVER( scp ) ;
-   char *func = "exec_server" ;
+   const char *server = SC_SERVER( scp ) ;
+   const char *func = "exec_server" ;
 
    /*
     * The following code solves a problem with post-version-4.3
@@ -86,11 +87,6 @@ PRIVATE void exec_server( struct server *serp )
       }
    }
 
-   (void) close( descriptor ) ;
-
-#ifndef solaris
-   no_control_tty() ;
-#endif
 
 #ifdef RLIMIT_NOFILE
    rl.rlim_cur = ps.ros.orig_max_descriptors ;
@@ -138,6 +134,12 @@ PRIVATE void exec_server( struct server *serp )
    }
 #endif
 
+   (void) close( descriptor ) ;
+
+#ifndef solaris
+   no_control_tty() ;
+#endif
+
    msg_suspend() ;
 
    (void) execve( server, SC_SERVER_ARGV( scp ),
@@ -159,9 +161,9 @@ PRIVATE void exec_server( struct server *serp )
  * of ps.ros.Argv's. 
  * The rest of ps.ros.Argv is cleared to spaces
  */
-PRIVATE void rename_process( char *name )
+static void rename_process( const char *name )
 {
-   char *from = name ;
+   const char *from = name ;
    char *to = ps.ros.Argv[ 0 ] ;
    int tmp_index = 1 ;
 
@@ -181,9 +183,9 @@ PRIVATE void rename_process( char *name )
 }
 
 
-PRIVATE void set_credentials( struct service_config *scp )
+static void set_credentials( const struct service_config *scp )
 {
-   char *func = "set_credentials" ;
+   const char *func = "set_credentials" ;
 
    if ( SC_SPECIFIED( scp, A_GROUP ) || SC_SPECIFIED( scp, A_USER ) ) {
       if ( ps.ros.is_superuser )
@@ -240,22 +242,15 @@ PRIVATE void set_credentials( struct service_config *scp )
          }
 #endif   /* ! NO_INITGROUPS */
       }
-      else
-         msg( LOG_WARNING, func, "can't change gid; not superuser" ) ;
    }
 
    if ( SC_SPECIFIED( scp, A_USER ) ) {
-      if ( ps.ros.is_superuser )
-      {
          if ( setuid( SC_UID( scp ) ) == -1 )
          {
             msg( LOG_ERR, func, "setuid failed: %m" ) ;
             _exit( 1 ) ;
          }
       }
-      else
-         msg( LOG_WARNING, func, "can't change uid; not superuser" ) ;
-   }
 
    if ( SC_SPECIFIED( scp, A_UMASK ) ) 
       umask(scp->sc_umask);
@@ -283,13 +278,17 @@ void child_process( struct server *serp )
    }
 #endif
 
+   if ( ! SC_IS_INTERCEPTED( scp ) )
+   {
+      set_credentials( scp ) ;
+      signal_default_state() ;
+      if ( SC_SPECIFIED( scp, A_NICE ) )
+         (void) nice( SC_NICE( scp ) ) ;
+   }
+
    if ( svc_child_access_control(sp, cp) != OK ) {
       if ( SVC_LOGS_USERID_ON_FAILURE( sp ) )
          (void) conn_add_alternative( cp, LOG_SERVICE( ps ) ) ;
-      if ( sp->svc_shutdown_func != NULL ) {
-         if ( ! SVC_RECORDS( sp ) || conn_add_alternative( cp, SHUTDOWN_SERVICE( ps ) ) == FAILED )
-            conn_shutdown( cp ) ;
-      }
          
       exit(0);
    }
@@ -315,13 +314,6 @@ void child_process( struct server *serp )
       }
    }
 
-   if ( ! SC_IS_INTERCEPTED( scp ) )
-   {
-      set_credentials( scp ) ;
-      signal_default_state() ;
-      if ( SC_SPECIFIED( scp, A_NICE ) )
-         (void) nice( SC_NICE( scp ) ) ;
-   }
 
    /* this is where the server gets executed  -bbraun */
    if ( ! SC_IS_INTERNAL( scp ) )
@@ -332,12 +324,14 @@ void child_process( struct server *serp )
       }
       else
       {
-#ifndef INET6
 #if defined(HAVE_SETENV)
          char buff[1024];
+
          strx_sprint(buff, sizeof(buff)-1, "REMOTE_HOST=%s", conn_addrstr(cp));
-         env_addstr(SC_ENV(scp)->env_handle, buff);
-#endif
+         if( env_addstr(SC_ENV(scp)->env_handle, buff) != ENV_OK ) {
+            msg( LOG_ERR, "child_process", "Error adding REMOTE_HOST variable for %s: %m", SC_NAME(scp) );
+            _exit( 1 ) ;
+         }
 #endif
          exec_server( serp ) ;
       }
@@ -353,28 +347,34 @@ void child_process( struct server *serp )
        *            <program_name> <service-id> interceptor
        */
       if ( SC_IS_INTERCEPTED( scp ) )
-         strx_print( INT_NULL, name, sizeof( name ),
+         strx_print( INT_NULL, name, sizeof( name ) - 1,
                            "%s %s interceptor", program_name, SC_ID( scp ) ) ;
       else
       {
          int namelen = sizeof( name ) - 1 ;      /* leave space for the NUL */
-#ifdef INET6
-         struct sockaddr_in6 *sinp = conn_address( SERVER_CONNECTION( serp ) ) ;
-#else
-         struct sockaddr_in *sinp = conn_address( SERVER_CONNECTION( serp ) ) ;
-#endif
-         int len = strx_nprint( name, namelen,
-                              "(%s service) %s", program_name, SC_ID( scp ) ) ;
-         
-#ifdef INET6
-         if ( SC_ACCEPTS_CONNECTIONS( scp ) && !IN6_IS_ADDR_UNSPECIFIED(sinp) )
-            strx_print( INT_NULL, &name[ len ], namelen - len,
-                                       " %s" , xntoa( SA(sinp) ) ) ;
-#else
-         if ( SC_ACCEPTS_CONNECTIONS( scp ) && sinp != SOCKADDRIN_NULL )
-            strx_print( INT_NULL, &name[ len ], namelen - len,
-                                       " %s", xntoa( SA(sinp) ) ) ;
-#endif
+         char host[NI_MAXHOST];
+         size_t hostlen = NI_MAXHOST;
+         socklen_t addrlen = 0;
+         union xsockaddr *sinp = conn_xaddress(SERVER_CONNECTION(serp));
+         int len;
+
+         if( sinp == NULL )
+            exit(0);
+
+         if( SC_IPV6(scp) ) addrlen = sizeof(struct sockaddr_in6);
+         else if( SC_IPV4(scp) ) addrlen = sizeof(struct sockaddr_in);
+
+         len = strx_nprint(name, namelen, "(%s service) %s", program_name,
+            SC_ID( scp ) ) ;
+
+         if( getnameinfo( SA(sinp), addrlen, host, hostlen, NULL, 0, 0) != 0 )
+               strcpy(host, "unknown");
+
+         if ( SC_IPV6(scp) && SC_ACCEPTS_CONNECTIONS( scp ) && 
+               !IN6_IS_ADDR_UNSPECIFIED(&sinp->sa_in6.sin6_addr) )
+            strx_print( INT_NULL, &name[ len ], namelen - len, " %s" , host ) ;
+         if ( SC_IPV4(scp) && SC_ACCEPTS_CONNECTIONS( scp ) )
+            strx_print( INT_NULL, &name[ len ], namelen - len, " %s", host ) ;
       }
       rename_process( name ) ;
       svc_internal( sp, serp ) ;
@@ -387,9 +387,9 @@ void child_process( struct server *serp )
 /*
  * This function is invoked when a SIGCLD is received
  */
-void child_exit()
+void child_exit(void)
 {
-   char *func = "child_exit" ;
+   const char *func = "child_exit" ;
 
    for ( ;; )         /* Find all children that exited */
    {
