@@ -6,5 +6,9 @@
 #ifndef _X_CHILD
 #define _X_CHILD
 
-void child_process( struct server *serp );
+#include "defs.h"
+
+void child_process(struct server *serp);
+void child_exit(void);
+
 #endif
