@@ -12,9 +12,11 @@
  * $Id: conf.h,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $
  */
 
+
 #include "pset.h"
 #include "sconf.h"
 
+
 struct configuration
 {
    pset_h cnf_service_confs ;
@@ -24,12 +26,12 @@ struct configuration
 #define CNF_DEFAULTS( confp )        (confp)->cnf_defaults
 #define CNF_SERVICE_CONFS( confp )   (confp)->cnf_service_confs
 
-status_e                 cnf_get(struct configuration *, time_t) ;
-status_e                 cnf_init(struct configuration *, int *, psi_h *) ;
-void                     cnf_free(struct configuration *) ;
-void                     cnf_dump(struct configuration *, int) ;
-struct service_config   *cnf_extract(struct configuration *, struct service_config *) ;
-unsigned                 cnf_start_services(struct configuration *) ;
+void cnf_free(struct configuration *confp);
+struct service_config *cnf_extract(struct configuration *confp,struct service_config *scp);
+void cnf_dump(struct configuration *confp,int fd);
+status_e cnf_init(struct configuration *confp,int *fdp,psi_h *iterp);
+unsigned cnf_start_services(struct configuration *confp);
+
 
 #endif    /* CONF_H */
 
