@@ -24,23 +24,16 @@
 #include <netdb.h>
 #endif
 
-#include "pset.h"
-
-#include "attr.h"
-#include "xconfig.h"
-#include "conf.h"
-#include "defs.h"
-#include "parse.h"
-#include "sconst.h"
-#include "sconf.h"
-#include "state.h"
-#include "options.h"
-#include "log.h"
-#include "special.h"
 #include "str.h"
-
-status_e setup_environ( struct service_config *, struct service_config *);
-void parse_inet_conf_file( int , struct configuration * );
+#include "sio.h"
+#include "confparse.h"
+#include "msg.h"
+#include "xconfig.h"
+#include "parse.h"
+#include "special.h"
+#include "sconst.h"
+#include "env.h"
+#include "sconf.h"
 
 /*
  * Pset iterator used by functions in this file.
@@ -50,14 +43,17 @@ void parse_inet_conf_file( int , struct configuration * );
  */
 static psi_h iter ;
 
-PRIVATE status_e fix_server_argv( struct service_config *scp )
+static status_e fix_server_argv( struct service_config *scp )
 {
    char *server_name ;
-   char *func = "fix_server_argv" ;
+   const char *func = "fix_server_argv" ;
+
+   if( scp->sc_server == NULL )
+      return( FAILED );
 
    if( SC_NAMEINARGS( scp ) ) {
       if( !SC_SPECIFIED(scp, A_SERVER_ARGS ) ){
-         msg(LOG_ERR, func, "Must specify server args if using NAMEINARGS flag\n");
+         msg(LOG_ERR, func, "Must specify server args if using NAMEINARGS flag");
          return( FAILED );
       }
 
@@ -117,10 +113,10 @@ PRIVATE status_e fix_server_argv( struct service_config *scp )
  *         'defaults' anymore.
  *      3) default values (like the service instance limit)
  */
-PRIVATE status_e attr_fill( struct service_config *scp, 
+static status_e attr_fill( struct service_config *scp, 
                             struct service_config *def )
 {
-   char *func = "attr_fill" ;
+   const char *func = "attr_fill" ;
 
    if ( ! SC_IS_INTERNAL( scp ) && fix_server_argv( scp ) == FAILED )
       return( FAILED ) ;
@@ -152,8 +148,8 @@ PRIVATE status_e attr_fill( struct service_config *scp,
 
    if ( ! SC_SPECIFIED( scp, A_CPS ) ) 
    {
-      scp->sc_time_conn_max = SC_SPECIFIED( def, A_CPS ) ? def->sc_time_conn_max : 0;
-      scp->sc_time_wait = SC_SPECIFIED( def, A_CPS ) ? def->sc_time_wait : 0;
+      scp->sc_time_conn_max = SC_SPECIFIED( def, A_CPS ) ? def->sc_time_conn_max : DEFAULT_LOOP_RATE;
+      scp->sc_time_wait = SC_SPECIFIED( def, A_CPS ) ? def->sc_time_wait : DEFAULT_LOOP_TIME;
       scp->sc_time_reenable = 0;
    }
 
@@ -167,13 +163,11 @@ PRIVATE status_e attr_fill( struct service_config *scp,
       SC_SPECIFY( scp, A_BIND ) ;
    }
 
-#ifdef INET6
    if ( ! SC_SPECIFIED( scp, A_V6ONLY ) ) {
       scp->sc_v6only = SC_SPECIFIED( def, A_V6ONLY ) ? def->sc_v6only : NO;
       SC_PRESENT( scp, A_V6ONLY );
       SC_SPECIFY( scp, A_V6ONLY );
    }
-#endif
 
    if ( ! SC_SPECIFIED( scp, A_DENY_TIME ) )
    {
@@ -181,6 +175,55 @@ PRIVATE status_e attr_fill( struct service_config *scp,
       SC_SPECIFY( scp, A_DENY_TIME ) ;
    }
 
+   if ( (! SC_IPV4( scp )) && (!SC_IPV6( scp )) )
+   {
+#ifdef INET6
+      M_SET(scp->sc_xflags, SF_IPV6);
+#else
+      M_SET(scp->sc_xflags, SF_IPV4);
+#endif
+   }
+
+   /* This should be removed if sock_stream is ever something other than TCP,
+    * or sock_dgram is ever something other than UDP.
+    */
+   if ( (! SC_SPECIFIED( scp, A_PROTOCOL )) && ( SC_SPECIFIED( scp, A_SOCKET_TYPE ) ) )
+   {
+      struct protoent *pep ;
+
+      if( scp->sc_socket_type == SOCK_STREAM ) {
+         if( (pep = getprotobyname( "tcp" )) != NULL ) {
+            scp->sc_protocol.name = new_string ( "tcp" );
+            if( scp->sc_protocol.name == NULL )
+               return( FAILED );
+            scp->sc_protocol.value = pep->p_proto ;
+            SC_SPECIFY(scp, A_PROTOCOL);
+         }
+      }
+
+      if( scp->sc_socket_type == SOCK_DGRAM ) {
+         if( (pep = getprotobyname( "udp" )) != NULL ) {
+            scp->sc_protocol.name = new_string ( "udp" );
+            if( scp->sc_protocol.name == NULL )
+               return( FAILED );
+            scp->sc_protocol.value = pep->p_proto ;
+            SC_SPECIFY(scp, A_PROTOCOL);
+         }
+      }
+   }
+   if ( ( SC_SPECIFIED( scp, A_PROTOCOL )) && (! SC_SPECIFIED( scp, A_SOCKET_TYPE ) ) )
+   {
+      if( (scp->sc_protocol.name != NULL) && (strcmp("tcp", scp->sc_protocol.name) == 0) ) {
+            scp->sc_socket_type = SOCK_STREAM;
+            SC_SPECIFY(scp, A_SOCKET_TYPE);
+      }
+
+      if( (scp->sc_protocol.name != NULL) && (strcmp("udp", scp->sc_protocol.name) == 0) ) {
+            scp->sc_socket_type = SOCK_DGRAM;
+            SC_SPECIFY(scp, A_SOCKET_TYPE);
+      }
+   }
+
    if ( USE_DEFAULT( scp, def, A_LOG_ON_SUCCESS ) )
    {
       scp->sc_log_on_success = def->sc_log_on_success ;
@@ -225,7 +268,7 @@ PRIVATE status_e attr_fill( struct service_config *scp,
 }
 
 
-PRIVATE void remove_disabled_services( struct configuration *confp )
+static void remove_disabled_services( struct configuration *confp )
 {
    pset_h disabled_services ;
    pset_h enabled_services ;
@@ -247,37 +290,22 @@ PRIVATE void remove_disabled_services( struct configuration *confp )
          register unsigned u ;
 
          for ( u = 0 ; u < pset_count( enabled_services ) ; u++ ) {
-/*
-            char * name = pset_pointer( enabled_services, u );
-*/
             if ( EQ( sid, (char *) pset_pointer( enabled_services, u ) ) ) {
                SC_ENABLE( scp );
                break;
             }
          }
       }
-
-      /* Remove any services that are left marked disabled */
-      for ( scp = SCP( psi_start( iter ) ) ; scp ; scp = SCP( psi_next(iter)) ){
-         if( SC_IS_DISABLED( scp ) ) {
-            syslog(LOG_ERR, "removing %s", scp->sc_name);
-            SC_DISABLE( scp );
-            sc_free(scp);
-            psi_remove(iter);
-         }
-      }
    }
 
-   /* This loop is to remove any services that have the flag "DISABLE"
-    * set.
-    */
+   /* Remove any services that are left marked disabled */
    for ( scp = SCP( psi_start( iter ) ) ; scp ; scp = SCP( psi_next(iter)) ){
-      if( SC_SDISABLE( scp ) ) {
-         syslog(LOG_ERR, "%s disabled, removing", scp->sc_name);
+      if( SC_IS_DISABLED( scp ) ) {
+         msg(LOG_DEBUG, "remove_disabled_services", "removing %s", scp->sc_name);
+         SC_DISABLE( scp );
          sc_free(scp);
          psi_remove(iter);
       }
-
    }
 
    if ( ! SC_SPECIFIED( defaults, A_DISABLED ) )
@@ -304,15 +332,14 @@ PRIVATE void remove_disabled_services( struct configuration *confp )
 /*
  * Check if all required attributes have been specified
  */
-PRIVATE status_e attr_check( struct service_config *scp )
+static status_e attr_check( struct service_config *scp )
 {
    mask_t         necessary_and_specified ;
    mask_t         necessary_and_missing ;
    mask_t         must_specify = NECESSARY_ATTRS ;
    int            attr_id ;
-   char            *attr_name ;
-   char            *func = "attr_check" ;
-   char            *attr_name_lookup() ;
+   char          *attr_name ;
+   const char    *func = "attr_check" ;
 
    /*
     * Determine what attributes must be specified
@@ -335,6 +362,17 @@ PRIVATE status_e attr_check( struct service_config *scp )
       }
    }
 
+   if( SC_IPV4( scp ) && SC_IPV6( scp ) ) {
+      msg( LOG_ERR, func, "Service %s specified as both IPv4 and IPv6", SC_NAME(scp));
+      return( FAILED );
+   }
+
+   if( (scp->sc_redir_addr != NULL) ) {
+#define DEFAULT_SERVER "/bin/true"
+       scp->sc_server = new_string( DEFAULT_SERVER );
+       SC_SPECIFY(scp, A_SERVER);
+   }
+
    /*
     * Check if all necessary attributes have been specified
     *
@@ -359,19 +397,6 @@ PRIVATE status_e attr_check( struct service_config *scp )
       if ( M_IS_SET( necessary_and_missing, attr_id ) && 
                   ( attr_name = attr_name_lookup( attr_id ) ) != NULL )
       {
-         /* This is here to make redir without server a valid option */
-         /* We must put something in server or things get messy */
-         if( attr_id == A_SERVER )
-         {
-            if( (scp->sc_redir_addr != NULL) )
-            {
-#define DEFAULT_SERVER "/bin/true"
-               scp->sc_server = new_string( DEFAULT_SERVER );
-               SC_PRESENT(scp, A_SERVER);
-            
-               return OK;
-            }
-         }
          msg( LOG_ERR, func,
             "Service %s missing attribute %s", scp->sc_id, attr_name ) ;
       }
@@ -388,9 +413,9 @@ PRIVATE status_e attr_check( struct service_config *scp )
  *      2. For RPC services, it finds the program number
  *      3. For non-RPC services, it finds the port number.
  */
-PRIVATE status_e check_entry( struct service_config *scp )
+static status_e check_entry( struct service_config *scp )
 {
-   char *func = "check_entry" ;
+   const char *func = "check_entry" ;
 
    if ( attr_check( scp ) == FAILED )
       return( FAILED ) ;
@@ -425,8 +450,6 @@ PRIVATE status_e check_entry( struct service_config *scp )
    {   /* If SENSOR flagged redirect to internal builtin function. */ 
       if ( SC_SENSOR( scp ) )
       {
-         extern pset_h global_no_access;
-         extern pset_h global_no_access_time;
          if (   global_no_access == NULL)
             global_no_access = pset_create(10, 10);
          if (   global_no_access_time == NULL)
@@ -491,11 +514,20 @@ PRIVATE status_e check_entry( struct service_config *scp )
       else
       {
          struct protoent *pep ;
+         char *protoname=CHAR_NULL;
 
-         sep = getservbyname( scp->sc_name, CHAR_NULL ) ;
+         if ( SC_SPECIFIED( scp, A_SOCKET_TYPE ) ) {
+            if (SC_SOCKET_TYPE(scp) == SOCK_DGRAM) {
+               protoname="udp";
+            } else if ((SC_SOCKET_TYPE(scp) == SOCK_STREAM)) {
+               protoname="tcp";
+            }
+         }
+
+         sep = getservbyname( scp->sc_name, protoname ) ;
          if ( sep == NULL )
          {
-            msg( LOG_ERR, func, "Unknown service: %s", scp->sc_name ) ;
+            msg( LOG_ERR, func, "Unknown service: %s/%s", scp->sc_name, protoname ? protoname : "(any protocol)" ) ;
             return( FAILED ) ;
          }
          scp->sc_protocol.name = new_string( sep->s_proto ) ;
@@ -532,77 +564,14 @@ PRIVATE status_e check_entry( struct service_config *scp )
    return( OK ) ;
 }
 
-
-#if CONF_TIMEOUT != 0 && ! defined( NO_TIMERS )
-
-#include <sys/time.h>
-#include "timer.h"
-#include "timemacros.h"
-
-static timer_h conf_timer ;
-
-status_e create_conf_timer()
-{
-   conf_timer = xtimer_create( TIMER_REAL, TIMER_RETURN_ERROR, INT_NULL ) ;
-   return( ( conf_timer == NULL ) ? FAILED : OK ) ;
-}
-
-#endif   /* CONF_TIMEOUT != 0 && ! NO_TIMERS */
-
-
 /*
  * Get a configuration from the specified file.
- * In case of timeout, do cleanup
  */
-PRIVATE status_e get_conf( int fd, struct configuration *confp, time_t timeout )
+static status_e get_conf( int fd, struct configuration *confp )
 {
-#if CONF_TIMEOUT != 0 && ! defined( NO_TIMERS )
-
-   struct itimerval itv ;
-   struct timer_action ta ;
-   char *func = "get_conf" ;
-   
-   if ( timeout )
-   {
-      if ( setjmp( ta.ta_env ) )
-      {
-         parse_end() ;
-         return( FAILED ) ;
-      }
-
-      /*
-       * Start the timer
-       */
-      TV_ZERO( itv.it_interval ) ;
-      itv.it_value.tv_sec   = timeout ;
-      itv.it_value.tv_usec   = 0 ;
-      ta.ta_flags            = TIMER_LONGJMP ;
-      ta.ta_func            = NULL ;
-      ta.ta_arg            = VOID_NULL ;
-      if ( xtimer_start( conf_timer, &itv, TIMER_RELATIVE, &ta ) == TIMER_ERR )
-      {
-         msg( LOG_ERR, func, "timer creation failed" ) ;
-         return( FAILED ) ;
-      }
-   }
-
-   parse_conf_file( fd, confp ) ;
-
-   if ( timeout )
-      xtimer_stop( conf_timer ) ;
-   parse_end() ;
-   return( OK ) ;
-
-#else      /* CONF_TIMEOUT == 0 || NO_TIMERS */
-
-#ifdef lint
-   timeout = timeout ;
-#endif
    parse_conf_file( fd, confp ) ;
    parse_end() ;
    return( OK ) ;
-#endif   /* CONF_TIMEOUT != 0 && ! NO_TIMERS */
-
 }
 
 
@@ -618,21 +587,16 @@ PRIVATE status_e get_conf( int fd, struct configuration *confp, time_t timeout )
 /*
  * Get a configuration by reading the configuration file.
  */
-status_e cnf_get( struct configuration *confp, time_t timeout )
+status_e cnf_get( struct configuration *confp )
 {
    int config_fd ;
    struct service_config *scp ;
-   char *func = "get_configuration" ;
+   const char *func = "get_configuration" ;
 
    if ( cnf_init( confp, &config_fd, &iter ) == FAILED )
       return( FAILED ) ;
 
-   if ( inet_compat ) 
-   {
-      syslog(LOG_ERR, "inet compatibility mode invoked");
-      parse_inet_conf_file( config_fd, confp );
-   }
-   else if ( get_conf( config_fd, confp, timeout ) == FAILED )
+   else if ( get_conf( config_fd, confp ) == FAILED )
    {
       Sclose( config_fd ) ;
       cnf_free( confp ) ;
@@ -670,7 +634,7 @@ status_e cnf_get( struct configuration *confp, time_t timeout )
        */
       if ( SC_IS_INTERCEPTED( scp ) )
       {
-         builtin_s *bp ;
+         const builtin_s *bp ;
 
          bp = spec_find( INTERCEPT_SERVICE_NAME, scp->sc_socket_type ) ;
          if ( bp == NULL )
@@ -712,4 +676,3 @@ status_e cnf_get( struct configuration *confp, time_t timeout )
    return( OK ) ;
 }
 
-
