@@ -5,7 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: connection.c,v 1.2 1999/10/13 04:22:34 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
@@ -20,24 +19,28 @@ static char RCSid[] = "$Id: connection.c,v 1.2 1999/10/13 04:22:34 bbraun Exp $"
 
 #include "sio.h"
 #include "connection.h"
-#include "service.h"
+#include "sconf.h"
+#include "msg.h"
+#include "main.h"
 #include "state.h"
-#include "log.h"
-#include "servicedefs.h"
+
 
 #define NEW_CONN()            NEW( connection_s )
-#define FREE_CONN( cop )         FREE( cop )
+#define FREE_CONN( cop )      FREE( cop )
 
 /*
  * Get a new connection request and initialize 'cp' appropriately
  */
-PRIVATE status_e get_connection( struct service *sp, connection_s *cp )
+static status_e get_connection( struct service *sp, connection_s *cp )
 {
    struct service_config *scp = SVC_CONF( sp );
-   int sin_len = sizeof( cp->co_remote_address ) ;
-   char *func = "get_connection" ;
+   int sin_len;
+   const char *func = "get_connection" ;
    int on = 1;
 
+   if( SC_IPV4(scp) ) sin_len = sizeof(struct sockaddr_in);
+   if( SC_IPV6(scp) ) sin_len = sizeof(struct sockaddr_in6);
+
    if ( SVC_SOCKET_TYPE( sp ) == SOCK_STREAM ) {
       /* If it's a TCP socket, and we're set to wait, the accept is
        * done by the child process.  Don't set NEW_DESCRIPTOR, since
@@ -47,7 +50,7 @@ PRIVATE status_e get_connection( struct service *sp, connection_s *cp )
       if( SC_WAITS( scp ) ) {
          cp->co_descriptor = SVC_FD( sp );
       } else {
-         cp->co_descriptor = accept( SVC_FD( sp ), SA( &cp->co_remote_address ), &sin_len ) ;
+         cp->co_descriptor = accept( SVC_FD( sp ), &(cp->co_remote_address.sa), &sin_len ) ;
          M_SET( cp->co_flags, COF_NEW_DESCRIPTOR ) ;
       }
 
@@ -65,15 +68,13 @@ PRIVATE status_e get_connection( struct service *sp, connection_s *cp )
          if( setsockopt(SVC_FD(sp), SOL_SOCKET, SO_KEEPALIVE, (char *)&on, sizeof(on)) < 0 )
             msg( LOG_WARNING, func, "service %s, setsockopt: %m", SVC_ID(sp));
 
-#ifdef INET6
-      if( !(SC_V6ONLY( scp ))  && (IN6_IS_ADDR_V4MAPPED(&cp->co_remote_address.sin6_addr) || IN6_IS_ADDR_V4COMPAT(&cp->co_remote_address.sin6_addr)) ) {
+      if( SC_IPV6(scp) && !(SC_V6ONLY( scp ))  && (IN6_IS_ADDR_V4MAPPED(&cp->co_remote_address.sa_in6.sin6_addr) || IN6_IS_ADDR_V4COMPAT(&cp->co_remote_address.sa_in6.sin6_addr)) ) {
          int af = AF_INET;
          if( setsockopt(cp->co_descriptor, IPPROTO_IPV6,
                IPV6_ADDRFORM, &af, sizeof(af)) ) {
             msg( LOG_WARNING, func, "service %s, IPV6_ADDRFORM", SVC_ID( sp) );
          }
       }
-#endif
 
       M_SET( cp->co_flags, COF_HAVE_ADDRESS ) ;
    }
@@ -88,7 +89,7 @@ PRIVATE status_e get_connection( struct service *sp, connection_s *cp )
           * select(2) guaranteed that we won't block on the recvfrom
           */
          if ( recvfrom( SVC_FD( sp ), &t_ch, 1, MSG_PEEK,
-                              SA( &cp->co_remote_address ), &sin_len ) == -1 )
+                              &cp->co_remote_address.sa, &sin_len ) == -1 )
          {
             msg( LOG_ERR, func, "service %s, recvfrom: %m", SVC_ID( sp ) ) ;
             return( FAILED ) ;
@@ -96,8 +97,6 @@ PRIVATE status_e get_connection( struct service *sp, connection_s *cp )
          M_SET( cp->co_flags, COF_HAVE_ADDRESS ) ;
       }
 
-      if( SC_WAITS(scp) )
-         FD_CLR(SVC_FD( sp ), &ps.rws.socket_mask ) ;
       cp->co_descriptor = SVC_FD( sp ) ;
    }
 
@@ -114,7 +113,7 @@ connection_s *conn_new( struct service *sp )
 {
    connection_s    new_conn ;
    connection_s   *cp ;
-   char           *func = "conn_new" ;
+   const char     *func = "conn_new" ;
 
    CLEAR( new_conn ) ;
 
@@ -137,11 +136,10 @@ connection_s *conn_new( struct service *sp )
    {
       out_of_memory( func ) ;
       conn_cleanup( &new_conn ) ;
-      conn_shutdown( &new_conn ) ;
-      conn_free( &new_conn ) ;
+      conn_free( &new_conn, 0 ) ;
       return( CONN_NULL ) ;
    }
-   *cp = new_conn ;
+   memcpy(cp, &new_conn, sizeof(connection_s));
    return( cp ) ;
 }
 
@@ -163,24 +161,25 @@ void conn_close( connection_s *cp )
 /*
  * Release the specified connection.
  * Certain actions may be performed before doing this:
- *      - invocation of the service shutdown function
  *      - drain of a single UDP packet if the socket type is SOCK_DGRAM
  */
-void conn_free( connection_s *cp )
+void conn_free( connection_s *cp, int release_mem )
 {
    struct service *sp = cp->co_sp ;
    int i ;
 
-   if ( SVC_IS_SUSPENDED( sp ) )
+   if( cp == NULL )
+      return;
+   if ( SVC_IS_SUSPENDED( sp ) ) {
+      if( debug.on )
+         msg( LOG_INFO, "conn_free", "Resuming suspended service") ;
       svc_resume( sp ) ;
+   }
 
    if ( cp->co_state == CONN_OPEN )
    {
-      if ( M_IS_SET( cp->co_flags, COF_SHUTDOWN ) )
-         svc_shutdown( sp, cp, (char **)NULL ) ;
-      
       if ( M_IS_SET( cp->co_flags, COF_CLEANUP ) && 
-                           SVC_SOCKET_TYPE( sp ) == SOCK_DGRAM )
+                  (SVC_SOCKET_TYPE( sp ) == SOCK_DGRAM) && SVC_IS_ACTIVE( sp ))
          drain( cp->co_descriptor ) ;
    }
 
@@ -197,13 +196,16 @@ void conn_free( connection_s *cp )
 
    conn_close( cp ) ;
 
-   FREE_CONN( cp ) ;
+   if (release_mem) {
+      FREE_CONN( cp ) ;
+      cp = NULL;
+   }
 }
 
 
 status_e conn_add_alternative( connection_s *cp, struct service *sp )
 {
-   char *func = "conn_add_alternative" ;
+   const char *func = "conn_add_alternative" ;
 
    if ( sp == NULL )
       return( FAILED ) ;
@@ -233,7 +235,7 @@ status_e conn_add_alternative( connection_s *cp, struct service *sp )
  */
 status_e conn_start_alternative( connection_s *cp )
 {
-   char *func = "conn_start_alternative" ;
+   const char *func = "conn_start_alternative" ;
 
    while ( cp->co_next_alternative < cp->co_alternative_count )
    {
@@ -250,28 +252,52 @@ status_e conn_start_alternative( connection_s *cp )
    return( FAILED ) ;
 }
 
+/* This returns a pointer to a local static stack variable.
+ * The behavior is a remnant of inet_ntoa() behavior.
+ */ 
+char *conn_addrstr( const connection_s *cp )
+{
+   static char name[NI_MAXHOST];
+   int len = 0;
 
-void conn_dump( connection_s *cp, int fd )
+   if( !M_IS_SET( (cp)->co_flags, COF_HAVE_ADDRESS ) )
+      return "<no address>";
+
+   if( cp->co_remote_address.sa.sa_family == AF_INET ) 
+      len = sizeof(struct sockaddr_in);
+   else if( cp->co_remote_address.sa.sa_family == AF_INET6 )
+      len = sizeof(struct sockaddr_in6);
+
+   if( getnameinfo( &cp->co_remote_address.sa, len,
+         name, NI_MAXHOST, NULL, 0, NI_NUMERICHOST ) ) {
+      return "<no address>";
+   }
+   return name;
+}
+
+void conn_dump( const connection_s *cp, int fd )
 {
-#ifndef INET6
    unsigned u ;
+   const char *name = conn_addrstr( cp );
 
    tabprint( fd, 1, "state = %s\n",
          ( cp->co_state == CONN_CLOSED ) ? "CLOSED" : "OPEN" ) ;
 
    tabprint( fd, 1, "service = %s\n", SVC_ID( cp->co_sp ) ) ;
    tabprint( fd, 1, "descriptor = %d\n", cp->co_descriptor ) ;
-   tabprint( fd, 1, "flags = %#x\n", cp->co_flags ) ;
-   tabprint( fd, 1, "remote_address = %s,%d\n",
-      inet_ntoa( cp->co_remote_address.sin_addr ),
-                              ntohs( cp->co_remote_address.sin_port ) ) ;
+#if defined(__GNUC__) && !defined(__arch64__) && !defined(__alpha__)
+   tabprint( fd, 1, "flags = %#llx\n", cp->co_flags ) ;
+#else
+   tabprint( fd, 1, "flags = %#lx\n", cp->co_flags ) ;
+#endif
+   tabprint( fd, 1, "remote_address = %s,%d\n", name,
+                              ntohs( cp->co_remote_address.sa_in.sin_port ) ) ;
    tabprint( fd, 1, "Alternative services = " ) ;
    for ( u = 0 ; u < cp->co_alternative_count ; u++ )
       Sprint( fd, " %s", SVC_ID( cp->co_alternatives[ u ] ) ) ;
    Sputchar( fd, '\n' ) ;
-   if ( cp->co_alternative_count > 0 )
+   if ( cp->co_alternative_count != 0 )
       tabprint( fd, 1, "next alternative service = %s\n",
             SVC_ID( cp->co_alternatives[ cp->co_next_alternative ] ) ) ;
-#endif
 }
 
