@@ -5,34 +5,28 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: env.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #include <syslog.h>
 #include <string.h>
 #include <sys/types.h>
 
-#include "misc.h"
-
-#include "attr.h"
-#include "defs.h"
-#include "xconfig.h"
-#include "sconf.h"
-#include "log.h"
+#include "pset.h"
+#include "env.h"
+#include "msg.h"
 
 extern char **environ ;
-
 env_h std_env ;            /* created from environ */
 
-status_e initenv()
+status_e initenv(void)
 {
    std_env = env_make( environ ) ;
    return( ( std_env == NULL ) ? FAILED : OK ) ;
 }
 
-PRIVATE status_e make_env_with_strings(struct environment *, env_h, pset_h);
-PRIVATE status_e make_env_from_vars(struct environment *, env_h, pset_h) ;
-PRIVATE status_e update_env_with_strings(env_h, pset_h) ;
+static status_e make_env_with_strings(struct environment *, env_h, pset_h);
+static status_e make_env_from_vars(struct environment *, env_h, pset_h) ;
+static status_e update_env_with_strings(env_h, pset_h) ;
 
 
 status_e setup_environ( struct service_config *scp, struct service_config *def )
@@ -98,12 +92,12 @@ status_e setup_environ( struct service_config *scp, struct service_config *def )
  * Create a new environment from environ and env_strings
  * env_strings contains strings of the form "var=value"
  */
-PRIVATE status_e make_env_with_strings( struct environment *ep, 
+static status_e make_env_with_strings( struct environment *ep, 
                                         env_h env, 
                                         pset_h env_strings )
 {
    env_h      new_env ;
-   char      *func = "make_env_with_strings" ;
+   const char *func = "make_env_with_strings" ;
 
    if ( ( new_env = env_create( env ) ) == ENV_NULL )
    {
@@ -123,14 +117,14 @@ PRIVATE status_e make_env_with_strings( struct environment *ep,
 }
 
 
-PRIVATE status_e make_env_from_vars( struct environment *ep, 
+static status_e make_env_from_vars( struct environment *ep, 
                                      env_h env, 
                                      pset_h vars )
 {
    env_h               new_env ;
    char               *varname ;
    unsigned            u ;
-   char               *func = "make_env_from_vars" ;
+   const char         *func = "make_env_from_vars" ;
 
    if ( ( new_env = env_create( ENV_NULL ) ) == ENV_NULL )
    {
@@ -161,10 +155,10 @@ PRIVATE status_e make_env_from_vars( struct environment *ep,
 }
 
 
-PRIVATE status_e update_env_with_strings( env_h env, pset_h strings )
+static status_e update_env_with_strings( env_h env, pset_h strings )
 {
    unsigned u ;
-   char *func = "update_env_with_strings" ;
+   const char *func = "update_env_with_strings" ;
 
    for ( u = 0 ; u < pset_count( strings ) ; u++ )
    {
@@ -178,7 +172,7 @@ PRIVATE status_e update_env_with_strings( env_h env, pset_h strings )
                return( FAILED ) ;
             
             case ENV_EBADSTRING:
-               msg( LOG_ERR, func, "Bad environment string: %s\n", p ) ;
+               msg( LOG_ERR, func, "Bad environment string: %s", p ) ;
                break ;
          }
    }
