@@ -5,7 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: int.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
@@ -18,23 +17,24 @@ static char RCSid[] = "$Id: int.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 #include <stdlib.h>
 #include <unistd.h>
 
-#include "xconfig.h"
-#include "connection.h"
-#include "service.h"
-#include "server.h"
 #include "int.h"
+#include "msg.h"
 #include "log.h"
-#include "signals.h"
-#include "state.h"
+#include "tcpint.h"
+#include "udpint.h"
+#include "sconf.h"
+#include "intcommon.h"
 #include "child.h"
+#include "state.h"
+#include "main.h"
+#include "signals.h"
+#include "xconfig.h"
+#include <netdb.h>
 
-void si_exit();
-void di_exit();
+static void start_server( struct intercept_s *ip );
+static void terminate_server( struct intercept_s *ip );
 
-PRIVATE void start_server( struct intercept *ip );
-PRIVATE void terminate_server( struct intercept *ip );
-
-typedef struct intercept *(*initfunc)() ;
+typedef struct intercept_s *(*initfunc)() ;
 
 struct lookup_table
 {
@@ -42,8 +42,6 @@ struct lookup_table
    int        socket_type ;
 } ;
 
-extern struct intercept *di_init() ;
-extern struct intercept *si_init() ;
 
 static struct lookup_table intercept_lookup_table[] =
    {
@@ -56,11 +54,11 @@ static struct lookup_table intercept_lookup_table[] =
 /*
  * This variable has file scope for the benefit of the signal handler
  */
-static struct intercept *intp = NULL;
+static struct intercept_s *intp = NULL;
 
 
 
-PRIVATE initfunc find_initializer( int type )
+static initfunc find_initializer( int type )
 {
    struct lookup_table *ltp ;
 
@@ -106,44 +104,42 @@ void intercept( struct server *serp )
 /*
  * Create a socket and bind it to (INADDR_LOOPBACK,0)
  */
-PRIVATE int get_server_socket( struct intercept *ip )
+static int get_server_socket( struct intercept_s *ip )
 {
    struct service *sp = SERVER_SERVICE( INT_SERVER( ip ) ) ;
-#ifdef INET6
-   struct sockaddr_in6 *sinp = INT_LOCALADDR( ip ) ;
-#else
-   struct sockaddr_in *sinp = INT_LOCALADDR( ip ) ;
-#endif
+   union xsockaddr *sinp = INT_LOCALADDR( ip ) ;
    int sd ;
    int size ;
-   uint16_t *port ;
-   char *func = "get_server_socket" ;
+   const char *func = "get_server_socket" ;
 
-#ifdef INET6
-   if ( ( sd = socket( AF_INET6, SVC_SOCKET_TYPE( sp ), 0 ) ) == -1 )
+   if( SC_IPV6(SVC_CONF(sp)) ) {
+      struct addrinfo hint, *res = NULL;
+      memset(&hint, 0, sizeof(struct addrinfo));
+      hint.ai_family = AF_INET6;
+      hint.ai_flags = AI_NUMERICHOST;
+      sinp->sa_in6.sin6_family = AF_INET6;
+      sinp->sa_in6.sin6_port = 0;
+      if( getaddrinfo("::1", NULL, &hint, &res) != 0  )
+         int_fail( ip, "can't find ::1" );
+      if( res == NULL )
+         int_fail( ip, "no results for ::1" );
+      if( res->ai_family != AF_INET6 )
+         int_fail( ip, "non IPv6 result for ::1" );
+      memcpy(sinp, res->ai_addr, sizeof( struct sockaddr_in6 ));
+      freeaddrinfo(res);
+      size = sizeof(struct sockaddr_in6);
+   } else if( SC_IPV4(SVC_CONF(sp)) ) {
+      sinp->sa_in.sin_family = AF_INET;
+      sinp->sa_in.sin_port = 0;
+      sinp->sa_in.sin_addr.s_addr = inet_addr( "127.0.0.1" );
+      size = sizeof(struct sockaddr_in);
+   } else
+      int_fail( ip, "unknown socket family" );
+
+   if ( ( sd = socket( sinp->sa.sa_family, SVC_SOCKET_TYPE( sp ), SC_PROTOVAL(SVC_CONF(sp)) ) ) == -1 )
       int_fail( ip, "socket creation" ) ;
-   sinp->sin6_family = AF_INET6 ;
 
-   /* Initialize the address to the local loop back */
-   inet_pton(AF_INET6, "::1", &sinp->sin6_addr);
-
-   sinp->sin6_port = 0;
-   port = &(sinp->sin6_port);
-#else
-   if ( ( sd = socket( AF_INET, SVC_SOCKET_TYPE( sp ), 0 ) ) == -1 )
-      int_fail( ip, "socket creation" ) ;
-   
-   sinp->sin_family = AF_INET ;
-#ifdef INADDR_LOOPBACK
-   sinp->sin_addr.s_addr = htonl( INADDR_LOOPBACK ) ;
-#else
-   sinp->sin_addr.s_addr = inet_addr( "127.0.0.1" ) ;
-#endif
-   sinp->sin_port = 0 ;
-   port = &sinp->sin_port;
-#endif /* INET6 */
-
-   if ( bind( sd, SA( sinp ), sizeof( *sinp ) ) == -1 )
+   if ( bind( sd, SA( sinp ), size ) == -1 )
       int_fail( ip, "bind" ) ;
    
    size = sizeof( *sinp ) ;
@@ -152,7 +148,7 @@ PRIVATE int get_server_socket( struct intercept *ip )
    
    if ( debug.on )
       msg( LOG_DEBUG, func, "address = %s, port = %d",
-         xntoa( SA(sinp) ), ntohs( *port ) ) ;
+         xaddrname( sinp ), ntohs( xaddrport( sinp ) ) ) ;
       
    if ( ip->int_socket_type == SOCK_STREAM )
       (void) listen( sd, LISTEN_BACKLOG ) ;
@@ -161,7 +157,7 @@ PRIVATE int get_server_socket( struct intercept *ip )
 }
 
 
-PRIVATE void start_server( struct intercept *ip )
+static void start_server( struct intercept_s *ip )
 {
    struct server      *serp = INT_SERVER( ip ) ;
    struct service     *sp = SERVER_SERVICE( serp ) ;
@@ -197,9 +193,9 @@ PRIVATE void start_server( struct intercept *ip )
  *         OK          if the server died
  *         FAILED       otherwise
  */
-PRIVATE status_e wait_child( struct intercept *ip )
+static status_e wait_child( struct intercept_s *ip )
 {
-   char *func = "wait_child" ;
+   const char *func = "wait_child" ;
    int status ;
    status_e ret = FAILED;
    pid_t pid ;
@@ -224,7 +220,7 @@ PRIVATE status_e wait_child( struct intercept *ip )
       }
       else
       {
-         int u;
+         unsigned u;
 
          /* Ideally, this will never be executed */
          msg( LOG_ERR, func,
@@ -259,8 +255,7 @@ PRIVATE status_e wait_child( struct intercept *ip )
 }
 
 
-
-PRIVATE void terminate_server( struct intercept *ip )
+static void terminate_server( struct intercept_s *ip )
 {
    pid_t pid = SERVER_PID( INT_SERVER( intp ) ) ;
 
@@ -278,7 +273,7 @@ PRIVATE void terminate_server( struct intercept *ip )
 
 void int_sighandler( int sig )
 {
-   char *func = "int_sighandler" ;
+   const char *func = "int_sighandler" ;
 
    if ( debug.on )
       msg( LOG_DEBUG, func, "Received signal %s", sig_name( sig ) ) ;
