@@ -5,8 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: intcommon.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
-
 #include "config.h"
 #include <sys/types.h>
 #include <sys/time.h>
@@ -17,16 +15,17 @@ static char RCSid[] = "$Id: intcommon.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp
 #include <stdlib.h>
 #include <unistd.h>
 
-#include "int.h"
-#include "defs.h"
-#include "service.h"
-#include "server.h"
-#include "xconfig.h"
-#include "state.h"
-#include "log.h"
+#include "intcommon.h"
+#include "msg.h"
 #include "signals.h"
+#include "connection.h"
+#include "sconf.h"
+#include "state.h"
+#include "main.h"
+#include "xconfig.h"
 
-void int_fail( struct intercept *ip, char *lsyscall )
+
+void int_fail( const struct intercept_s *ip, const char *lsyscall )
 {
    msg( LOG_ERR, "fail", "%s failed: %m", lsyscall ) ;
    (*ip->int_ops->exit)() ;
@@ -39,7 +38,7 @@ void int_fail( struct intercept *ip, char *lsyscall )
  */
 int int_select( int max, fd_set *read_mask )
 {
-   char *func = "int_select" ;
+   const char *func = "int_select" ;
 
    for ( ;; )
    {
@@ -62,10 +61,10 @@ int int_select( int max, fd_set *read_mask )
 }
 
 
-void int_exit( struct intercept *ip )
+void int_exit( struct intercept_s *ip )
 {
    int status = SERVER_EXITSTATUS( INT_SERVER( ip ) ) ;
-   char *func = "int_exit" ;
+   const char *func = "int_exit" ;
 
    if ( debug.on )
    {
@@ -82,17 +81,17 @@ void int_exit( struct intercept *ip )
 /*
  * The ops vector must be installed before invoking this function
  */
-void int_init( struct intercept *ip, struct server *serp )
+void int_init( struct intercept_s *ip, struct server *serp )
 {
    unsigned u ;
-   char *func = "int_init" ;
+   const char *func = "int_init" ;
 
    /*
     * Sanity test
     */
    if ( SERVER_SERVICE( serp ) != SERVER_CONNSERVICE( serp ) )
    {
-      msg( LOG_ERR, "server service (%s) != connection service (%s)",
+      msg( LOG_ERR, func, "server service (%s) != connection service (%s)",
                            SVC_ID( SERVER_SERVICE( serp ) ),
                               SVC_ID( SERVER_CONNSERVICE( serp ) ) ) ;
       exit( 1 ) ;
@@ -138,42 +137,26 @@ void int_init( struct intercept *ip, struct server *serp )
 }
 
 
-
 /*
  * Make a new connection to the local server
  */
-#ifdef INET6
-channel_s *int_newconn( struct intercept *ip, 
-                        struct sockaddr_in6 *sinp, 
+channel_s *int_newconn( struct intercept_s *ip, 
+                        union xsockaddr *sinp,
                         int remote_socket )
-#else
-channel_s *int_newconn( struct intercept *ip, 
-                        struct sockaddr_in *sinp, 
-                        int remote_socket )
-#endif
 {
-   struct service         *sp         = SERVER_SERVICE( INT_SERVER( ip ) ) ;
-   int                  socket_type = SVC_SOCKET_TYPE( sp ) ;
-#ifdef INET6
-   struct sockaddr_in6   *local      = INT_LOCALADDR( ip ) ;
-#else
-   struct sockaddr_in   *local      = INT_LOCALADDR( ip ) ;
-#endif
-   char                   *sid         = SVC_ID( sp ) ;
+   struct service       *sp          = SERVER_SERVICE( INT_SERVER( ip ) ) ;
+   int                   socket_type = SVC_SOCKET_TYPE( sp ) ;
+   union xsockaddr      *local       = INT_LOCALADDR( ip ) ;
+   char                 *sid         = SVC_ID( sp ) ;
    channel_s            *chp ;
-   int                  sd ;
-   char                  *func = "int_newconn" ;
+   int                   sd ;
+   const char           *func = "int_newconn" ;
 
    /*
     * Get a socket and connect it to the local address
     *
-    * XXX:   should we use SC_PROTOVAL to explicitly specify the protocol ?
     */
-#ifdef INET6
-   if ( ( sd = socket( AF_INET6, socket_type, 0 ) ) == -1 )
-#else
-   if ( ( sd = socket( AF_INET, socket_type, 0 ) ) == -1 )
-#endif
+   if ( ( sd = socket( local->sa.sa_family, socket_type, SC_PROTOVAL(SVC_CONF(sp)) ) ) == -1 )
    {
       msg( LOG_ERR, func,"(intercepting %s) socket creation failed: %m", sid ) ;
       return( CHANNEL_NULL ) ;
@@ -220,15 +203,9 @@ channel_s *int_newconn( struct intercept *ip,
  * *addr_checked is set to TRUE of FALSE depending on whether there
  * is already a connection from the same IP address in the table.
  */
-#ifdef INET6
-channel_s *int_lookupconn( struct intercept *ip, 
-                           struct sockaddr_in6 *sinp, 
+channel_s *int_lookupconn( struct intercept_s *ip, 
+                           union xsockaddr *sinp,
                            bool_int *addr_checked )
-#else
-channel_s *int_lookupconn( struct intercept *ip, 
-                           struct sockaddr_in *sinp, 
-                           bool_int *addr_checked )
-#endif
 {
    unsigned      u ;
    pset_h      conntab = INT_CONNECTIONS( ip ) ;
@@ -239,21 +216,12 @@ channel_s *int_lookupconn( struct intercept *ip,
    {
       register channel_s *chp = CHP( pset_pointer( conntab, u ) ) ;
 
-#ifdef INET6
-      if ( memcmp(&chp->ch_from.sin6_addr, &sinp->sin6_addr, sizeof(sinp->sin6_addr)) == 0 )
+      if ( memcmp( &chp->ch_from, sinp, sizeof( *sinp ) ) == 0 )
       {
          *addr_checked = TRUE ;
-         if ( chp->ch_from.sin6_port == sinp->sin6_port )
+         if ( xaddrport(&chp->ch_from) == xaddrport(sinp) )
             return( chp ) ;
       }
-#else
-      if ( chp->ch_from.sin_addr.s_addr == sinp->sin_addr.s_addr )
-      {
-         *addr_checked = TRUE ;
-         if ( chp->ch_from.sin_port == sinp->sin_port )
-            return( chp ) ;
-      }
-#endif
    }
    return( CHANNEL_NULL ) ;
 }
