@@ -5,8 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: log.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
-
 #include "config.h"
 #include <stdlib.h>
 #include <sys/types.h>
@@ -15,55 +13,48 @@ static char RCSid[] = "$Id: log.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 #include <syslog.h>
 #include <time.h>
 #include <stdio.h>
+#include <stdarg.h>
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
 
-#include "sio.h"
 #include "str.h"
-
-#include "connection.h"
-#include "defs.h"
-#include "access.h"
+#include "log.h"
+#include "sconf.h"
 #include "sconst.h"
-#include "service.h"
-#include "server.h"
-#include "xconfig.h"
+#include "msg.h"
+
 
 #define LOGBUF_SIZE                  1024
 
-struct xaddr
+static char ipv6_ret[NI_MAXHOST];
+
+char *xaddrname(const union xsockaddr *inaddr)
 {
-   unsigned char a;
-   unsigned char b;
-   unsigned char c;
-   unsigned char d;
-};
-
-
-#ifdef INET6
-static char ipv6_ret[48];
-#endif
-char *xntoa(struct sockaddr *inaddr)
-{
-#ifdef INET6
-   bzero(ipv6_ret, sizeof(ipv6_ret));
-
-   inet_ntop(AF_INET6, &( ((struct sockaddr_in6 *)inaddr)->sin6_addr), ipv6_ret, sizeof(ipv6_ret)-1);
-
+   int len = 0;
+   if( inaddr->sa.sa_family == AF_INET )  len = sizeof(struct sockaddr_in);
+   if( inaddr->sa.sa_family == AF_INET6 ) len = sizeof(struct sockaddr_in6);
+   memset(ipv6_ret, 0, sizeof(ipv6_ret));
+   if( getnameinfo(&inaddr->sa, len, ipv6_ret, sizeof(ipv6_ret), NULL, 0, NI_NUMERICHOST) )
+      return "<unknown>";
    return ipv6_ret;
-#else
-   return inet_ntoa( ((struct sockaddr_in *)inaddr)->sin_addr );
-#endif
-
 }
 
+uint16_t xaddrport(const union xsockaddr *inaddr)
+{
+   if( inaddr->sa.sa_family == AF_INET )  return inaddr->sa_in.sin_port;
+   if( inaddr->sa.sa_family == AF_INET6 ) return inaddr->sa_in6.sin6_port;
+   return 0;
+}
 
-PRIVATE int log_common(mask_t *, char *, int, connection_s *) ;
+static int log_common(mask_t *, char *, int, const connection_s *) ;
 
 /*
  * This function writes log records of the form:
  *
  *      START: service [pid] [from_address]
  */
-void svc_log_success( struct service *sp, connection_s *cp, pid_t pid )
+void svc_log_success( struct service *sp, const connection_s *cp, pid_t pid )
 {
    char                    buf[ LOGBUF_SIZE ] ;
    int                     bufsize ;
@@ -92,7 +83,7 @@ void svc_log_success( struct service *sp, connection_s *cp, pid_t pid )
    len += cc ;
    bufsize -= cc ;
 
-   xlog_write( sp->svc_log, buf, len, XLOG_NOFLAGS ) ;
+   xlog_write( sp->svc_log, buf, len, XLOG_NO_ERRNO ) ;
 }
 
 
@@ -103,7 +94,7 @@ void svc_log_success( struct service *sp, connection_s *cp, pid_t pid )
  *
  */
 void svc_log_failure( struct service *sp, 
-                      connection_s *cp, 
+                      const connection_s *cp, 
                       access_e access_failure )
 {
    char                    buf[ LOGBUF_SIZE ] ;
@@ -129,31 +120,25 @@ void svc_log_failure( struct service *sp,
    len += cc ;
    bufsize -= cc ;
 
-   xlog_write( sp->svc_log, buf, len, XLOG_NOFLAGS ) ;
+   xlog_write( sp->svc_log, buf, len, XLOG_NO_ERRNO ) ;
 }
 
 
 
-PRIVATE int log_common( mask_t *logmask, 
+static int log_common( mask_t *logmask, 
                         char *buf, 
                         int bufsize, 
-                        connection_s *cp )
+                        const connection_s *cp )
 {
    int len = 0 ;
-   int cc ;
 
    if ( M_IS_SET( *logmask, LO_HOST ) )
-   {
-      cc = strx_nprint( &buf[ len ], bufsize, " from=%s", conn_addrstr( cp ) ) ;
-      len += cc ;
-      bufsize -= cc ;
-   }
+      len = strx_nprint( buf, bufsize, " from=%s", conn_addrstr( cp ) ) ;
    return( len ) ;
 }
 
 
-
-void svc_log_exit( struct service *sp, struct server *serp )
+void svc_log_exit( struct service *sp, const struct server *serp )
 {
    char                    buf[ LOGBUF_SIZE ] ;
    int                     bufsize ;
@@ -161,7 +146,7 @@ void svc_log_exit( struct service *sp, struct server *serp )
    int                     len ;
    int                     exit_status = SERVER_EXITSTATUS( serp ) ;
    struct service_config   *scp = SVC_CONF( sp ) ;
-   char                    *func = "log_exit" ;
+   const char                    *func = "log_exit" ;
 
    if ( ! SVC_LOGS_ON_EXIT( sp ) )
       return ;
@@ -219,12 +204,12 @@ void svc_log_exit( struct service *sp, struct server *serp )
       time_t current_time ;
 
       (void) time( &current_time ) ;
-      cc = strx_nprint( &buf[ len ], bufsize, " duration=%d(sec)", 
-                           current_time - SERVER_STARTTIME( serp ) ) ;
+      cc = strx_nprint( &buf[ len ], bufsize, " duration=%ld(sec)",
+         (long)(current_time - SERVER_STARTTIME( serp )) ) ;
       len += cc ;
       bufsize -= cc ;
    }
-   xlog_write( sp->svc_log, buf, len, XLOG_NOFLAGS ) ;
+   xlog_write( sp->svc_log, buf, len, XLOG_NO_ERRNO ) ;
 }
 
 
@@ -234,7 +219,7 @@ void svc_log_exit( struct service *sp, struct server *serp )
  * going through the proper channels (i.e. log_{success,failure} and log_exit)
  */
 /* VARARGS3 */
-void svc_logprint( struct service *sp, char *line_id, char *fmt, ...)
+void svc_logprint( struct service *sp, char *line_id, const char *fmt, ...)
 {
    char     buf[ LOGBUF_SIZE ] ;
    int      bufsize = sizeof( buf ) ;
@@ -249,6 +234,6 @@ void svc_logprint( struct service *sp, char *line_id, char *fmt, ...)
    va_start( ap, fmt ) ;
    cc = strx_nprintv( &buf[ len ], bufsize-len, fmt, ap ) ;
    va_end( ap ) ;
-   xlog_write( sp->svc_log, buf, len+cc, XLOG_NO_SIZECHECK ) ;
+   xlog_write( sp->svc_log, buf, len+cc, XLOG_NO_ERRNO | XLOG_NO_SIZECHECK ) ;
 }
 
