@@ -10,6 +10,9 @@
 
 #include <sys/socket.h>
 
+#include "defs.h"
+#include "access.h"
+
 /*
  * $Id: log.h,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $
  */
@@ -53,14 +56,16 @@ struct log
 #define log_filelog( lp )            (&(lp)->l_fl)
 #define log_syslog( lp )             (&(lp)->l_sl)
 
-status_e log_start() ;
-void log_end() ;
-
-char *xntoa(struct sockaddr *);
-void msg( int, char *, char *, ...);
-void out_of_memory(char *) ;
-void msg_suspend();
-void msg_resume();
-
+char *xaddrname(const union xsockaddr *inaddr);
+uint16_t xaddrport(const union xsockaddr *inaddr);
+void svc_log_success(struct service *sp, const connection_s *cp,pid_t pid);
+void svc_log_failure(struct service *sp, const connection_s *cp,access_e access_failure);
+void svc_log_exit(struct service *sp,const struct server *serp);
+void svc_logprint(struct service *sp,char *line_id,const char *fmt,...)
+#ifdef __GNUC__
+	__attribute__ ((format (printf, 3, 4)));
+#else
+	;
+#endif
 #endif   /* LOG_H */
 
