@@ -5,9 +5,8 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: main.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
+
 #include "config.h"
-char program_version[] = XINETD_VERSION ;
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -15,35 +14,47 @@ char program_version[] = XINETD_VERSION ;
 #include <errno.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <sys/ioctl.h>
 
+#include "main.h"
+#include "init.h"
+#include "msg.h"
+#include "internals.h"
+#include "signals.h"
+#include "service.h"
+#include "sconf.h"
+#include "xtimer.h"
 #include "sio.h"
 
-#include "options.h"
-
-#include "service.h"
-#include "state.h"
-#include "servicedefs.h"
-#include "signals.h"
-
-PRIVATE void main_loop(void);
-PRIVATE void find_bad_fd(void) ;
+static void main_loop(void);
+static void find_bad_fd(void) ;
 
 /*
  * The following are the only global variables of this program
  */
 struct program_state ps ;
 struct debug debug ;
+char program_version[] = XINETD_VERSION ;
+int signals_pending[2];
 
 /*
  * This is where the story starts...
  */
 int main( int argc, char *argv[] )
 {
-   char            *func = "main" ;
+   const char            *func = "main" ;
 
+   sio_init();
    init_daemon( argc, argv ) ;
    init_services() ;
 
+   /* Do the chdir after reading the config file.  Relative path names
+    * will work better.  
+    */
+   if (chdir("/") < 0) {
+      msg(LOG_ERR, func, "Can't chdir to /: %m");
+   }
+
    /* Print out all the options we're compiled with.  Makes support 
     * a tad easier.
     * Also, try to get them all into one syslog message for atomicity
@@ -68,19 +79,12 @@ int main( int argc, char *argv[] )
       ps.rws.available_services,
          ( ps.rws.available_services != 1 ) ? "s" : "" ) ;
 
-   if ( cc_option )
-#ifndef NO_TIMERS
-      enable_periodic_check( cc_option_arg ) ;
-#else
-      msg( LOG_WARNING, func, "-cc option not supported" ) ;
-#endif
-   
    /*
     * The reason for doing the setjmp here instead of in main_loop is
     * that setjmp is not guaranteed to restore register values which
     * can cause a problem for register variables
     */
-   if ( setjmp( ps.rws.env ) == 0 )
+   if ( sigsetjmp( ps.rws.env, 1 ) == 0 )
       ps.rws.env_is_valid = TRUE ;
 
    main_loop() ;
@@ -93,14 +97,20 @@ int main( int argc, char *argv[] )
 /*
  * What main_loop does:
  *
- *      check if any flags are set
  *      select on all active services
  *      for each socket where a request is pending
  *         try to start a server
  */
-PRIVATE void main_loop(void)
+static void main_loop(void)
 {
-   char            *func = "main_loop" ;
+   const char      *func = "main_loop" ;
+   struct timeval   tv, *tvptr = NULL;
+
+   FD_SET(signals_pending[0], &ps.rws.socket_mask);
+   if ( signals_pending[0] > ps.rws.mask_max )
+      ps.rws.mask_max = signals_pending[0] ;
+   if ( signals_pending[1] > ps.rws.mask_max )
+      ps.rws.mask_max = signals_pending[1] ;
 
    for ( ;; )
    {
@@ -108,37 +118,50 @@ PRIVATE void main_loop(void)
       int n_active ;
       unsigned u ;
 
-      check_flags() ;
-
-      if ( debug.on )
+      if ( debug.on ) 
          msg( LOG_DEBUG, func,
                "active_services = %d", ps.rws.active_services ) ;
 
-      if ( ps.rws.active_services == 0 )
-      {
-         signal_wait() ;
-         continue ;
+      /* get the next timer value, if there is one, and select for that time */
+      if( (tv.tv_sec = xtimer_nexttime()) >= 0 ) {
+         tv.tv_usec = 0;
+         tvptr = &tv;
+      } else {
+         tvptr = NULL;
+         /* If no timers and no services, wait until we get a signal */
+         if ( ps.rws.active_services == 0 ) {
+            signal_wait() ;
+            continue ;
+         }
       }
 
       read_mask = ps.rws.socket_mask ;
       n_active = select( ps.rws.mask_max+1, &read_mask,
-                        FD_SET_NULL, FD_SET_NULL, TIMEVAL_NULL ) ;
+                        FD_SET_NULL, FD_SET_NULL, tvptr ) ;
       if ( n_active == -1 )
       {
-         if ( errno == EINTR )
+         if ( errno == EINTR ) {
             continue ;
-         else if ( errno == EBADF )
+         } else if ( errno == EBADF )
             find_bad_fd() ;
-         else
-            msg( LOG_NOTICE, func, "select error %m" ) ;
          continue ;
       }
-      else if ( n_active == 0 )
+      else if ( n_active == 0 ) {
+         xtimer_poll();
          continue ;
+      }
 
       if ( debug.on )
          msg( LOG_DEBUG, func, "select returned %d", n_active ) ;
 
+      xtimer_poll();
+
+      if( FD_ISSET(signals_pending[0], &read_mask) ) {
+         check_pipe();
+            if ( --n_active == 0 )
+               continue ;
+      }
+
       for ( u = 0 ; u < pset_count( SERVICES( ps ) ) ; u++ )
       {
          struct service *sp ;
@@ -168,12 +191,12 @@ PRIVATE void main_loop(void)
  * When we identify such a bad fd, we remove it from the mask
  * and deactivate the service.
  */
-PRIVATE void find_bad_fd(void)
+static void find_bad_fd(void)
 {
    int fd ;
    struct stat st ;
    unsigned bad_fd_count = 0 ;
-   char *func = "find_bad_fd" ;
+   const char *func = "find_bad_fd" ;
 
    for ( fd = 0 ; fd < ps.ros.max_descriptors ; fd++ )
       if ( FD_ISSET( fd, &ps.rws.socket_mask ) && fstat( fd, &st ) == -1 )
@@ -223,7 +246,7 @@ void quit_program(void)
 {
    unsigned u ;
    struct service_config *scp = NULL;
-   char *func = "quit_program" ;
+   const char *func = "quit_program" ;
 
    for ( u = 0 ; u < pset_count( SERVICES( ps ) ) ; u++ ) {
       scp = SVC_CONF( SP(pset_pointer(SERVICES(ps), u)) );
@@ -240,8 +263,8 @@ void quit_program(void)
          svc_deactivate( SP( pset_pointer( SERVICES( ps ), u ) ) ) ;
    }
 
-   if( pid_file_option ) {
-      unlink(pid_file_option);
+   if ( ps.ros.pid_file ) {
+      unlink(ps.ros.pid_file);
    }
 
    msg( LOG_WARNING, func, "Exiting..." ) ;
