@@ -12,13 +12,17 @@
  * $Id: mask.h,v 1.2 1999/10/15 05:18:39 bbraun Exp $
  */
 
+#include "config.h"
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
 /*
- * Macros about masks
+ * Macros about masks - Note: 64 bits is needed because of attr.h
  */
 
 #define mask_t uint64_t
 
-#define MASK_EMPTY                  0
 #define MASK_NULL                   ((mask_t *)0)
 
 #define XMASK( v )                  ( (mask_t)1 << ( (v)-1 ) )
