@@ -5,21 +5,21 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: nvlists.c,v 1.2 1999/10/13 04:22:34 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <syslog.h>
 
-#include "defs.h"
+#include "nvlists.h"
 #include "sconf.h"
 
+
 /*
  * A NULL value for the name field marks the end of the table
  */
 
-struct name_value service_types[] =
+const struct name_value service_types[] =
    {
 #ifndef NO_RPC
       { "RPC",                        ST_RPC                  },
@@ -31,7 +31,8 @@ struct name_value service_types[] =
    } ;
 
 
-struct name_value service_flags[] =
+/* REUSE is only used for backward compatibility.  All services are now reuse */
+const struct name_value service_flags[] =
    {
       { "REUSE",                      SF_REUSE               },
       { "INTERCEPT",                  SF_INTERCEPT           },
@@ -39,15 +40,16 @@ struct name_value service_flags[] =
       { "IDONLY",                     SF_IDONLY              },
       { "NAMEINARGS",                 SF_NAMEINARGS          },
       { "NODELAY",                    SF_NODELAY             },
-      { "DISABLE",                    SF_DISABLE             },
       { "KEEPALIVE",                  SF_KEEPALIVE           },
       { "NOLIBWRAP",                  SF_NOLIBWRAP           },
       { "SENSOR",                     SF_SENSOR              },
+      { "IPv4",                       SF_IPV4                },
+      { "IPv6",                       SF_IPV6                },
       { CHAR_NULL,                    0                      }
    } ;
 
 
-struct name_value socket_types[] =
+const struct name_value socket_types[] =
    {
       { "stream",                     SOCK_STREAM           },
       { "dgram",                      SOCK_DGRAM            },
@@ -58,7 +60,7 @@ struct name_value socket_types[] =
    } ;
 
 
-struct name_value success_log_options[] =
+const struct name_value success_log_options[] =
    {
       { "HOST",        LO_HOST     },
       { "DURATION",    LO_DURATION },
@@ -69,7 +71,7 @@ struct name_value success_log_options[] =
    } ;
 
 
-struct name_value failure_log_options[] = 
+const struct name_value failure_log_options[] = 
    {
       { "HOST",        LO_HOST     },
       { "ATTEMPT",     LO_ATTEMPT  },
@@ -80,7 +82,7 @@ struct name_value failure_log_options[] =
 
 
 
-struct name_value syslog_facilities[] =
+const struct name_value syslog_facilities[] =
    {
       { "daemon",         LOG_DAEMON      },
       { "auth",           LOG_AUTH        },
@@ -101,7 +103,7 @@ struct name_value syslog_facilities[] =
    } ;
 
 
-struct name_value syslog_levels[] = 
+const struct name_value syslog_levels[] = 
    {
       { "emerg",         LOG_EMERG      },
       { "alert",         LOG_ALERT      },
