@@ -6,47 +6,37 @@
  */
 
 #include "config.h"
-#include "options.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "sio.h"
+
 #include "str.h"
+#include "sio.h"
+#include "options.h"
+#include "main.h"
+#include "util.h"
+#include "internals.h" /* for enable_periodic_check() */
 
 /*
  * $Id: options.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $
  */
 
 
-extern char program_version[];
 
-int d_option ;
-int f_option ;
-char * f_option_arg_1 ;
 int filelog_option ;
 char * filelog_option_arg_1 ;
 int syslog_option ;
 char * syslog_option_arg_1 ;
-int reuse_option ;
-int inet_compat ;
-int limit_option ;
-unsigned limit_option_arg_1 ;
-int loop_option ;
-unsigned loop_option_arg_1 ;
-char *pid_file_option=NULL ;
 int logprocs_option ;
 unsigned logprocs_option_arg_1 ;
-int shutdownprocs_option ;
-unsigned shutdownprocs_option_arg_1 ;
-int cc_option ;
-unsigned cc_option_arg_1 ;
 int stayalive_option=0;
 
 char *program_name ;
+static void usage(void);
 
 int opt_recognize( int argc, char *argv[] )
 {
-   int arg ;
+   int arg, arg_1 ;
 
    program_name = strrchr( argv[ 0 ], '/' ) ;
    program_name = ( program_name == NULL ) ? argv[ 0 ] : program_name + 1 ;
@@ -55,13 +45,12 @@ int opt_recognize( int argc, char *argv[] )
       if ( argv[ arg ][ 0 ] == '-' && argv[ arg ][ 1 ] != 0 )
       {
          if ( strcmp( &argv[ arg ][ 1 ], "d" ) == 0 ) 
-            d_option = 1 ;
+            debug.on = 1 ;
          else if ( strcmp( &argv[ arg ][ 1 ], "f" ) == 0 ) 
          {
             if ( ++arg == argc )
                usage() ;
-            f_option_arg_1 = ( argv[ arg ] ) ;
-            f_option = 1 ;
+            ps.ros.config_file = argv[ arg ];
          }
          else if ( strcmp( &argv[ arg ][ 1 ], "filelog" ) == 0 ) 
          {
@@ -78,25 +67,19 @@ int opt_recognize( int argc, char *argv[] )
             syslog_option = 1 ;
          }
          else if ( strcmp( &argv[ arg ][ 1 ], "reuse" ) == 0 ) 
-            reuse_option = 1 ;
+            ; /* This is now a null option, kept for compatibility */
          else if ( strcmp( &argv[ arg ][ 1 ], "limit" ) == 0 ) 
          {
             if ( ++arg == argc )
                usage() ;
-            limit_option_arg_1 = atoi( argv[ arg ] ) ;
-            limit_option = 1 ;
-         }
-         else if ( strcmp( &argv[ arg ][ 1 ], "loop" ) == 0 ) 
-         {
-            if ( ++arg == argc )
+            if ( parse_int( argv[ arg ], 10, NUL, &arg_1 ) || arg_1 < 0 )
                usage() ;
-            loop_option_arg_1 = atoi( argv[ arg ] ) ;
-            loop_option = 1 ;
+            ps.ros.process_limit = arg_1 ;
          }
          else if ( strcmp( &argv[ arg ][ 1 ], "pidfile" ) == 0 ){
                             if( ++arg ==argc )
                                 usage () ;
-                            pid_file_option = (char *)new_string( argv[arg] );
+                            ps.ros.pid_file = (char *)new_string( argv[arg] );
                         }
          else if ( strcmp( &argv[ arg ][ 1 ], "stayalive" )==0) {
             stayalive_option = 1;
@@ -105,27 +88,25 @@ int opt_recognize( int argc, char *argv[] )
          {
             if ( ++arg == argc )
                usage() ;
-            logprocs_option_arg_1 = atoi( argv[ arg ] ) ;
+            if ( parse_int( argv[ arg ], 10, NUL, &arg_1 ) || arg_1 < 0 )
+               usage() ;
+            logprocs_option_arg_1 = arg_1 ;
             logprocs_option = 1 ;
          }
          else if ( strcmp( &argv[ arg ][ 1 ], "shutdownprocs" ) == 0 ) 
          {
             if ( ++arg == argc )
                usage() ;
-            shutdownprocs_option_arg_1 = atoi( argv[ arg ] ) ;
-            shutdownprocs_option = 1 ;
+            Sprint(2, "The shutdownprocs option has been deprecated.\n");
          }
          else if ( strcmp( &argv[ arg ][ 1 ], "cc" ) == 0 ) 
          {
             if ( ++arg == argc )
                usage() ;
-            cc_option_arg_1 = atoi( argv[ arg ] ) ;
-            cc_option = 1 ;
-         }
-         else if ( strcmp( &argv[ arg ][ 1 ], "inet-compat" ) == 0 )
-         {
-            reuse_option = 1;
-            inet_compat = 1;
+            if ( parse_int( argv[ arg ], 10, NUL, &arg_1 ) || arg_1 < 0 )
+               usage() ;
+            ps.ros.cc_interval = arg_1;
+            enable_periodic_check( arg_1 ) ;
          }
          else if ( strcmp( &argv[ arg ][ 1 ], "version" ) == 0 )
          {
@@ -155,8 +136,8 @@ int opt_recognize( int argc, char *argv[] )
    return( arg ) ;
 }
 
-void usage()
+static void usage(void)
 {
-   Sprint( 2, "Usage: %s [-d] [-f config_file] [-filelog filename] [-syslog facility] [-reuse] [-limit proc_limit] [-loop loop_rate] [-pidfile filaneme] [-logprocs limit] [-shutdownprocs limit] [-cc interval]\n", program_name ) ;
+   Sprint( 2, "Usage: %s [-d] [-f config_file] [-filelog filename] [-syslog facility] [-reuse] [-limit proc_limit] [-pidfile filaneme] [-logprocs limit] [-shutdownprocs limit] [-cc interval]\n", program_name ) ;
    exit( 1 ) ;
 }
