@@ -5,7 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: parse.c,v 1.4 1999/10/18 05:21:14 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
@@ -21,23 +20,18 @@ static char RCSid[] = "$Id: parse.c,v 1.4 1999/10/18 05:21:14 bbraun Exp $" ;
 #include <pwd.h>
 
 #include "str.h"
-#include "pset.h"
-#include "sio.h"
-
-#include "defs.h"
-#include "sconf.h"
-#include "conf.h"
-#include "attr.h"
 #include "parse.h"
-#include "addr.h"
 #include "parsers.h"
-#include "service.h"
+#include "msg.h"
+#include "parsesup.h"
+#include "addr.h"
+#include "includedir.h"
+
 
 #ifndef NAME_MAX
 #define NAME_MAX 255
 #endif
 
-void handle_includedir( char *, struct configuration *);
 
 /*
  * A NULL value for the name field marks the end of the table
@@ -103,9 +97,7 @@ static struct attribute service_attributes[] =
 #ifdef RLIMIT_STACK
    { "rlimit_stack",   A_RLIMIT_STACK,   1,  rlim_stack_parser      },
 #endif
-#ifdef INET6
    { "v6only",         A_V6ONLY,         1,  v6only_parser          },
-#endif
    { "deny_time",      A_DENY_TIME,      1,  deny_time_parser       },
    { "umask",          A_UMASK,          1,  umask_parser           },
    { NULL,             A_NONE,          -1,  NULL                   }
@@ -133,24 +125,21 @@ static struct attribute default_attributes[] =
 #ifdef HAVE_LOADAVG
       { "max_load",        A_MAX_LOAD,        1,   max_load_parser       },
 #endif
-#ifdef INET6
       { "v6only",          A_V6ONLY,         1,    v6only_parser         },
-#endif
       { "umask",           A_UMASK,          1,    umask_parser          },
       { NULL,              A_NONE,           0,    NULL                  }
    } ;
 
 
 #define MODIFIABLE( ap )       ( (ap)->a_nvalues == -2 )
-#define VAR_VALUES( ap )         ( (ap)->a_nvalues < 0 )
 #define FIXED_VALUES( ap )         ( (ap)->a_nvalues > 0 )
 
 int line_count ;
 
-PRIVATE void get_service_entry( int fd, pset_h, char *, 
+static void get_service_entry( int fd, pset_h, char *, 
 	struct service_config * );
-PRIVATE entry_e  find_next_entry(int , char **) ;
-PRIVATE status_e parse_entry(entry_e, int, struct service_config *,
+static entry_e  find_next_entry(int , char **) ;
+static status_e parse_entry(entry_e, int, struct service_config *,
 	struct service_config *) ;
 
 
@@ -168,7 +157,7 @@ char *attr_name_lookup( int id )
 }
 
 
-void parse_end()
+void parse_end(void)
 {
    endprotoent() ;
    endpwent() ;
@@ -205,7 +194,7 @@ void parse_conf_file( int fd, struct configuration *confp )
    struct service_config   *default_config   = CNF_DEFAULTS( confp ) ;
    boolean_e                found_defaults   = NO ;
    struct service_config    default_default_config ;
-   char                    *func      = "parse_conf_file" ;
+   const char              *func      = "parse_conf_file" ;
    int                      incfd;
 
    line_count = 0 ;
@@ -221,7 +210,6 @@ void parse_conf_file( int fd, struct configuration *confp )
        * will point to malloc'ed memory
        */
       entry_type = find_next_entry( fd, &service_name ) ;
-
       switch ( entry_type )
       {
       case INCLUDE_ENTRY:
@@ -275,14 +263,14 @@ void parse_conf_file( int fd, struct configuration *confp )
  *
  * followed by a line containing only the ENTRY_BEGIN character
  */
-PRIVATE entry_e find_next_entry( int fd, char **snamep )
+static entry_e find_next_entry( int fd, char **snamep )
 {
    char           *p ;
    str_h           strp ;
    char           *sname = NULL;
    entry_e         entry_type=0;
    char           *line = next_line( fd ) ;
-   char           *func = "find_next_entry" ;
+   const char     *func = "find_next_entry" ;
 
    if ( line == CHAR_NULL )
       return( NO_ENTRY ) ;
@@ -379,14 +367,14 @@ PRIVATE entry_e find_next_entry( int fd, char **snamep )
  *      2. Determine service id
  *      3. Insert entry in table
  */
-PRIVATE void get_service_entry( int fd, 
+static void get_service_entry( int fd, 
                                 pset_h sconfs,
                                 char *name,
                                 struct service_config *defaults )
 {
    struct service_config   *scp ;
    unsigned                 u ;
-   char                    *func = "get_service_entry" ;
+   const char              *func = "get_service_entry" ;
 
    scp = sc_alloc( name ) ;
    if ( scp == NULL )
@@ -398,6 +386,7 @@ PRIVATE void get_service_entry( int fd,
 
    if ( parse_entry( SERVICE_ENTRY, fd, scp, defaults ) == FAILED )
    {
+      free( name );
       sc_free( scp ) ;
       skip_entry( fd ) ;
       return ;
@@ -412,6 +401,7 @@ PRIVATE void get_service_entry( int fd,
       else
       {
          out_of_memory( func ) ;
+         free( name );
          sc_free( scp ) ;
          return ;
       }
@@ -426,11 +416,11 @@ PRIVATE void get_service_entry( int fd,
       {
          if( (SCP(pset_pointer(sconfs, u))->sc_bind_addr != NULL) &&
             (scp->sc_bind_addr != NULL) && 
-          (memcmp(&SCP(pset_pointer(sconfs, u))->sc_bind_addr, &scp->sc_bind_addr, sizeof( scp->sc_bind_addr ) ) != 0) )
+          (memcmp(SCP(pset_pointer(sconfs, u))->sc_bind_addr, scp->sc_bind_addr, sizeof( *scp->sc_bind_addr ) ) != 0) )
          {
             continue;
-         } else if( SC_SDISABLE( SCP(pset_pointer(sconfs, u)) ) || 
-            SC_SDISABLE(scp) ) {
+         } else if( SC_IS_DISABLED( SCP(pset_pointer(sconfs, u)) ) || 
+            SC_IS_DISABLED(scp) ) {
 
             /* Allow multiple configs, as long as all but one are
              * disabled.
@@ -461,12 +451,10 @@ PRIVATE void get_service_entry( int fd,
  *      no_access
  *      passenv
  */
-PRIVATE void fill_attribute( unsigned attr_id, 
+static void fill_attribute( unsigned attr_id, 
                              struct service_config *scp, 
                              struct service_config *def )
 {
-   status_e copy_pset() ;
-
    switch ( attr_id )
    {
       case A_LOG_ON_SUCCESS:
@@ -501,11 +489,11 @@ PRIVATE void fill_attribute( unsigned attr_id,
 /*
  * Find the attribute with the specified name
  */
-PRIVATE struct attribute *attr_lookup( struct attribute attr_array[], 
+static struct attribute *attr_lookup( struct attribute attr_array[], 
                                        char *attr_name )
 {
    struct attribute *ap ;
-   char *func = "attr_lookup" ;
+   const char *func = "attr_lookup" ;
 
    for ( ap = &attr_array[ 0 ] ; ap->a_name ; ap++ )
       if ( EQ( attr_name, ap->a_name ) )
@@ -525,7 +513,7 @@ PRIVATE struct attribute *attr_lookup( struct attribute attr_array[],
  *
  * Invoke appropriate parser
  */
-PRIVATE void identify_attribute( entry_e entry_type, 
+static void identify_attribute( entry_e entry_type, 
                                  struct service_config *scp, 
                                  struct service_config *defaults,
                                  char *attr_name, 
@@ -533,7 +521,7 @@ PRIVATE void identify_attribute( entry_e entry_type,
                                  pset_h attr_values )
 {
    struct attribute   *ap ;
-   char               *func = "identify_attribute" ;
+   const char         *func = "identify_attribute" ;
 
    if ( entry_type == SERVICE_ENTRY )
       ap = attr_lookup( service_attributes, attr_name ) ;
@@ -586,7 +574,15 @@ PRIVATE void identify_attribute( entry_e entry_type,
    }
 
    if ( (*ap->a_parser)( attr_values, scp, op ) == OK )
-      SC_SPECIFY( scp, ap->a_id ) ;
+   {
+	SC_SPECIFY( scp, ap->a_id ) ;
+   }
+   else
+   {
+      parsemsg( LOG_ERR, func,
+         "Error parsing attribute %s - DISABLING SERVICE", attr_name ) ;
+      SC_DISABLE( scp );
+   }
 }
 
 
@@ -594,7 +590,7 @@ PRIVATE void identify_attribute( entry_e entry_type,
  * Read the entry line-by-line and add the information in scp
  * Use defaults to initialize modifiable entry fields.
  */
-PRIVATE status_e parse_entry( entry_e entry_type, 
+static status_e parse_entry( entry_e entry_type, 
                               int fd, 
                               struct service_config *scp, 
                               struct service_config *defaults )
@@ -603,7 +599,7 @@ PRIVATE status_e parse_entry( entry_e entry_type,
    char               *line ;
    char               *attr_name ;
    enum assign_op      op ;
-   char               *func = "get_attributes" ;
+   const char         *func = "get_attributes" ;
 
    if ( ! attr_values && ( attr_values = pset_create( 10, 10 ) ) == NULL )
    {
