@@ -5,7 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: parsers.c,v 1.5 1999/10/14 23:46:27 bbraun Exp $" ;
 
 #include "config.h"
 #include <ctype.h>
@@ -15,11 +14,13 @@ static char RCSid[] = "$Id: parsers.c,v 1.5 1999/10/14 23:46:27 bbraun Exp $" ;
 #include <pwd.h>
 #include <grp.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <limits.h> /* For LONG_MIN and LONG_MAX */
+#include <errno.h>
 #ifdef HAVE_LOADAVG
 #include <stdio.h>
 #endif
@@ -28,13 +29,15 @@ static char RCSid[] = "$Id: parsers.c,v 1.5 1999/10/14 23:46:27 bbraun Exp $" ;
 #define X_OK 1
 #endif
 
-#include "pset.h"
 #include "str.h"
-#include "defs.h"
-#include "parse.h"
-#include "sconf.h"
+#include "parsers.h"
+#include "msg.h"
+#include "nvlists.h"
+#include "env.h"
 #include "xconfig.h"
 #include "addr.h"
+#include "libportable.h"
+#include "timex.h"
 
 #define NEW_SET( set, v1, v2 )                 \
    if ( (set) == NULL && \
@@ -44,15 +47,7 @@ static char RCSid[] = "$Id: parsers.c,v 1.5 1999/10/14 23:46:27 bbraun Exp $" ;
       return( FAILED ) ;   \
    }
 
-extern struct name_value success_log_options[] ;
-extern struct name_value failure_log_options[] ;
-extern struct name_value service_types[] ;
-extern struct name_value socket_types[] ;
-extern struct name_value service_flags[] ;
-extern struct name_value syslog_facilities[] ;
-extern struct name_value syslog_levels[] ;
 
-extern env_h std_env ;
 
 
 /*
@@ -60,19 +55,19 @@ extern env_h std_env ;
  * them to "*maskp" (apply means add or remove depending on "op")
  * "description" describes the type of flags.
  */
-PRIVATE status_e parse_value_list( pset_h values, 
+static status_e parse_value_list( pset_h values, 
                                    mask_t *maskp, 
-                                   struct name_value list[], 
+                                   const struct name_value list[], 
                                    enum assign_op op, 
-                                   char *description )
+                                   const char *description )
 {
    unsigned            u ;
-   struct name_value  *nvp ;
-   char               *func = "parse_value_list" ;
+   const struct name_value  *nvp ;
+   const char               *func = "parse_value_list" ;
 
    for ( u=0; u<pset_count( values ); u++ )
    {
-      char *name = (char *) pset_pointer( values, u ) ;
+      const char *name = (char *) pset_pointer( values, u ) ;
 
       nvp = nv_find_value( list, name ) ;
       if ( nvp != NULL )
@@ -107,13 +102,13 @@ status_e flags_parser( pset_h values,
 }
 
 
-status_e socket_type_parser( pset_h values, 
+status_e socket_type_parser( const pset_h values, 
                              struct service_config *scp, 
                              enum assign_op op )
 {
-   struct name_value *nvp ;
-   char *type = (char *) pset_pointer( values, 0 ) ;
-   char *func = "socket_type_parser" ;
+   const struct name_value *nvp ;
+   const char *type = (char *) pset_pointer( values, 0 ) ;
+   const char *func = "socket_type_parser" ;
 
    nvp = nv_find_value( socket_types, type ) ;
    if ( nvp != NULL )
@@ -135,22 +130,31 @@ status_e rpc_version_parser( pset_h values,
 {
    struct rpc_data *rdp = SC_RPCDATA( scp ) ;
    char *version = (char *) pset_pointer( values, 0 ) ;
+   int min_version=0, max_version=0;
    char *p = strchr( version, '-' ) ;
-   char *func = "rpc_version_parser" ;
+   const char *func = "rpc_version_parser" ;
 
    if ( p == NULL )
-      rdp->rd_min_version = rdp->rd_max_version = atoi( version ) ;
+   {
+      if ( parse_base10(version, &min_version) )
+         max_version = min_version - 1;
+      else
+         max_version = min_version;
+   }
    else
    {
       *p = NUL ;
-      rdp->rd_min_version = atoi( version ) ;
-      rdp->rd_max_version = atoi( p+1 ) ;
-      if ( rdp->rd_min_version > rdp->rd_max_version )
-      {
-         parsemsg( LOG_ERR, func, "bad version range: %s", version ) ;
-         return( FAILED ) ;
-      }
+      if ( parse_base10(version, &min_version) ||
+           parse_base10(p+1, &max_version) )
+         max_version = min_version - 1;
    }
+   if ( min_version > max_version )
+   {
+      parsemsg( LOG_ERR, func, "bad version range: %s", version ) ;
+      return( FAILED ) ;
+   }
+   rdp->rd_min_version = min_version;
+   rdp->rd_max_version = max_version;
    return( OK ) ;
 }
 
@@ -159,8 +163,14 @@ status_e rpc_number_parser( pset_h values,
                             struct service_config *scp, 
                             enum assign_op op )
 {
-   SC_RPCDATA( scp )->rd_program_number = 
-                           atoi( (char *) pset_pointer( values, 0 ) ) ;
+   int num;
+
+   if ( parse_base10((char *) pset_pointer( values, 0 ), &num) ) {
+      parsemsg(LOG_ERR, "rpc_number_parser", "Error parsing: %s", 
+         (char *)pset_pointer( values, 0 ));
+      return( FAILED );
+   }
+   SC_RPCDATA( scp )->rd_program_number = num;
    return( OK ) ;
 }
 
@@ -171,7 +181,7 @@ status_e protocol_parser( pset_h values,
 {
    char *proto_name = (char *) pset_pointer( values, 0 ) ;
    struct protoent *pep ;
-   char *func = "protocol_parser" ;
+   const char *func = "protocol_parser" ;
 
    if ( ( pep = getprotobyname( proto_name ) ) == NULL )
    {
@@ -196,7 +206,7 @@ status_e wait_parser( pset_h values,
                       enum assign_op op )
 {
    char *val = (char *) pset_pointer( values, 0 ) ;
-   char *func = "wait_parser" ;
+   const char *func = "wait_parser" ;
 
    if ( EQ( val, "yes" ) )
       scp->sc_wait = YES ;
@@ -214,7 +224,7 @@ status_e user_parser( pset_h values,
 {
    char *user = (char *) pset_pointer( values, 0 ) ;
    struct passwd *pw ;
-   char *func = "user_parser" ;
+   const char *func = "user_parser" ;
 
    pw = getpwnam( user ) ;
    if ( pw == NULL )
@@ -222,7 +232,7 @@ status_e user_parser( pset_h values,
       parsemsg( LOG_ERR, func, "Unknown user: %s", user ) ;
       return( FAILED ) ;
    }
-   
+   str_fill( pw->pw_passwd, ' ' );
    scp->sc_uid = pw->pw_uid ;
    scp->sc_user_gid = pw->pw_gid ;
    return( OK ) ;
@@ -233,14 +243,14 @@ status_e group_parser( pset_h values,
                        struct service_config *scp, 
                        enum assign_op op )
 {
-   char *group = (char *) pset_pointer( values, 0 ) ;
+   char *group_ptr = (char *) pset_pointer( values, 0 ) ;
    struct group *grp ;
-   char *func = "group_parser" ;
+   const char *func = "group_parser" ;
 
-   grp = getgrnam( group ) ;
+   grp = getgrnam( group_ptr ) ;
    if ( grp == NULL )
    {
-      parsemsg( LOG_ERR, func, "Unknown group: %s", group ) ;
+      parsemsg( LOG_ERR, func, "Unknown group: %s", group_ptr ) ;
       return( FAILED ) ;
    }
    
@@ -256,9 +266,9 @@ status_e svcdisable_parser( pset_h values,
    char *val = (char *) pset_pointer( values, 0 ) ;
    
    if( EQ( val, "yes" ) )
-      M_SET( scp->sc_xflags, SF_DISABLE ) ;
+      SC_DISABLE( scp );
    else if( EQ( val, "no" ) ) 
-      M_CLEAR( scp->sc_xflags, SF_DISABLE ) ;
+      SC_ENABLE( scp );
 
    return( OK );
 }
@@ -269,7 +279,7 @@ status_e groups_parser( pset_h values,
                         enum assign_op op )
 {
    char *val = (char *) pset_pointer( values, 0 ) ;
-   char *func = "groups_parser" ;
+   const char *func = "groups_parser" ;
 
    if ( EQ( val, "yes" ) )
       scp->sc_groups = YES ;
@@ -282,13 +292,12 @@ status_e groups_parser( pset_h values,
 }
 
 
-#ifdef INET6
 status_e v6only_parser( pset_h values, 
                         struct service_config *scp, 
                         enum assign_op op )
 {
    char *val = (char *) pset_pointer( values, 0 );
-   char *func = "v6only_parser" ;
+   const char *func = "v6only_parser" ;
 
    if ( EQ( val, "yes" ) )
       scp->sc_v6only = YES;
@@ -298,7 +307,6 @@ status_e v6only_parser( pset_h values,
       parsemsg( LOG_ERR, func, "Bad value for v6only: %s", val );
    return( OK );
 }
-#endif
 
 
 status_e server_parser( pset_h values, 
@@ -306,13 +314,23 @@ status_e server_parser( pset_h values,
                         enum assign_op op )
 {
    char *server = (char *) pset_pointer( values, 0 ) ;
-   char *func = "server_parser" ;
+   const char *func = "server_parser" ;
+   struct stat sb;
 
+   /* 
+    * Access is used so that the real user ID permissions
+    * are checked.
+    */
    if ( access( server, X_OK ) == -1 )
    {
       parsemsg( LOG_ERR, func, "Server %s is not executable", server ) ;
       return( FAILED ) ;
    }
+   if (stat(server, &sb) == -1)
+   {
+      parsemsg( LOG_ERR, func, "Unable to stat: %s.", server ) ;
+      return( FAILED ) ;
+   }
 
    scp->sc_server = new_string( server ) ;
    if ( scp->sc_server == NULL )
@@ -333,7 +351,7 @@ status_e server_args_parser( pset_h values,
    unsigned i ;
    unsigned argv_index ;
    unsigned n_args = pset_count( values ) ;
-   char *func = "server_args_parser" ;
+   const char *func = "server_args_parser" ;
 
    /*
     * Create the argv for a future exec call
@@ -387,17 +405,17 @@ status_e instances_parser( pset_h values,
                            enum assign_op op )
 {
    char *instances = (char *) pset_pointer( values, 0 ) ;
-   char *func = "instances_parser" ;
+   const char *func = "instances_parser" ;
 
    if ( EQ( instances, "UNLIMITED" ) )
       scp->sc_instances = UNLIMITED ;
    else
    {
-      scp->sc_instances = atoi( instances ) ;
-      if ( scp->sc_instances < 0 )
+      if ( parse_base10(instances, &scp->sc_instances) ||
+           scp->sc_instances < 0 )
       {
          parsemsg( LOG_ERR, func,
-            "Number of instances is negative: %s", instances ) ;
+            "Number of instances is invalid: %s", instances ) ;
          return( FAILED ) ;
       }
    }
@@ -410,16 +428,16 @@ status_e per_source_parser( pset_h values,
                             enum assign_op op )
 {
    char *per_source = (char *) pset_pointer( values, 0 ) ;
-   char *func = "per_source_parser" ;
+   const char *func = "per_source_parser" ;
 
    if ( EQ( per_source, "UNLIMITED" ) )
       scp->sc_per_source = UNLIMITED;
    else
    {
-      scp->sc_per_source = atoi( per_source ) ;
-      if ( scp->sc_per_source < 0 )
+      if ( parse_base10(per_source, &scp->sc_per_source) ||
+           scp->sc_per_source < 0 )
       {
-         parsemsg( LOG_ERR, func, "Number of per source instances is negative: %s", per_source ) ;
+         parsemsg( LOG_ERR, func, "Number of per source instances is invalid: %s", per_source ) ;
          return( FAILED );
       }
    }
@@ -433,33 +451,29 @@ status_e cps_parser( pset_h values,
 {
    char *cps = (char *) pset_pointer(values, 0);
    char *waittime = (char *) pset_pointer(values, 1);
-#ifdef NO_TIMERS
-   parsemsg(LOG_ERR, "cps_parser", "Timers not supported on this OS, cps directive will be ignored");
-#endif
+   int waittime_int, conn_max;
 
    if( cps == NULL || waittime == NULL ) {
-      parsemsg(LOG_ERR, "cps_parser", "NULL options specified in cps\n");
+      parsemsg(LOG_ERR, "cps_parser", "NULL options specified in cps");
       return( FAILED );
    }
-   scp->sc_time_conn_max = strtol(cps, NULL, 10);
-   if( scp->sc_time_conn_max == LONG_MIN || 
-      scp->sc_time_conn_max == LONG_MAX ) {
-
-      parsemsg(LOG_ERR, "cps_parser", "cps arguement not a number\n");
+   if( parse_base10(cps, &conn_max) ) {
+      parsemsg(LOG_ERR, "cps_parser", "cps argument not a number");
       scp->sc_time_conn_max = 0;
       scp->sc_time_wait = 0;
       return(FAILED);
    }
-   scp->sc_time_wait = strtol(waittime, NULL, 10);
-   if( scp->sc_time_wait == LONG_MIN || scp->sc_time_wait == LONG_MAX ) {
-      parsemsg(LOG_ERR, "cps_parser", "cps time arguement not a number\n");
+   if( parse_base10(waittime, &waittime_int) ) {
+      parsemsg(LOG_ERR, "cps_parser", "cps time argument not a number");
       scp->sc_time_conn_max = 0;
       scp->sc_time_wait = 0;
       return(FAILED);
    }
+   scp->sc_time_wait = waittime_int;
+   scp->sc_time_conn_max = conn_max;
 
    if( scp->sc_time_conn_max < 0 || scp->sc_time_wait < 0 ) {
-      parsemsg(LOG_ERR, "cps_parser", "cps arguements invalid\n");
+      parsemsg(LOG_ERR, "cps_parser", "cps arguments invalid");
       scp->sc_time_conn_max = 0;
       scp->sc_time_wait = 0;
       return(FAILED);
@@ -472,7 +486,7 @@ status_e id_parser( pset_h values,
                     struct service_config *scp, 
                     enum assign_op op )
 {
-   char *func = "id_parser" ;
+   const char *func = "id_parser" ;
 
    scp->sc_id = new_string( (char *) pset_pointer( values, 0 ) ) ;
    if ( scp->sc_id != NULL )
@@ -490,23 +504,24 @@ status_e port_parser( pset_h values,
                       struct service_config *scp, 
                       enum assign_op op )
 {
-   unsigned long port = atoi( (char *) pset_pointer( values, 0 ) ) ;
-   char *func = "port_parser" ;
+   int port;
+   const char *func = "port_parser" ;
 
-   if ( port >= PORT_MAX )
+   if ( parse_base10((char *) pset_pointer( values, 0 ), &port) ||
+        port < 0 || port >= PORT_MAX )
    {
-      parsemsg( LOG_ERR, func, "port number exceeds %d", PORT_MAX-1 ) ;
+      parsemsg( LOG_ERR, func, "port number is invalid" ) ;
       return( FAILED ) ;
    }
-   scp->sc_port = port ;
+   scp->sc_port = (uint16_t)port ;
    return( OK ) ;
 }
 
 
-PRIVATE status_e add_new_string( pset_h set, char *str )
+static status_e add_new_string( pset_h set, char *str )
 {
    char *p = new_string( str ) ;
-   char *func = "add_new_string" ;
+   const char *func = "add_new_string" ;
 
    if ( p == NULL )
    {
@@ -528,7 +543,7 @@ status_e env_parser( pset_h values,
                      enum assign_op op )
 {
    unsigned u ;
-   char *func = "env_parser" ;
+   const char *func = "env_parser" ;
 
    if ( op == MINUS_EQ )
    {
@@ -539,7 +554,7 @@ status_e env_parser( pset_h values,
 
    NEW_SET( scp->sc_env_var_defs, 5, 5 ) ;
 
-   if ( op == SET_EQ && pset_count( scp->sc_env_var_defs ) > 0 )
+   if ( op == SET_EQ && pset_count( scp->sc_env_var_defs ) != 0 )
    {
       pset_apply( scp->sc_env_var_defs, free, NULL ) ;
       pset_clear( scp->sc_env_var_defs ) ;
@@ -571,7 +586,7 @@ status_e passenv_parser( pset_h values,
 {
    pset_h var_set ;
    unsigned u ;
-   char *func = "passenv_parser" ;
+   const char *func = "passenv_parser" ;
 
    NEW_SET( scp->sc_pass_env_vars, 0, 0 ) ;
 
@@ -630,7 +645,7 @@ status_e disabled_parser( pset_h values,
                           enum assign_op op )
 {
    unsigned u ;
-   char *func = "disabled_parser" ;
+   const char *func = "disabled_parser" ;
 
    NEW_SET( scp->sc_disabled, pset_count( values ), 0 ) ;
    
@@ -650,7 +665,7 @@ status_e enabled_parser( pset_h values,
                          enum assign_op op )
 {
    unsigned u ;
-   char *func = "enabled_parser" ;
+   const char *func = "enabled_parser" ;
 
    NEW_SET( scp->sc_enabled, pset_count( values ), 0 ) ;
    
@@ -668,31 +683,52 @@ status_e enabled_parser( pset_h values,
  * Interpret a number of the form: <num>[m|M|k|K]
  * m and M mean megabytes, k and K mean kilobytes, nothing means bytes
  */
-PRIVATE unsigned get_limit( char *limitstr )
+static int get_limit( char *limit_str, rlim_t *res )
 {
-   int multiplier ;
-   char *p = limitstr + strlen( limitstr ) - 1;
-   while ( isspace( *p ) )
-      p --;
+   int limit_int;
+   int multiplier;
+   char *p;
 
-   if (*p == 'k' || *p == 'K')
+   if (*limit_str == NUL) {
+      *res = 0;
+      return -1;
+   }
+
+   p = limit_str + strlen( limit_str ) - 1;
+   while ( p > limit_str && isspace( *p ) )
+      p--;
+
+   if (*p == 'k' || *p == 'K') {
+      *p = NUL;
       multiplier = 1024;
-   else if (*p == 'm' || *p == 'M')
+   } else if (*p == 'm' || *p == 'M') {
+      *p = NUL;
       multiplier = 1024 * 1024;
-   else
+   } else
       multiplier = 1;
 
-   return( (unsigned) atoi( limitstr ) * multiplier ) ;
+   if (parse_base10(limit_str, &limit_int)) {
+      *res = 0;
+      return -1;
+   }
+
+   *res = (rlim_t)limit_int * multiplier;
+   if (*res / multiplier != limit_int) {
+      *res = 0;
+      return -1;
+   }
+
+   return 0;
 }
 
 
-PRIVATE status_e parse_filelog( struct filelog *flp, pset_h values )
+static status_e parse_filelog( struct filelog *flp, pset_h values )
 {
-   unsigned      soft_limit ;
-   unsigned      hard_limit ;
+   rlim_t      soft_limit ;
+   rlim_t      hard_limit ;
    char         *file ;
    unsigned      count = pset_count( values ) ;
-   char         *func = "parse_filelog" ;
+   const char    *func = "parse_filelog" ;
 
    if ( count < 2 || count > 4 )
    {
@@ -712,10 +748,9 @@ PRIVATE status_e parse_filelog( struct filelog *flp, pset_h values )
     */
    if ( count > 2 )
    {
-      soft_limit = get_limit( (char *) pset_pointer( values, 2 ) ) ;
-      if ( soft_limit == 0 )
+      if ( get_limit( (char *) pset_pointer( values, 2 ), &soft_limit ) )
       {
-         parsemsg( LOG_ERR, func, "soft limit is 0" ) ;
+         parsemsg( LOG_ERR, func, "soft limit is invalid" ) ;
          free( file ) ;
          return( FAILED ) ;
       }
@@ -730,12 +765,17 @@ PRIVATE status_e parse_filelog( struct filelog *flp, pset_h values )
        */
       if ( count == 4 )
       {
-         hard_limit = get_limit( (char *) pset_pointer( values, 3 ) ) ;
+         if ( get_limit( (char *) pset_pointer( values, 3 ), &hard_limit) )
+         {
+            parsemsg( LOG_ERR, func, "hard limit is invalid" ) ;
+            free( file ) ;
+            return( FAILED ) ;
+         }
          if ( hard_limit < soft_limit )
          {
             parsemsg( LOG_ERR, func,
-               "hard limit (%d) is less than soft limit (%d)",
-                     hard_limit, soft_limit ) ;
+               "hard limit (%lu) is less than soft limit (%lu)",
+                     (unsigned long)hard_limit, (unsigned long)soft_limit ) ;
             free( file ) ;
             return( FAILED ) ;
          }
@@ -758,13 +798,13 @@ PRIVATE status_e parse_filelog( struct filelog *flp, pset_h values )
 }
 
 
-PRIVATE status_e parse_syslog( struct syslog *slp, pset_h values )
+static status_e parse_syslog( struct syslog *slp, pset_h values )
 {
-   char               *facility ;
-   char               *level ;
-   struct name_value *nvp ;
-   unsigned            count = pset_count( values ) ;
-   char               *func = "parse_syslog" ;
+   const char               *facility ;
+   const char               *level ;
+   const struct name_value  *nvp ;
+   unsigned                  count = pset_count( values ) ;
+   const char               *func  = "parse_syslog" ;
 
    if ( count < 2 || count > 3 )
    {
@@ -803,7 +843,7 @@ status_e log_type_parser( pset_h values,
 {
    struct log *lp = SC_LOG( scp ) ;
    char *type ;
-   char *func = "parse_log_type" ;
+   const char *func = "parse_log_type" ;
 
    type = (char *) pset_pointer( values, 0 ) ;
 
@@ -828,11 +868,10 @@ status_e log_type_parser( pset_h values,
 }
 
 
-
-PRIVATE status_e parse_log_flags( pset_h values, 
+static status_e parse_log_flags( pset_h values, 
                                   enum assign_op op, 
                                   mask_t *maskp, 
-                                  struct name_value options[], 
+                                  const struct name_value options[], 
                                   char *name )
 {
    if ( op == SET_EQ )
@@ -864,14 +903,14 @@ status_e log_on_failure_parser( pset_h values,
 
 
 
-PRIVATE status_e parse_inet_addresses( pset_h values, 
+static status_e parse_inet_addresses( pset_h values, 
                                        enum assign_op op, 
                                        pset_h *addr_list )
 {
    unsigned             u ;
    pset_h               addr_set ;
    statfunc             addrlist_func ;
-   char                 *func = "parse_inet_addresses" ;
+   const char          *func = "parse_inet_addresses" ;
    
    NEW_SET( *addr_list, 0, 0 ) ;
 
@@ -920,17 +959,17 @@ status_e banner_parser(pset_h values,
                        struct service_config *scp, 
                        enum assign_op op)
 {
-   char *func = "banner_parser";
+   const char *func = "banner_parser";
 
    if( pset_pointer(values, 0) == NULL )
    {
-      msg(LOG_ERR, func, "%s: pset_pointer returned null\n", func);
+      msg(LOG_ERR, func, "pset_pointer returned NULL");
       return FAILED;
    }
 
    scp->sc_banner = new_string( pset_pointer(values,0) );
    if( scp->sc_banner == NULL ) {
-      msg(LOG_ERR, func, "%s: Out of memory\n", func);
+      msg(LOG_ERR, func, ES_NOMEM);
       return FAILED;
    }
 
@@ -941,16 +980,16 @@ status_e banner_success_parser(pset_h values,
                                struct service_config *scp, 
                                enum assign_op op)
 {
-   char *func = "banner_success_parser";
+   const char *func = "banner_success_parser";
 
    if( pset_pointer(values, 0) == NULL ) {
-      msg(LOG_ERR, func, "%s: pset_pointer returned null.\n", func);
+      msg(LOG_ERR, func, "pset_pointer returned NULL" );
       return FAILED;
    }
 
    scp->sc_banner_success = new_string(pset_pointer(values,0) );
    if( scp->sc_banner_success == NULL ) {
-      msg(LOG_ERR, func, "%s: Out of memory.\n", func);
+      msg(LOG_ERR, func, ES_NOMEM);
       return FAILED;
    }
 
@@ -961,16 +1000,16 @@ status_e banner_fail_parser(pset_h values,
                             struct service_config *scp, 
                             enum assign_op op)
 {
-   char *func = "banner_fail_parser";
+   const char *func = "banner_fail_parser";
 
    if( pset_pointer(values, 0) == NULL ) {
-      msg(LOG_ERR, func, "%s: pset_pointer returned NULL\n", func);
+      msg(LOG_ERR, func, "pset_pointer returned NULL");
       return FAILED;
    }
 
    scp->sc_banner_fail = new_string(pset_pointer(values,0) );
    if( scp->sc_banner_fail == NULL ) {
-      msg(LOG_ERR, func, "%s: Out of memory\n", func);
+      msg(LOG_ERR, func, ES_NOMEM);
       return FAILED;
    }
 
@@ -982,16 +1021,16 @@ status_e max_load_parser(pset_h values,
                          struct service_config *scp, 
                          enum assign_op op)
 {
-   char *func = "max_load_parser" ;
+   const char *func = "max_load_parser" ;
    char *adr = (char *)pset_pointer(values, 0);
 
    if( sscanf(adr, "%lf", &scp->sc_max_load) < 1 ) {
-      msg(LOG_ERR, func, "error reading max_load argument\n");
+      msg(LOG_ERR, func, "error reading max_load argument");
       return FAILED;
    }
 
    if( scp->sc_max_load == 0 ) {
-      msg(LOG_ERR, func, "error parsing max_load arguement\n");
+      msg(LOG_ERR, func, "error parsing max_load argument");
       return FAILED;
    }
 
@@ -1004,102 +1043,44 @@ status_e redir_parser(pset_h values,
                       enum assign_op op)
 {
    char *adr = (char *)pset_pointer(values, 0);
-   char *foo;
+   const char *func = "redir_parser";
    char *uptmp;
-#ifdef INET6
-   char v6addr[46];
-   struct addrinfo hints, *res, *ressave;
-   int n;
-   char *func="redir_parser";
+   int uptmp_int;
+   struct addrinfo hints, *res;
 
+   /* XXX: error checks */
    uptmp = pset_pointer(values, 1);
-   scp->sc_redir_port = strtol(uptmp, NULL, 10);
-
-   scp->sc_redir_addr = (struct in6_addr *)malloc(sizeof(struct in6_addr));
+   parse_base10(uptmp, &uptmp_int);
 
+   scp->sc_redir_addr = (union xsockaddr *)malloc(sizeof(union xsockaddr));
    if( scp->sc_redir_addr == NULL )
    {
-      msg(LOG_ERR, func, "Cant allocate space for redir addr");
+      msg(LOG_ERR, func, "can't allocate space for redir addr");
       return FAILED;
    }
 
    bzero(&hints, sizeof(hints));
    hints.ai_flags = AI_CANONNAME;
-   hints.ai_family = AF_INET6;
 
-   if( (n = getaddrinfo(adr, NULL, &hints, &res)) < 0 ) {
-      hints.ai_family = AF_INET;
-      if( (n = getaddrinfo(adr, NULL, &hints, &res ) ) < 0 ) {
-         msg(LOG_ERR, func, "%s: Trying IPv4 name failed", func);
-         return FAILED;
-      }
-
-      bzero(v6addr, 46);
-      hints.ai_family = AF_INET6;
-      /* force the IPv4 function */
-      xaddrmap(inet_ntoa( ((struct sockaddr_in *)(res->ai_addr))->sin_addr), v6addr);
-      freeaddrinfo(res);
-
-      if( (n = getaddrinfo(v6addr, NULL, &hints, &res)) < 0 ) {
-         msg(LOG_ERR, func, "%s: bad address", func);
-         return FAILED;
-      }
+   if( getaddrinfo(adr, NULL, &hints, &res) < 0 ) {
+      msg(LOG_ERR, func, "bad address");
+      return FAILED;
    }
 
    if( (res == NULL) || (res->ai_addr == NULL) ) {
-      msg(LOG_ERR, func, "%s: no addresses returned", func);
+      msg(LOG_ERR, func, "no addresses returned");
       return FAILED;
    }
       
-   memcpy(scp->sc_redir_addr, &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr, sizeof(struct in6_addr));
+   if( (res->ai_family == AF_INET) || (res->ai_family == AF_INET6) )
+      memcpy(scp->sc_redir_addr, res->ai_addr, res->ai_addrlen);
+   if( scp->sc_redir_addr->sa.sa_family == AF_INET ) 
+      scp->sc_redir_addr->sa_in.sin_port = uptmp_int;
+   if( scp->sc_redir_addr->sa.sa_family == AF_INET6 ) 
+      scp->sc_redir_addr->sa_in6.sin6_port = uptmp_int;
 
+   freeaddrinfo(res);
    return OK;
-
-#else
-   struct hostent *he;
-
-   uptmp = pset_pointer(values, 1);
-   scp->sc_redir_port = strtol(uptmp, NULL, 10);
-   scp->sc_redir_addr = (struct in_addr *)malloc(sizeof(struct in_addr));
-
-   if( scp->sc_redir_addr == NULL )
-   {
-      parsemsg(LOG_ERR, "redir_parser", "Cant allocate space for redir addr");
-      return FAILED;
-   }
-
-   he = gethostbyname(adr);
-   if( he == NULL )
-   {
-#if defined(NO_INET_ATON)
-      tempaddr = inet_addr(adr);
-      bcopy(&tempaddr, scp->sc_redir_addr, sizeof(tempaddr));
-#else
-      if( inet_aton(adr, scp->sc_redir_addr) < 1 )
-      {
-         parsemsg(LOG_ERR, "redir_parser", "Invalid IP address %s", adr);
-         free(scp->sc_redir_addr);
-         return FAILED;
-      }
-#endif
-   }
-   else
-   {
-      /* gethostbyname succeeded */
-      bcopy(he->h_addr, scp->sc_redir_addr, sizeof(he->h_addr));
-   }
-
-   foo = xntoa( SA(scp->sc_redir_addr) );
-
-   if( foo == NULL )
-   {
-      parsemsg(LOG_ERR, "redir_parser", "xntoa failed");
-      free(scp->sc_redir_addr);
-      return FAILED;
-   }
-
-   return OK;
-#endif /* INET6 */
 }
 
 status_e bind_parser( pset_h values, 
@@ -1107,97 +1088,39 @@ status_e bind_parser( pset_h values,
                       enum assign_op op)
 {
    char *adr = (char *)pset_pointer(values, 0);
-   char *foo;
-#ifdef INET6
-   char *func = "bind_parser";
-   unsigned long tempaddr;
+   const char *func = "bind_parser";
    int n;
-   char v6addr[46];
    struct addrinfo hints, *res;
 
-   scp->sc_bind_addr = (struct in6_addr *)malloc(sizeof(struct in6_addr));
-
+   scp->sc_bind_addr = (union xsockaddr *)malloc(sizeof(union xsockaddr));
    if( scp->sc_bind_addr == NULL )
    {
-      msg(LOG_ERR, func, "%s: cant allocate space for bind addr\n", func);
+      msg(LOG_ERR, func, "can't allocate space for bind addr");
       return FAILED;
    }
 
    bzero(&hints, sizeof(hints));
    hints.ai_flags = AI_CANONNAME;
-   hints.ai_family = AF_INET6;
 
-   if( (n = getaddrinfo(adr, NULL, &hints, &res ) ) < 0 ) {
-      hints.ai_family = AF_INET;
-      if( (n = getaddrinfo(adr, NULL, &hints, &res ) ) < 0 ) {
-         msg(LOG_ERR, func, "%s: Trying IPv4 name failed\n", func);
-         return FAILED;
-      }
-
-      bzero(v6addr, 46);
-      hints.ai_family = AF_INET6;
-      xaddrmap(inet_ntoa( ((struct sockaddr_in *)(res->ai_addr))->sin_addr), v6addr);
-      freeaddrinfo(res);
-
-      if( (n = getaddrinfo(v6addr, NULL, &hints, &res ) ) < 0 ) {
-         msg(LOG_ERR, func, "%s: bad address", func);
-         return FAILED;
-      }
-   }
-
-   if( ( res == NULL ) || ( res->ai_addr == NULL ) ) {
-      msg(LOG_ERR, func, "%s: no addresses returned", func);
+   if( (n = getaddrinfo(adr, NULL, &hints, &res) ) < 0 ) {
+      msg(LOG_ERR, func, "bad address");
+      free( scp->sc_bind_addr );
+      scp->sc_bind_addr = NULL;
       return FAILED;
    }
 
-   memcpy(scp->sc_bind_addr, &((struct sockaddr_in6 *)(res->ai_addr))->sin6_addr, sizeof(struct in6_addr));
+   if( (res == NULL) || (res->ai_addr == NULL) ) {
+      msg(LOG_ERR, func, "no addresses returned");
+      free( scp->sc_bind_addr );
+      scp->sc_bind_addr = NULL;
+      return FAILED;
+   }
+      
+   if( (res->ai_family == AF_INET) || (res->ai_family == AF_INET6) )
+      memcpy(scp->sc_bind_addr, res->ai_addr, res->ai_addrlen);
 
    freeaddrinfo(res);
-   return OK;
-#else
-   struct hostent *he;
-
-   scp->sc_bind_addr = (struct in_addr *)malloc(sizeof(struct in_addr));
-
-   if( scp->sc_bind_addr == NULL )
-   {
-      parsemsg(LOG_ERR, "bind_parser", "Cant allocate space for bind addr");
-      return FAILED;
-   }
-
-   he = gethostbyname(adr);
-   if( he == NULL )
-   {
-#if defined(NO_INET_ATON)
-      tempaddr = inet_addr(adr);
-      bcopy(&tempaddr, scp->sc_bind_addr, sizeof(tempaddr));
-#else
-      if( inet_aton(adr, scp->sc_bind_addr) < 1 )
-      {
-         parsemsg(LOG_ERR, "bind_parser", "invalid IP address %s", adr);
-         free(scp->sc_bind_addr);
-         return FAILED;
-      }
-#endif
-   }
-   else
-   {
-      /* gethostbyname succeeded, so we now have the address */
-      bcopy(he->h_addr, scp->sc_bind_addr, sizeof(he->h_addr));
-   }
-
-   foo = inet_ntoa(*scp->sc_bind_addr);
-
-   if( foo == NULL )
-   {
-      parsemsg(LOG_ERR, "bind_parser", "inet_ntoa failed");
-      free(scp->sc_bind_addr);
-      return FAILED;
-   }
-   
-   return OK;
-#endif /* INET6 */
-   
+   return( OK );
 }
 
 status_e access_times_parser( pset_h values, 
@@ -1205,8 +1128,7 @@ status_e access_times_parser( pset_h values,
                               enum assign_op op )
 {
    unsigned u ;
-   char *func = "access_times_parser" ;
-   status_e ti_add( pset_h iset, char *interval_str );
+   const char *func = "access_times_parser" ;
 
    NEW_SET( scp->sc_access_times, 0, 0 ) ;
 
@@ -1225,7 +1147,10 @@ status_e nice_parser( pset_h values,
                       struct service_config *scp, 
                       enum assign_op op )
 {
-   scp->sc_nice = atoi( (char *) pset_pointer( values, 0 ) ) ;
+   if ( parse_base10((char *) pset_pointer( values, 0 ), &scp->sc_nice) ) {
+      parsemsg(LOG_ERR, "nice_parser", "Error parsing: %s", (char *)pset_pointer( values, 0 ));
+      return( FAILED );
+   }
    return( OK ) ;
 }
 
@@ -1235,17 +1160,16 @@ status_e rlim_as_parser( pset_h values,
                          enum assign_op op )
 {
    char *mem = (char *) pset_pointer( values, 0 ) ;
-   char *func = "rlim_as_parser" ;
+   const char *func = "rlim_as_parser" ;
 
    if ( EQ( mem, "UNLIMITED" ) )
-      scp->sc_rlim_as = RLIM_INFINITY ;
+      scp->sc_rlim_as = (rlim_t)RLIM_INFINITY ;
    else
    {
-      scp->sc_rlim_as = (rlim_t) get_limit( mem );
-      if ( scp->sc_rlim_as < 0 )
+      if ( get_limit ( mem, &scp->sc_rlim_as) )
       {
          parsemsg( LOG_ERR, func,
-            "Address space limit is negative: %s", mem ) ;
+            "Address space limit is invalid: %s", mem ) ;
          return( FAILED ) ;
       }
    }
@@ -1258,20 +1182,21 @@ status_e rlim_cpu_parser( pset_h values,
                           struct service_config *scp, 
                           enum assign_op op )
 {
-   char *mem = (char *) pset_pointer( values, 0 ) ;
-   char *func = "rlim_cpu_parser" ;
+   char *cpu_str = (char *) pset_pointer( values, 0 ) ;
+   int cpu_int;
+   const char *func = "rlim_cpu_parser" ;
 
-   if ( EQ( mem, "UNLIMITED" ) )
-      scp->sc_rlim_cpu = RLIM_INFINITY ;
+   if ( EQ( cpu_str, "UNLIMITED" ) )
+      scp->sc_rlim_cpu = (rlim_t)RLIM_INFINITY ;
    else
    {
-      scp->sc_rlim_cpu = (rlim_t) atoi( mem ) ;
-      if ( scp->sc_rlim_cpu < 0 )
+      if ( parse_base10(cpu_str, &cpu_int) || cpu_int < 0 )
       {
          parsemsg( LOG_ERR, func,
-            "CPU limit is negative: %s", mem ) ;
+            "CPU limit is invalid: %s", cpu_str ) ;
          return( FAILED ) ;
       }
+      scp->sc_rlim_cpu = (rlim_t) cpu_int ;
    }
    return( OK ) ;
 }
@@ -1283,17 +1208,16 @@ status_e rlim_data_parser( pset_h values,
                            enum assign_op op )
 {
    char *mem = (char *) pset_pointer( values, 0 ) ;
-   char *func = "rlim_data_parser" ;
+   const char *func = "rlim_data_parser" ;
 
    if ( EQ( mem, "UNLIMITED" ) )
-      scp->sc_rlim_data = RLIM_INFINITY ;
+      scp->sc_rlim_data = (rlim_t)RLIM_INFINITY ;
    else
    {
-      scp->sc_rlim_data = (rlim_t) get_limit( mem ) ;
-      if ( scp->sc_rlim_data < 0 )
+      if ( get_limit ( mem, &scp->sc_rlim_data ) )
       {
          parsemsg( LOG_ERR, func,
-            "Data limit is negative: %s", mem ) ;
+            "Data limit is invalid: %s", mem ) ;
          return( FAILED ) ;
       }
    }
@@ -1307,17 +1231,16 @@ status_e rlim_rss_parser( pset_h values,
                           enum assign_op op )
 {
    char *mem = (char *) pset_pointer( values, 0 ) ;
-   char *func = "rlim_rss_parser" ;
+   const char *func = "rlim_rss_parser" ;
 
    if ( EQ( mem, "UNLIMITED" ) )
-      scp->sc_rlim_rss = RLIM_INFINITY ;
+      scp->sc_rlim_rss = (rlim_t)RLIM_INFINITY ;
    else
    {
-      scp->sc_rlim_rss = (rlim_t) get_limit( mem ) ;
-      if ( scp->sc_rlim_rss < 0 )
+      if ( get_limit ( mem, &scp->sc_rlim_rss ) )
       {
          parsemsg( LOG_ERR, func,
-            "RSS limit is negative: %s", mem ) ;
+            "RSS limit is invalid: %s", mem ) ;
          return( FAILED ) ;
       }
    }
@@ -1331,17 +1254,16 @@ status_e rlim_stack_parser( pset_h values,
                             enum assign_op op )
 {
    char *mem = (char *) pset_pointer( values, 0 ) ;
-   char *func = "rlim_stack_parser" ;
+   const char *func = "rlim_stack_parser" ;
 
    if ( EQ( mem, "UNLIMITED" ) )
-      scp->sc_rlim_stack = RLIM_INFINITY ;
+      scp->sc_rlim_stack = (rlim_t)RLIM_INFINITY ;
    else
    {
-      scp->sc_rlim_stack = (rlim_t) get_limit( mem ) ;
-      if ( scp->sc_rlim_stack < 0 )
+      if ( get_limit ( mem, &scp->sc_rlim_stack ) )
       {
          parsemsg( LOG_ERR, func,
-            "Stack limit is negative: %s", mem ) ;
+            "Stack limit is invalid: %s", mem ) ;
          return( FAILED ) ;
       }
    }
@@ -1359,8 +1281,10 @@ status_e deny_time_parser( pset_h values,
       scp->sc_deny_time = -1 ;
    else if ( EQ( deny_time, "NEVER" ) )
       scp->sc_deny_time = 0 ;
-   else
-      scp->sc_deny_time = atoi( deny_time ) ;
+   else if ( parse_base10( deny_time, &scp->sc_deny_time ) ) {
+      parsemsg(LOG_ERR, "deny_time_parser", "Error parsing: %s", deny_time);
+      return( FAILED );
+   }
    return( OK ) ;
 }
 
@@ -1368,16 +1292,15 @@ status_e umask_parser( pset_h values,
                        struct service_config *scp, 
                        enum assign_op op )
 {
-   char *umaskstr = (char *)pset_pointer(values, 0);
+   char *umask_str = (char *)pset_pointer(values, 0);
+   int umask_int;
 
-   scp->sc_umask = strtol(umaskstr, NULL, 8);
-   if( errno == ERANGE )
-      return( FAILED );
-
-   if( scp->sc_umask < 0)
+   if( parse_int(umask_str, 8, -1, &umask_int) ||
+       umask_int < 0 || umask_int > 0777)
    {
-      parsemsg(LOG_ERR, "umask_parser", "umask arguement not a number.\n");
+      parsemsg(LOG_ERR, "umask_parser", "umask argument is invalid.\n");
       return FAILED ;
    }
+   scp->sc_umask = umask_int;
    return( OK );
 }
