@@ -5,17 +5,18 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: sconf.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
-
 #include "config.h"
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
 
-#include "state.h"
-#include "defs.h"
-#include "attr.h"
-#include "sconf.h"
-#include "addr.h"
-#include "timex.h"
 #include "str.h"
+#include "sio.h"
+#include "sconf.h"
+#include "timex.h"
+#include "addr.h"
+#include "nvlists.h"
+
 
 #define NEW_SCONF()               NEW( struct service_config )
 #define FREE_SCONF( scp )         FREE( scp )
@@ -29,14 +30,6 @@ static char RCSid[] = "$Id: sconf.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $"
                                     x = NULL ;              \
                                  }
 
-extern struct name_value success_log_options[] ;
-extern struct name_value failure_log_options[] ;
-extern struct name_value service_types[] ;
-extern struct name_value service_flags[] ;
-extern struct name_value socket_types[] ;
-extern struct name_value syslog_facilities[] ;
-extern struct name_value syslog_levels[] ;
-
 
 /*
  * Allocate a new service_config and initialize the service name field
@@ -46,7 +39,7 @@ extern struct name_value syslog_levels[] ;
 struct service_config *sc_alloc( char *name )
 {
    struct service_config *scp ;
-   char *func = "sc_alloc" ;
+   const char *func = "sc_alloc" ;
 
    scp = NEW_SCONF() ;
    if ( scp == NULL )
@@ -60,7 +53,7 @@ struct service_config *sc_alloc( char *name )
 }
 
 
-PRIVATE void release_string_pset( pset_h pset )
+static void release_string_pset( pset_h pset )
 {
    pset_apply( pset, free, NULL ) ;
    pset_destroy( pset ) ;
@@ -125,13 +118,13 @@ void sc_free( struct service_config *scp )
 /*
  * Create a configuration for one of the special services
  */
-struct service_config *sc_make_special( char *service_name, 
-                                        builtin_s *bp, 
+struct service_config *sc_make_special( const char *service_name, 
+                                        const builtin_s *bp, 
                                         int instances )
 {
    char *name ;
    struct service_config *scp ;
-   char *func = "sc_make" ;
+   const char *func = "sc_make" ;
 
    name = new_string( service_name ) ;
    if ( name == NULL )
@@ -177,7 +170,7 @@ struct service_config *sc_make_special( char *service_name,
 
 
 
-PRIVATE void dump_log_data( int fd, struct service_config *scp, int tab_level )
+static void dump_log_data( int fd, struct service_config *scp, int tab_level )
 {
    struct log *lp = SC_LOG( scp ) ;
    struct filelog *flp ;
@@ -234,7 +227,7 @@ void sc_dump( struct service_config *scp,
               int tab_level, 
               bool_int is_defaults )
 {
-   struct name_value    *nvp ;
+   const struct name_value    *nvp ;
    unsigned             u ;
    char                 **pp ;
 
@@ -291,9 +284,28 @@ void sc_dump( struct service_config *scp,
    if ( SC_SPECIFIED( scp, A_GROUPS ) )
       tabprint( fd, tab_level+1, "Groups = %d\n", scp->sc_groups );
 
+   if ( SC_SPECIFIED( scp, A_CPS ) )
+      tabprint( fd, tab_level+1, "CPS = %lu %lu\n", scp->sc_time_wait, 
+         scp->sc_time_conn_max );
+
+
+   if ( SC_SPECIFIED( scp, A_BIND ) && scp->sc_bind_addr ) {
+      char bindname[NI_MAXHOST];
+      int len = 0;
+      if( scp->sc_bind_addr->sa.sa_family == AF_INET ) 
+         len = sizeof(struct sockaddr_in);
+      if( scp->sc_bind_addr->sa.sa_family == AF_INET6 ) 
+         len = sizeof(struct sockaddr_in6);
+      memset(bindname, 0, sizeof(bindname));
+      if( getnameinfo(&scp->sc_bind_addr->sa, len, bindname, NI_MAXHOST, 
+            NULL, 0, 0) != 0 ) 
+         strcpy(bindname, "unknown");
+      tabprint( fd, tab_level+1, "Bind = %s\n", bindname );
+   }
+
    if ( ! is_defaults )
    {
-      if ( ! SC_IS_INTERNAL( scp ) )
+      if ( (! SC_IS_INTERNAL( scp )) && (scp->sc_redir_addr == NULL) )
       {
          tabprint( fd, tab_level+1, "Server = %s\n", scp->sc_server ) ;
          tabprint( fd, tab_level+1, "Server argv =" ) ;
@@ -302,6 +314,21 @@ void sc_dump( struct service_config *scp,
          Sputchar( fd, '\n' ) ;
       } 
 
+      if ( scp->sc_redir_addr != NULL ) 
+      {
+         char redirname[NI_MAXHOST];
+         int len = 0;
+         if( scp->sc_redir_addr->sa.sa_family == AF_INET ) 
+            len = sizeof(struct sockaddr_in);
+         if( scp->sc_redir_addr->sa.sa_family == AF_INET6 ) 
+            len = sizeof(struct sockaddr_in6);
+         memset(redirname, 0, sizeof(redirname));
+         if( getnameinfo(&scp->sc_redir_addr->sa, len,  redirname, NI_MAXHOST, 
+               NULL, 0, 0) != 0 ) 
+            strcpy(redirname, "unknown");
+         tabprint( fd, tab_level+1, "Redirect = %s:%d\n", redirname, ntohs(scp->sc_redir_addr->sa_in.sin_port) );
+      }
+
       if ( SC_IS_RPC( scp ) )
       {
          struct rpc_data *rdp = SC_RPCDATA( scp ) ;
