@@ -11,27 +11,27 @@
 /*
  * $Id: sconf.h,v 1.3 1999/10/13 04:22:34 bbraun Exp $
  */
-
-#include <netinet/in.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netdb.h>
-#include <sys/time.h>
-#include <sys/resource.h>
-
-#include "pset.h"
-#include "env.h"
-#include "sio.h"
-
-#include "defs.h"
-#include "log.h"
-#include "mask.h"
-#include "builtins.h"
 #include "config.h"
-
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
+#include <sys/types.h>
+#include <netinet/in.h>
+#include <sys/socket.h>
+#include <netdb.h>
+#include <sys/time.h>
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+#include "libportable.h"
+
+#include "pset.h"
+#include "m_env.h"
+#include "mask.h"
+#include "defs.h"
+#include "log.h"
+#include "builtins.h"
+#include "attr.h"
 
 /*
  * Service types
@@ -40,7 +40,7 @@
 #define ST_INTERNAL     2
 #define ST_UNLISTED     3
 #define ST_SPECIAL      4
-#define ST_DISABLED     5 /* Note, this is *only* used by the enable option */
+#define ST_DISABLED     5 
 
 /*
  * Service flags
@@ -51,10 +51,11 @@
 #define SF_IDONLY       4
 #define SF_NAMEINARGS   5
 #define SF_NODELAY      6
-#define SF_DISABLE      7
-#define SF_KEEPALIVE    8
-#define SF_NOLIBWRAP    9
-#define SF_SENSOR       10
+#define SF_KEEPALIVE    7
+#define SF_NOLIBWRAP    8
+#define SF_SENSOR       9
+#define SF_IPV4         10
+#define SF_IPV6         11
 
 /*
  * Values for log options
@@ -65,10 +66,7 @@
 #define LO_EXIT      5
 #define LO_RECORD    6
 #define LO_PID       7
-#define LO_START     8
-#define LO_USERID    9
-
-
+#define LO_USERID    8
 
 struct rpc_data
 {
@@ -79,7 +77,7 @@ struct rpc_data
 
 #define RD_MINVERS( rdp )      (rdp)->rd_min_version
 #define RD_MAXVERS( rdp )      (rdp)->rd_max_version
-#define RD_PROGNUM( rdp )       (rdp)->rd_program_number
+#define RD_PROGNUM( rdp )      (rdp)->rd_program_number
 
 typedef enum { NO_ENV = 0, STD_ENV, DEF_ENV, CUSTOM_ENV } environ_e ;
 
@@ -97,16 +95,16 @@ struct service_config
    mask_t               sc_specified_attributes; /* specified attributes */
    mask_t               sc_attributes_present; /* includes those from defaults*/
    mask_t               sc_type;               /* RPC, UNLISTED etc           */
-   mask_t               sc_xflags ;            /* REUSE, INTERCEPT etc        */
+   mask_t               sc_xflags ;            /* INTERCEPT etc               */
    char                *sc_name;               /* e g  "echo"                 */
    char                *sc_id ;                /* e.g. "echo-stream"          */
    uint16_t             sc_port ;              /* in host byte order          */
    int                  sc_socket_type ;       /* e.g. SOCK_DGRAM             */
    struct name_value    sc_protocol ;          /* e.g. "TCP", IPPROTO_TCP     */
    boolean_e            sc_wait ;
-   int                  sc_uid ;
-   int                  sc_user_gid ;          /* gid corresponding to uid    */
-   int                  sc_gid ;               /* gid corresponding to group  */
+   uid_t                sc_uid ;
+   gid_t                sc_user_gid ;          /* gid corresponding to uid    */
+   gid_t                sc_gid ;               /* gid corresponding to group  */
    char                *sc_server ;
    char               **sc_server_argv ;
    int                  sc_instances ;
@@ -123,16 +121,10 @@ struct service_config
    pset_h               sc_disabled ;     /* used only by the default entry */
    pset_h               sc_enabled ;      /* used only by the default entry */
    struct environment   sc_environment ;
-   builtin_s           *sc_builtin ;
-   uint16_t             sc_redir_port;
-#ifdef INET6
-   struct in6_addr     *sc_redir_addr ;
-   struct in6_addr     *sc_bind_addr ;
+   const builtin_s     *sc_builtin ;
+   union xsockaddr     *sc_redir_addr ;
+   union xsockaddr     *sc_bind_addr ;
    boolean_e            sc_v6only;
-#else
-   struct in_addr      *sc_redir_addr ;
-   struct in_addr      *sc_bind_addr ;
-#endif
    char                *sc_banner ;
    int                  sc_per_source ;
    boolean_e            sc_groups ;
@@ -140,8 +132,8 @@ struct service_config
    char                *sc_banner_fail ;
    double               sc_max_load ;
    time_t               sc_time_limit ;
-   int                  sc_time_conn ;
-   int                  sc_time_conn_max ;
+   time_t               sc_time_conn ;
+   time_t               sc_time_conn_max ;
    time_t               sc_time_wait ;
    time_t               sc_time_reenable ;
    rlim_t               sc_rlim_as;
@@ -164,7 +156,6 @@ struct service_config
 #define SC_LOG( scp )            (&(scp)->sc_log)
 #define SC_RPCDATA( scp )        (&(scp)->sc_rd)
 #define SC_ENV( scp )            (&(scp)->sc_environment)
-#define SC_BUILTIN( scp )        (scp)->sc_builtin
 #define SC_PORT( scp )           (scp)->sc_port
 #define SC_NICE( scp )           (scp)->sc_nice
 #define SC_SOCKET_TYPE( scp )    (scp)->sc_socket_type
@@ -186,9 +177,7 @@ struct service_config
 #define SC_RLIM_DATA( scp )      (scp)->sc_rlim_data
 #define SC_RLIM_RSS( scp )       (scp)->sc_rlim_rss
 #define SC_RLIM_STACK( scp )     (scp)->sc_rlim_stack
-#ifdef INET6
 #define SC_V6ONLY( scp )         (scp)->sc_v6only
-#endif
 
 /*
  * Field set macros
@@ -202,15 +191,15 @@ struct service_config
     builtin_forks( (scp)->sc_builtin ) )
 #define SC_WAITS( scp )           ( (scp)->sc_wait == YES )
 #define SC_RETRY( scp )           ( M_IS_CLEAR( (scp)->sc_xflags, SF_NORETRY ) )
-#define SC_REUSE_ADDRESS( scp )   M_IS_SET( (scp)->sc_xflags, SF_REUSE )
 #define SC_MUST_IDENTIFY( scp )   M_IS_SET( (scp)->sc_xflags, SF_IDONLY )
 #define SC_NAMEINARGS( scp )      M_IS_SET( (scp)->sc_xflags, SF_NAMEINARGS )
 #define SC_NODELAY( scp )         M_IS_SET( (scp)->sc_xflags, SF_NODELAY )
-#define SC_SDISABLE( scp )        M_IS_SET( (scp)->sc_xflags, SF_DISABLE )
 #define SC_KEEPALIVE( scp )       M_IS_SET( (scp)->sc_xflags, SF_KEEPALIVE )
 #define SC_NOLIBWRAP( scp )       M_IS_SET( (scp)->sc_xflags, SF_NOLIBWRAP )
 #define SC_IS_INTERCEPTED( scp ) ( M_IS_SET( (scp)->sc_xflags, SF_INTERCEPT ) )
 #define SC_SENSOR( scp )          M_IS_SET( (scp)->sc_xflags, SF_SENSOR )
+#define SC_IPV4( scp )            M_IS_SET( (scp)->sc_xflags, SF_IPV4 )
+#define SC_IPV6( scp )            M_IS_SET( (scp)->sc_xflags, SF_IPV6 )
 
 #define SC_IS_RPC( scp )         ( M_IS_SET( (scp)->sc_type, ST_RPC ) )
 #define SC_IS_INTERNAL( scp )    ( M_IS_SET( (scp)->sc_type, ST_INTERNAL ) )
@@ -251,29 +240,22 @@ struct service_config
                   M_SET( (scp)->sc_specified_attributes, (attr) ) ;  \
                   SC_PRESENT( (scp), (attr) ) ;                      \
                }
-#define SC_UNSPECIFY( scp, attr )   \
-               M_CLEAR( (scp)->sc_specified_attributes, (attr) )
 
 #define SC_IS_PRESENT( scp, attr )  \
                M_IS_SET( (scp)->sc_attributes_present, (attr) )
 #define SC_PRESENT( scp, attr )     \
                M_SET( (scp)->sc_attributes_present, (attr) )
-#define SC_ABSENT( scp, attr )      \
-               {                                                     \
-                  M_CLEAR( (scp)->sc_attributes_present, (attr) ) ;  \
-                  SC_UNSPECIFY( (scp), (attr) ) ;                      \
-               }
 
 #define sc_getgid( scp ) ( SC_SPECIFIED( scp, A_GROUP ) \
    ? (scp)->sc_gid : (scp)->sc_user_gid )
 #define sc_internal( scp, serp )  builtin_invoke( (scp)->sc_builtin, serp )
 #define sc_make_external( scp )   M_CLEAR( (scp)->sc_type, ST_INTERNAL )
 
-struct service_config   *sc_alloc(char *) ;
-void                     sc_free(struct service_config *) ;
-void                     sc_dump(struct service_config *, int, int, bool_int) ;
-struct service_config   *sc_make_special(char *, builtin_s *, int) ;
-bool_int                 sc_different_confs(struct service_config *, struct service_config *) ;
+struct service_config *sc_alloc(char *name);
+void sc_free(struct service_config *scp);
+struct service_config *sc_make_special(const char *service_name,const builtin_s *bp,int instances);
+void sc_dump(struct service_config *scp,int fd,int tab_level,bool_int is_defaults);
+bool_int sc_different_confs(struct service_config *scp1,struct service_config *scp2);
+
 
 #endif   /* SCONF_H */
-
