@@ -5,8 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: server.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
-
 #include "config.h"
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -22,18 +20,20 @@ static char RCSid[] = "$Id: server.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $"
 #include <sys/file.h>
 #endif
 
+#include "pset.h"
 #include "sio.h"
-
-#include "state.h"
-#include "access.h"
-#include "connection.h"
-#include "xconfig.h"
-#include "service.h"
 #include "server.h"
-#include "servicedefs.h"
-#include "log.h"
+#include "util.h"
+#include "msg.h"
+#include "service.h"
+#include "sconf.h"
+#include "state.h"
+#include "main.h"
+#include "xconfig.h"
+#include "retry.h"
 #include "child.h"
 
+
 #define NEW_SERVER()                NEW( struct server )
 #define FREE_SERVER( serp )         FREE( serp )
 
@@ -54,10 +54,10 @@ extern int errno ;
 /*
  * Allocate a server, initialize it from init_serp, and insert it in stab
  */
-PRIVATE struct server *server_alloc( struct server *init_serp, pset_h stab )
+static struct server *server_alloc( const struct server *init_serp, pset_h stab )
 {
    struct server   *serp ;
-   char            *func = "server_alloc" ;
+   const char      *func = "server_alloc" ;
 
    serp = NEW_SERVER() ;
    if ( serp == NULL )
@@ -101,10 +101,10 @@ void server_release( struct server *serp )
  *         ignored (i.e. nothing is logged).
  *      -   it can be identified in the log because the server pid is 0.
  */
-PRIVATE void server_internal( struct server *serp )
+static void server_internal( struct server *serp )
 {
    struct service *sp = serp->svr_sp ;
-   char *func = "server_internal" ;
+   const char *func = "server_internal" ;
 
    serp->svr_pid = 0 ;
    if ( SVC_ACCEPTS_CONNECTIONS( sp ) &&
@@ -129,7 +129,7 @@ status_e server_run( struct service *sp, connection_s *cp )
 {
    struct server    server ;
    struct server   *serp ;
-   char            *func = "server_run" ;
+   const char      *func = "server_run" ;
 
    CLEAR( server ) ;
    server.svr_sp = sp ;
@@ -141,7 +141,7 @@ status_e server_run( struct service *sp, connection_s *cp )
     * high volume machines, this needs to run from a timer services
     * loop.
     */
-   scrub_global_access_lists( cp );
+   scrub_global_access_lists();
 
    /*
     * Allocate a server struct only if we will fork a new process
@@ -154,8 +154,12 @@ status_e server_run( struct service *sp, connection_s *cp )
        */
       if (svc_child_access_control( sp, cp ) == OK)
          server_internal( &server ) ;
-      conn_cleanup( cp ) ;   /* forces a drain on DGRAM...   */
-      conn_free( cp ) ;      /* Parent must close socket   */
+      if ( SVC_WAITS( sp ) )
+         svc_resume( sp );
+      if ( SVC_IS_ACTIVE( sp ) ) {
+         conn_cleanup( cp ) ;   /* forces a drain on DGRAM...   */
+         conn_free( cp, 1 ) ;      /* Parent must close socket   */
+      }
       return( OK ) ;
    }
 
@@ -203,7 +207,7 @@ status_e server_run( struct service *sp, connection_s *cp )
 status_e server_start( struct server *serp )
 {
    struct service   *sp = serp->svr_sp ;
-   char             *func = "server_start" ;
+   const char       *func = "server_start" ;
 
    if( debug.on )
       msg( LOG_DEBUG, func, "Starting service %s", SC_NAME( SVC_CONF( sp ) ) );
@@ -246,9 +250,9 @@ status_e server_start( struct server *serp )
 }
 
 
-void server_dump( struct server *serp, int fd )
+void server_dump( const struct server *serp, int fd )
 {
-   struct service *sp = serp->svr_sp ;
+   const struct service *sp = serp->svr_sp ;
 
    Sprint( fd, "%s server\n", SVC_ID( sp ) ) ;
    Sprint( fd, "pid = %d\n", serp->svr_pid ) ;
@@ -273,7 +277,7 @@ void server_dump( struct server *serp, int fd )
 void server_end( struct server *serp )
 {
    struct service *sp = serp->svr_sp ;
-   char *func = "server_end" ;
+   const char *func = "server_end" ;
 
    if ( PROC_EXITED( serp->svr_exit_status ) || 
          PROC_SIGNALED( serp->svr_exit_status ) )
