@@ -12,12 +12,13 @@
  * $Id: server.h,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $
  */
 
+#include "config.h"
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <time.h>
 
 #include "defs.h"
-#include "service.h"
-#include "connection.h"
+
 
 #ifdef NO_POSIX_TYPES
 typedef int pid_t ;
@@ -59,13 +60,6 @@ struct server
 #define server_set_exit_status( serp, status )   \
                   (serp)->svr_exit_status = (status)
 
-void            server_release(struct server *) ;
-status_e        server_run(struct service *, connection_s *) ;
-status_e        server_start(struct server *) ;
-void            server_dump(struct server *, int) ;
-void            server_end(struct server *) ;
-struct server   *server_lookup(pid_t) ;
-status_e        schedule_retry( struct server * );
 
 /*
  * Macros for compatibility
@@ -84,5 +78,13 @@ status_e        schedule_retry( struct server * );
 #define PROC_TERMSIG( s )        (((union wait *)&(s))->w_T.w_Termsig)
 #endif   /* OLD_WAIT */
 
+void server_release(struct server *serp);
+status_e server_run(struct service *sp,connection_s *cp);
+status_e server_start(struct server *serp);
+void server_dump(const struct server *serp,int fd);
+void server_end(struct server *serp);
+struct server *server_lookup(pid_t pid);
+
+
 #endif   /* SERVER_H */
 
