@@ -5,8 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: service.c,v 1.3 1999/10/13 04:22:34 bbraun Exp $" ;
-
 #include "config.h"
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -19,6 +17,7 @@ static char RCSid[] = "$Id: service.c,v 1.3 1999/10/13 04:22:34 bbraun Exp $" ;
 #include <unistd.h>
 #include <signal.h>
 #include <time.h>
+#include <errno.h>
 #include <netinet/in.h>
 #ifndef NO_RPC
  #ifdef HAVE_RPC_PMAP_CLNT_H
@@ -30,6 +29,7 @@ static char RCSid[] = "$Id: service.c,v 1.3 1999/10/13 04:22:34 bbraun Exp $" ;
   #include <rpc/xdr.h>
   #include <rpc/pmap_clnt.h>
  #endif
+ #include <rpc/rpc.h>
 #endif
 
 /* This is because of differences between glibc2 and libc5 in linux... */
@@ -44,31 +44,26 @@ static char RCSid[] = "$Id: service.c,v 1.3 1999/10/13 04:22:34 bbraun Exp $" ;
 #endif
 
 #include "sio.h"
-
-#include "options.h"
-
-#include "access.h"
-#include "attr.h"
 #include "service.h"
-#include "server.h"
-#include "state.h"
+#include "util.h"
+#include "main.h"
+#include "sconf.h"
+#include "msg.h"
+#include "options.h"
+#include "logctl.h"
 #include "xconfig.h"
-#include "connection.h"
-#include "sconst.h"
-#include "servicedefs.h"
-#include "log.h"
+
+
 
 #define NEW_SVC()              NEW( struct service )
 #define FREE_SVC( sp )         FREE( sp )
 
-#define SUSPEND( sp )          (sp)->svc_state = SVC_SUSPENDED
-#define RESUME( sp )           (sp)->svc_state = SVC_ACTIVE
 #define DISABLE( sp )          (sp)->svc_state = SVC_DISABLED
 
-PRIVATE status_e svc_generic_handler( struct service *sp, connection_s *cp );
-PRIVATE void deactivate( struct service *sp );
+static status_e svc_generic_handler( struct service *sp, connection_s *cp );
+static void deactivate( const struct service *sp );
 
-static struct name_value service_states[] =
+static const struct name_value service_states[] =
    {
       { "Not started",        (int) SVC_NOT_STARTED    },
       { "Active",             (int) SVC_ACTIVE         },
@@ -86,7 +81,7 @@ static struct name_value service_states[] =
 struct service *svc_new( struct service_config *scp )
 {
    struct service *sp ;
-   char *func = "svc_new" ;
+   const char *func = "svc_new" ;
 
    sp = NEW_SVC() ;
    if ( sp == NULL )
@@ -104,7 +99,7 @@ struct service *svc_new( struct service_config *scp )
 struct service *svc_make_special( struct service_config *scp, statfunc handler )
 {
    struct service      *sp ;
-   char               *func = "svc_make_special" ;
+   const char          *func = "svc_make_special" ;
 
    if ( ( sp = svc_new( scp ) ) == NULL )
    {
@@ -155,10 +150,10 @@ void svc_setup_address_control( struct service *sp )
 }
 
 
-PRIVATE status_e set_fd_modes( struct service *sp )
+static status_e set_fd_modes( struct service *sp )
 {
    int sd = SVC_FD( sp ) ;
-   char *func = "set_fd_modes" ;
+   const char *func = "set_fd_modes" ;
 
    /*
     * There is a possibility of blocking on a send/write if
@@ -193,42 +188,32 @@ PRIVATE status_e set_fd_modes( struct service *sp )
 
 #ifndef NO_RPC
 
-PRIVATE status_e activate_rpc( struct service *sp )
+static status_e activate_rpc( struct service *sp )
 {
-#ifdef INET6
-   struct sockaddr_in6    sin ;
-#else
-   struct sockaddr_in     sin ;
-#endif
-   int                    sin_len ;
+   union xsockaddr        tsin;
+   int                    sin_len = sizeof(tsin);
    unsigned long          vers ;
    struct service_config *scp = SVC_CONF( sp ) ;
    struct rpc_data       *rdp = SC_RPCDATA( scp ) ;
    char                  *sid = SC_ID( scp ) ;
    unsigned               registered_versions = 0 ;
    int                    sd = SVC_FD( sp ) ;
-   char                  *func = "activate_rpc" ;
+   const char            *func = "activate_rpc" ;
 
-   CLEAR( sin ) ;
-
-#ifdef INET6
-   sin.sin6_family = AF_INET6 ;
-   if( scp->sc_bind_addr != NULL )
-      memcpy(&(sin.sin6_addr), &scp->sc_bind_addr, sizeof(sin.sin6_addr));
+   if( scp->sc_bind_addr != 0 )
+      memcpy( &tsin, scp->sc_bind_addr, sizeof(tsin) );
    else
-      bzero(&sin.sin6_addr, sizeof(sin.sin6_addr));
-      
-   sin.sin6_port = 0 ;
-#else
-   sin.sin_family = AF_INET ;
-   if( scp->sc_bind_addr != NULL )
-      sin.sin_addr.s_addr = scp->sc_bind_addr->s_addr ;
-   else
-      sin.sin_addr.s_addr = htonl( INADDR_ANY ) ;
-   sin.sin_port = 0 ;         /* let the system give us a port */
-#endif /* INET6 */
+      memset( &tsin, 0, sizeof(tsin));
 
-   if ( bind( sd, SA( &sin ), sizeof( sin ) ) == -1 )
+   if( SC_IPV4( scp ) ) {
+      tsin.sa_in.sin_family = AF_INET ;
+      sin_len = sizeof(struct sockaddr_in);
+   } else if( SC_IPV6( scp ) ) {
+      tsin.sa_in6.sin6_family = AF_INET6 ;
+      sin_len = sizeof(struct sockaddr_in6);
+   }
+
+   if ( bind( sd, &tsin.sa, sin_len ) == -1 )
    {
       msg( LOG_ERR, func, "bind failed (%m). service = %s", sid ) ;
       return( FAILED ) ;
@@ -237,18 +222,17 @@ PRIVATE status_e activate_rpc( struct service *sp )
    /*
     * Find the port number that was assigned to the socket
     */
-   sin_len = sizeof( sin ) ;
-   if ( getsockname( sd, SA( &sin ), &sin_len ) == -1 )
+   if ( getsockname( sd, &tsin.sa, &sin_len ) == -1 )
    {
       msg( LOG_ERR, func,
             "getsockname failed (%m). service = %s", sid ) ;
       return( FAILED ) ;
    }
-#ifdef INET6
-   sc_set_port( scp, ntohs( sin.sin6_port ) ) ;
-#else
-   sc_set_port( scp, ntohs( sin.sin_port ) ) ;
-#endif
+   
+   if( tsin.sa.sa_family == AF_INET ) 
+      sc_set_port( scp, ntohs( tsin.sa_in.sin_port ) ) ;
+   else if( tsin.sa.sa_family == AF_INET6 )
+      sc_set_port( scp, ntohs( tsin.sa_in6.sin6_port ) ) ;
 
    /*
     * Try to register as many versions as possible
@@ -277,66 +261,48 @@ PRIVATE status_e activate_rpc( struct service *sp )
 #endif   /* ! NO_RPC */
 
 
-PRIVATE status_e activate_normal( struct service *sp )
+static status_e activate_normal( struct service *sp )
 {
-#ifdef INET6
-   struct sockaddr_in6     sin ;
-#else
-   struct sockaddr_in      sin ;
-#endif
+   union xsockaddr         tsin;
    int                     sd             = SVC_FD( sp ) ;
    struct service_config  *scp            = SVC_CONF( sp ) ;
    uint16_t                service_port   = SC_PORT( scp ) ;
    char                   *sid            = SC_ID( scp ) ;
-   char                   *func           = "activate_normal" ;
-
-#ifdef INET6
-   bzero(&sin, sizeof(sin));
-
-   sin.sin6_family = AF_INET6;
-   sin.sin6_port = htons( service_port );
-   bzero( &(sin.sin6_addr), sizeof( sin.sin6_addr ) );
+   const char             *func           = "activate_normal" ;
+   int                     sin_len        = sizeof(tsin);
+   int                     on             = 1;
 
    if( scp->sc_bind_addr != NULL )
-      memcpy(&sin.sin6_addr, scp->sc_bind_addr, sizeof(sin.sin6_addr));
+      memcpy(&tsin, scp->sc_bind_addr, sin_len);
    else
-      bzero(&sin.sin6_addr, sizeof(sin.sin6_addr));
-
-#else
-   CLEAR( sin ) ;
-
-   sin.sin_family = AF_INET ;
-   if( scp->sc_bind_addr != NULL )
-      sin.sin_addr.s_addr = scp->sc_bind_addr->s_addr ;
-   else
-      sin.sin_addr.s_addr = htonl( INADDR_ANY ) ;
-   sin.sin_port = htons( service_port ) ;
-#endif
-
-   if ( reuse_option || SC_REUSE_ADDRESS( scp ) )
-   {
-      int on = 1 ;
-
-      if ( setsockopt( sd, SOL_SOCKET, SO_REUSEADDR, (char *) &on, sizeof( on ) ) == -1 )
-         msg( LOG_WARNING, func, "setsockopt SO_REUSEADDR failed (%m). service = %s", sid ) ;
+      memset(&tsin, 0, sin_len);
+   
+   if( SC_IPV4( scp ) ) {
+      tsin.sa_in.sin_family = AF_INET ;
+      tsin.sa_in.sin_port = htons( service_port ) ;
+      sin_len = sizeof(struct sockaddr_in);
+   } else if( SC_IPV6( scp ) ) {
+      tsin.sa_in6.sin6_family = AF_INET6;
+      tsin.sa_in6.sin6_port = htons( service_port );
+      sin_len = sizeof(struct sockaddr_in6);
    }
 
+   if ( setsockopt( sd, SOL_SOCKET, SO_REUSEADDR, (char *) &on, sizeof( on ) ) == -1 )
+      msg( LOG_WARNING, func, "setsockopt SO_REUSEADDR failed (%m). service = %s", sid ) ;
+
    if( SC_NODELAY( scp ) && (scp->sc_protocol.value == IPPROTO_TCP) )
    {
-      int on = 1;
-
       if ( setsockopt( sd, IPPROTO_TCP, TCP_NODELAY, (char *) &on, sizeof(on) ) == -1 )
          msg( LOG_WARNING, func, "setsockopt TCP_NODELAY failed (%m). service = %s", sid ) ;
    }
 
    if( SC_KEEPALIVE( scp ) && (scp->sc_protocol.value == IPPROTO_TCP) ) 
    {
-      int on = 1;
       if(setsockopt(sd, SOL_SOCKET, SO_KEEPALIVE, (char *)&on, sizeof(on)) < 0 )
          msg( LOG_WARNING, func, "setsockopt SO_KEEPALIVE failed (%m). service = %s", sid ) ;
    }
 
-   if ( bind( sd, SA( &sin ), sizeof( sin ) ) == -1 )
+   if ( bind( sd, &tsin.sa, sin_len ) == -1 )
    {
       msg( LOG_ERR, func, "bind failed (%m). service = %s", sid ) ;
       return( FAILED ) ;
@@ -352,16 +318,15 @@ status_e svc_activate( struct service *sp )
 {
    struct service_config    *scp = SVC_CONF( sp ) ;
    status_e                  status ;
-   char                     *func = "svc_activate" ;
-   voidfunc                  get_shutdown_by_name() ;
+   const char                     *func = "svc_activate" ;
 
-#ifdef INET6
-   sp->svc_fd = socket( AF_INET6, 
-                        SC_SOCKET_TYPE( scp ), SC_PROTOVAL( scp ) ) ;
-#else
-   sp->svc_fd = socket( AF_INET, 
-                        SC_SOCKET_TYPE( scp ), SC_PROTOVAL( scp ) ) ;
-#endif
+   if( SC_IPV4( scp ) ) {
+      sp->svc_fd = socket( AF_INET, 
+                           SC_SOCKET_TYPE( scp ), SC_PROTOVAL( scp ) ) ;
+   } else if( SC_IPV6( scp ) ) {
+      sp->svc_fd = socket( AF_INET6, 
+                           SC_SOCKET_TYPE( scp ), SC_PROTOVAL( scp ) ) ;
+   }
 
    if ( sp->svc_fd == -1 )
    {
@@ -399,7 +364,6 @@ status_e svc_activate( struct service *sp )
     * Initialize the service data
     */
    sp->svc_running_servers   = sp->svc_retry_servers = 0 ;
-   sp->svc_shutdown_func   = get_shutdown_by_name( SC_NAME( scp ) ) ;
    sp->svc_handler_func    = svc_generic_handler ;
    svc_setup_address_control( sp ) ;
 
@@ -421,7 +385,7 @@ status_e svc_activate( struct service *sp )
 }
 
 
-PRIVATE void deactivate( struct service *sp )
+static void deactivate( const struct service *sp )
 {
    (void) close( SVC_FD( sp ) ) ;
 
@@ -432,7 +396,7 @@ PRIVATE void deactivate( struct service *sp )
    if ( SC_IS_RPC( SVC_CONF( sp ) ) )
    {
       unsigned long vers ;
-      struct rpc_data *rdp = SC_RPCDATA( SVC_CONF( sp ) ) ;
+      const struct rpc_data *rdp = SC_RPCDATA( SVC_CONF( sp ) ) ;
 
       for ( vers = RD_MINVERS( rdp ) ; vers <= RD_MAXVERS( rdp ) ; vers++ ) {
          (void) pmap_unset( RD_PROGNUM( rdp ), vers ) ;
@@ -467,13 +431,12 @@ void svc_deactivate( struct service *sp )
 }
 
 
-
 /*
  * Suspend a service
  */
 void svc_suspend( struct service *sp )
 {
-   char *func = "svc_suspend" ;
+   const char *func = "svc_suspend" ;
 
    if ( ! SVC_IS_ACTIVE( sp ) )
    {
@@ -495,7 +458,7 @@ void svc_suspend( struct service *sp )
  */
 void svc_resume( struct service *sp )
 {
-   char *func = "svc_resume" ;
+   const char *func = "svc_resume" ;
 
    FD_SET( SVC_FD( sp ), &ps.rws.socket_mask ) ;
    ps.rws.active_services++ ;
@@ -519,7 +482,7 @@ void svc_resume( struct service *sp )
 int svc_release( struct service *sp )
 {
    char *sid = SVC_ID( sp ) ;
-   char *func = "svc_release" ;
+   const char *func = "svc_release" ;
 
    if ( sp->svc_ref_count == 0 )
    {
@@ -549,7 +512,7 @@ int svc_release( struct service *sp )
 }
 
 
-void svc_dump( struct service *sp, int fd )
+void svc_dump( const struct service *sp, int fd )
 {
    tabprint( fd, 0, "Service = %s\n", SC_NAME( SVC_CONF( sp ) ) ) ;
    tabprint( fd, 1, "State = %s\n",
@@ -563,56 +526,11 @@ void svc_dump( struct service *sp, int fd )
       tabprint( fd, 1, "retry servers = %d\n", sp->svc_retry_servers ) ;
       tabprint( fd, 1, "attempts = %d\n", sp->svc_attempts ) ;
       tabprint( fd, 1, "service fd = %d\n", sp->svc_fd ) ;
-      tabprint( fd, 1, "shutdown function = %s\n",
-                     get_shutdown_by_addr( sp->svc_shutdown_func ) ) ;
    }
    Sputchar( fd, '\n' ) ;
 }
 
 
-/*
- * Returns TRUE if the server instantiation rate is over the limit
- */
-bool_int svc_looping( struct service *sp )
-{
-   time_t            current_time ;
-   register time_t   time_diff ;
-   char            *func = "svc_looping" ;
-
-   (void) time( &current_time ) ;
-
-   if ( sp->svc_attempts == 0 )
-   {
-      sp->svc_attempts++ ;
-      sp->svc_start_time = current_time ;
-      return( FALSE ) ;
-   }
-
-   time_diff = current_time - sp->svc_start_time ;
-   if ( time_diff <= LOOP_INTERVAL )
-   {
-      sp->svc_attempts++ ;
-      if ( time_diff == 0 )
-         time_diff = 1 ;
-      if ( sp->svc_attempts/time_diff > ps.ros.loop_rate )
-      {
-         FD_CLR( SVC_FD( sp ), &ps.rws.socket_mask ) ;
-         svc_deactivate( sp ) ;
-         msg( LOG_ERR, func,
-            "%s service was deactivated because of looping", SVC_ID( sp ) ) ;
-         return( TRUE ) ;
-      }
-   }
-   else
-   {
-      sp->svc_start_time = current_time ;
-      sp->svc_attempts = 1 ;
-   }
-   return( FALSE ) ;
-}
-
-
-
 void svc_request( struct service *sp )
 {
    connection_s *cp ;
@@ -620,34 +538,18 @@ void svc_request( struct service *sp )
    cp = conn_new( sp ) ;
    if ( cp == CONN_NULL )
       return ;
-   if ( svc_handle( sp, cp ) == OK ) 
-      return ;
-   else
+
+   if ( svc_handle( sp, cp ) != OK ) 
    {
       conn_cleanup( cp ) ;
       if ( conn_start_alternative( cp ) == FAILED )
-         conn_free( cp ) ;
+         conn_free( cp, 1 ) ;
    }
 }
 
 
-
-PRIVATE status_e svc_generic_handler( struct service *sp, connection_s *cp )
+static status_e svc_generic_handler( struct service *sp, connection_s *cp )
 {
-   struct service_config *scp = SVC_CONF( sp ) ;
-
-   /*
-    * There is no loop check for services that accept connections because
-    * the connections are accepted by xinetd, therefore the queue will get
-    * empty eventually even if the server is faulty.
-    * Services for which no fork(2) occurs are not checked because
-    *    a) they are internal and we assume they are not faulty
-    *    b) we have no way of determining if something is going wrong
-    *       for example, UDP discard can do its job very quickly
-    */
-   if ( !SC_ACCEPTS_CONNECTIONS( scp ) && SVC_FORKS( sp ) && svc_looping( sp ) )
-      return( FAILED ) ;
-
    if ( svc_parent_access_control( sp, cp ) == OK ) {
       return( server_run( sp, cp ) ) ;
    }
@@ -657,7 +559,6 @@ PRIVATE status_e svc_generic_handler( struct service *sp, connection_s *cp )
     * The services must be added in the proper order because they
     * are invoked in that order:
     *         logging
-    *         shutdown
     */
 
    /* Note that this done in the parent process.  Logging the
@@ -667,22 +568,15 @@ PRIVATE status_e svc_generic_handler( struct service *sp, connection_s *cp )
    if ( SVC_LOGS_USERID_ON_FAILURE( sp ) )
       (void) conn_add_alternative( cp, LOG_SERVICE( ps ) ) ;
 
-   if ( sp->svc_shutdown_func != NULL )
-   {
-      if ( ! SVC_RECORDS( sp ) || 
-            conn_add_alternative( cp, SHUTDOWN_SERVICE( ps ) ) == FAILED )
-         conn_shutdown( cp ) ;
-   }
-
    return( FAILED ) ;
 }
 
 #define TMPSIZE 1024
 /* Print the banner that is supposed to always be printed */
-PRIVATE int banner_always( struct service *sp, connection_s *cp )
+static int banner_always( const struct service *sp, const connection_s *cp )
 {
-   char *func = "banner_always";
-   struct service_config *scp = SVC_CONF( sp ) ;
+   const char *func = "banner_always";
+   const struct service_config *scp = SVC_CONF( sp ) ;
 
    /* print the banner regardless of access control */
    if ( scp->sc_banner != NULL ) {
@@ -691,7 +585,7 @@ PRIVATE int banner_always( struct service *sp, connection_s *cp )
       int bannerfd = open(scp->sc_banner, O_RDONLY);
 
       if( bannerfd < 0 ) {
-         msg( LOG_ERR, func, "service = %s, open of banner %s failed\n", SVC_ID( sp ), scp->sc_banner);
+         msg( LOG_ERR, func, "service = %s, open of banner %s failed", SVC_ID( sp ), scp->sc_banner);
          return(-1);
       }
 
@@ -707,8 +601,7 @@ PRIVATE int banner_always( struct service *sp, connection_s *cp )
                break;
             }
          }
-         Sprint(cp->co_descriptor, "%s", tmpbuf);
-         Sflush(cp->co_descriptor);
+         write(cp->co_descriptor, tmpbuf, retval);
          if( retval < TMPSIZE )
             break;
 
@@ -721,10 +614,11 @@ PRIVATE int banner_always( struct service *sp, connection_s *cp )
    return(0);
 }
 
-PRIVATE int banner_fail( struct service *sp, connection_s *cp )
+static int banner_fail( const struct service *sp, const connection_s *cp )
 {
-   char *func = "banner_fail";
-   struct service_config *scp = SVC_CONF( sp ) ;
+   const char *func = "banner_fail";
+   const struct service_config *scp = SVC_CONF( sp ) ;
+
 
    if ( scp->sc_banner_fail != NULL )
    {
@@ -734,14 +628,13 @@ PRIVATE int banner_fail( struct service *sp, connection_s *cp )
 
       if( bannerfd < 0 )
       {
-         msg( LOG_ERR, func, "service = %s, open of banner %s failed\n", 
+         msg( LOG_ERR, func, "service = %s, open of banner %s failed", 
             SVC_ID( sp ), scp->sc_banner_fail);
          return(-1);
       }
 
-      bzero(tmpbuf, TMPSIZE);
-      while( (retval = read(bannerfd, tmpbuf, TMPSIZE)) )
-      {
+      memset(tmpbuf, 0, sizeof(tmpbuf));
+      while( (retval = read(bannerfd, tmpbuf, sizeof(tmpbuf))) ) {
          if (retval == -1)
          {
             if (errno == EINTR)
@@ -752,12 +645,13 @@ PRIVATE int banner_fail( struct service *sp, connection_s *cp )
                break;
             }
          }
-         Sprint(cp->co_descriptor, "%s", tmpbuf);
-         Sflush(cp->co_descriptor);
+         write(cp->co_descriptor, tmpbuf, retval);
 
-         if( retval < TMPSIZE )
-            break;
-         bzero(tmpbuf, TMPSIZE);
+         if( retval < sizeof(tmpbuf) ) {
+            close(bannerfd);
+            return(0);
+         }
+         memset(tmpbuf, 0, sizeof(tmpbuf));
       }
 
       close(bannerfd);
@@ -766,10 +660,10 @@ PRIVATE int banner_fail( struct service *sp, connection_s *cp )
    return(0);
 }
 
-PRIVATE int banner_success( struct service *sp, connection_s *cp )
+static int banner_success( const struct service *sp, const connection_s *cp )
 {
-   char *func = "banner_success";
-   struct service_config *scp = SVC_CONF( sp ) ;
+   const char *func = "banner_success";
+   const struct service_config *scp = SVC_CONF( sp ) ;
 
    /* print the access granted banner */
    if ( scp->sc_banner_success != NULL ) {
@@ -778,7 +672,7 @@ PRIVATE int banner_success( struct service *sp, connection_s *cp )
       int bannerfd = open(scp->sc_banner_success, O_RDONLY);
    
       if( bannerfd < 0 ) {
-         msg( LOG_ERR, func, "service = %s, open of banner %s failed\n", SVC_ID( sp ), scp->sc_banner_success);
+         msg( LOG_ERR, func, "service = %s, open of banner %s failed", SVC_ID( sp ), scp->sc_banner_success);
          return(-1);
       }
    
@@ -794,8 +688,7 @@ PRIVATE int banner_success( struct service *sp, connection_s *cp )
                break;
             }
          }
-         Sprint(cp->co_descriptor, "%s", tmpbuf);
-         Sflush(cp->co_descriptor);
+         write(cp->co_descriptor, tmpbuf, retval);
 
          if( retval < TMPSIZE )
             break;
@@ -807,10 +700,11 @@ PRIVATE int banner_success( struct service *sp, connection_s *cp )
    return(0);
 }
 
-PRIVATE status_e failed_service(struct service *sp, 
+static status_e failed_service(struct service *sp, 
                                 connection_s *cp, 
                                 access_e result)
 {
+   struct service_config *scp = SVC_CONF( sp ) ;
 
    if ( result != AC_OK )
    {
@@ -825,33 +719,50 @@ PRIVATE status_e failed_service(struct service *sp,
        */
       if ( result == AC_ADDRESS && SVC_SOCKET_TYPE( sp ) == SOCK_DGRAM )
       {
-#ifdef INET6
-         struct sockaddr_in6 *sinp = conn_address( cp ) ;
-         struct sockaddr_in6 *last = &sp->svc_last_dgram_addr ;
-#else
-         struct sockaddr_in *sinp = conn_address( cp ) ;
-         struct sockaddr_in *last = &sp->svc_last_dgram_addr ;
-#endif
-         time_t current_time ;
+         if( SC_IPV4( scp ) ) {
+            struct sockaddr_in *sinp = SAIN(conn_address( cp )) ;
+            struct sockaddr_in *last = SAIN(sp->svc_last_dgram_addr) ;
+            time_t current_time ;
 
-         (void) time( &current_time ) ;
+            if (sinp == NULL )
+               return FAILED;
 
-#ifdef INET6
-         if ( IN6_ARE_ADDR_EQUAL(&(sinp->sin6_addr), &(last->sin6_addr)) && sinp->sin6_port == last->sin6_port )
-#else
-         if ( sinp->sin_addr.s_addr == last->sin_addr.s_addr &&
+            if( sp->svc_last_dgram_addr == NULL ) {
+               sp->svc_last_dgram_addr = (struct sockaddr *)malloc( sizeof(struct sockaddr_in) );
+            }
+
+            (void) time( &current_time ) ;
+            if ( sinp->sin_addr.s_addr == last->sin_addr.s_addr &&
                                           sinp->sin_port == last->sin_port )
-#endif
-         {
-            if ( current_time - sp->svc_last_dgram_time <= DGRAM_IGNORE_TIME )
-               report_failure = FALSE ;
+            {
+               if ( current_time - sp->svc_last_dgram_time <= DGRAM_IGNORE_TIME )
+                  report_failure = FALSE ;
+               else
+                  sp->svc_last_dgram_time = current_time ;
+            }
             else
+            {
+               memcpy(sp->svc_last_dgram_addr, sinp, sizeof(struct sockaddr_in));
                sp->svc_last_dgram_time = current_time ;
-         }
-         else
-         {
-            sp->svc_last_dgram_addr = *sinp ;
-            sp->svc_last_dgram_time = current_time ;
+            }
+         } else if( SC_IPV6( scp ) ) {
+            struct sockaddr_in6 *sinp = SAIN6(conn_address( cp )) ;
+            struct sockaddr_in6 *last = SAIN6(sp->svc_last_dgram_addr) ;
+            time_t current_time ;
+
+            (void) time( &current_time ) ;
+            if ( IN6_ARE_ADDR_EQUAL(&(sinp->sin6_addr), &(last->sin6_addr)) && sinp->sin6_port == last->sin6_port )
+            {
+               if ( current_time - sp->svc_last_dgram_time <= DGRAM_IGNORE_TIME )
+                  report_failure = FALSE ;
+               else
+                  sp->svc_last_dgram_time = current_time ;
+            }
+            else
+            {
+               memcpy(sp->svc_last_dgram_addr, sinp, sizeof(struct sockaddr_in6));
+               sp->svc_last_dgram_time = current_time ;
+            }
          }
       }
 
@@ -894,58 +805,6 @@ status_e svc_child_access_control( struct service *sp, connection_s *cp )
    return( OK ) ;
 }
 
-/*
- * Implement shutdown protocol (usually sends an error indication).
- */
-void svc_shutdown( struct service *sp, connection_s *cp, char **msgp )
-{
-   int                  old_flags = 0;
-   bool_int             no_block ;
-   int                  fd        = CONN_DESCRIPTOR( cp ) ;
-   char                 *func     = "svc_shutdown" ;
-
-#ifdef DEBUG
-   if ( sp != CONN_SERVICE( cp ) )
-   {
-      msg( LOG_CRIT, func, "service = %s, service of connection = %s",
-         SVC_ID( sp ), SVC_ID( CONN_SERVICE( cp ) ) ) ;
-      return ;
-   }
-#endif
-
-   if ( sp->svc_shutdown_func == NULL )
-      return ;
-
-   no_block = ( msgp == NULL ) ;
-
-   /*
-    * Ensure that we won't block in the shutdown function
-    */
-   if ( no_block )
-   {
-      if ( ( old_flags = fcntl( fd, F_GETFL, 0 ) ) == -1 )
-      {
-         msg( LOG_ERR, func, "fcntl-getflags failed: %m" ) ;
-         return ;
-      }
-
-      if ( fcntl( fd, F_SETFL, FNDELAY ) == -1 )
-      {
-         msg( LOG_ERR, func, "fcntl-setflags failed: %m" ) ;
-         return ;
-      }
-   }
-
-   (*sp->svc_shutdown_func)( fd, msgp ) ;
-
-   /*
-    * Don't bother with restoring the flags if this is a new descriptor
-    */
-   if ( no_block && SC_ACCEPTS_CONNECTIONS( SVC_CONF( sp ) ) )
-      (void) fcntl( fd, F_SETFL, old_flags ) ;
-}
-
-
 /*
  * Invoked when a server of the specified service dies
  */
@@ -985,6 +844,6 @@ void svc_postmortem( struct service *sp, struct server *serp )
    if ( co_sp != sp && SVC_IS_LOGGING( co_sp ) )
       xlog_control( SVC_LOG( co_sp ), XLOG_SIZECHECK ) ;
 
-   conn_free( cp ) ;
+   conn_free( cp, 1 ) ;
 }
 
