@@ -8,17 +8,21 @@
 #ifndef SERVICE_H
 #define SERVICE_H
 
+#include "config.h"
 #include <sys/types.h>
 #include <netinet/in.h>
+#include <time.h>
+
+#include "pset.h"
+#include "xlog.h"
+#include "defs.h"
+#include "server.h"
 
 /*
  * $Id: service.h,v 1.2 1999/10/13 04:22:34 bbraun Exp $
  */
 
-#include "xlog.h"
 
-#include "defs.h"
-#include "sconf.h"
 
 /*
  * NOTE: A service can be disabled but not deleted if it has any servers
@@ -45,14 +49,9 @@ struct service
    unsigned               svc_running_servers ;
    unsigned               svc_retry_servers ;
    unsigned               svc_attempts ; /* # of attempts to start server*/
-   time_t                 svc_start_time ;   /* since this time */
    /*
-    * svc_shutdown: invoked to playback the protocol and get information
-    *      from the other end. It also sets up the connection
-    *      for an orderly close.
     * svc_handler:   invoked to handle new requests
     */
-   voidfunc               svc_shutdown_func ; /* ARGS:int sd, char**msgp */
    statfunc               svc_handler_func ;/*ARGS:service *,connection **/
    /*
     * These fields are used for access control; they either point to the
@@ -65,17 +64,16 @@ struct service
     * These fields are used to avoid generating too many messages when
     * receiving datagrams from a bad address.
     */
-#ifdef INET6
-   struct sockaddr_in6                     svc_last_dgram_addr ;
-#else
-   struct sockaddr_in                      svc_last_dgram_addr ;
-#endif
+   struct sockaddr                        *svc_last_dgram_addr ;
    time_t                                  svc_last_dgram_time ;
    xlog_h                                  svc_log ;
 } ;
 
 
 #define SP( p )                  ( (struct service *) (p) )
+#define SUSPEND( sp )          (sp)->svc_state = SVC_SUSPENDED
+#define RESUME( sp )           (sp)->svc_state = SVC_ACTIVE
+
 
 /*
  * Field access macros
@@ -97,7 +95,6 @@ struct service
 /*
  * Predicate checking macros
  */
-#define SVC_LOGUSER( sp )          SC_LOGUSER( SVC_CONF( sp ) )
 #define SVC_FORKS( sp )            SC_FORKS( SVC_CONF( sp ) )
 #define SVC_RETRY( sp )            SC_RETRY( SVC_CONF( sp ) )
 #define SVC_WAITS( sp )            SC_WAITS( SVC_CONF( sp ) )
@@ -133,7 +130,7 @@ struct service
 
 #define svc_dec_running_servers( sp )                                         \
    {                                                                          \
-      if ( SVC_RUNNING_SERVERS( sp ) > 0 )                                    \
+      if ( SVC_RUNNING_SERVERS( sp ) != 0 )                                   \
          (sp)->svc_running_servers-- ;                                        \
       else                                                                    \
          msg( LOG_ERR, func,                                                  \
@@ -145,4 +142,20 @@ struct service
 #define svc_inc_retries( sp )               (sp)->svc_retry_servers++
 #define svc_dec_retries( sp )               (sp)->svc_retry_servers--
 
+struct service *svc_new(struct service_config *scp);
+struct service *svc_make_special(struct service_config *scp,statfunc handler);
+void svc_free(struct service *sp);
+void svc_setup_address_control(struct service *sp);
+status_e svc_activate(struct service *sp);
+void svc_deactivate(struct service *sp);
+void svc_suspend(struct service *sp);
+void svc_resume(struct service *sp);
+int svc_release(struct service *sp);
+void svc_dump(const struct service *sp,int fd);
+void svc_request(struct service *sp);
+status_e svc_parent_access_control(struct service *sp,connection_s *cp);
+status_e svc_child_access_control(struct service *sp,connection_s *cp);
+void svc_postmortem(struct service *sp,struct server *serp);
+
+
 #endif   /* SERVICE_H */
