@@ -5,36 +5,43 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: signals.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
+#if defined(HAVE_STRSIGNAL)
+#define _GNU_SOURCE
+#endif
 #include <sys/types.h>
 #include <sys/time.h>
 #include <signal.h>
 #include <syslog.h>
 #include <errno.h>
-#if defined(linux)
-#define __USE_GNU
-#endif
 #include <string.h>
 #include <unistd.h>
 #include <time.h>
+#include <sys/ioctl.h>
+#ifdef HAVE_SYS_FILIO_H
+#include <sys/filio.h>
+#endif
 
 #include "str.h"
-
-#include "state.h"
-#include "defs.h"
-#include "xconfig.h"
-#include "flags.h"
 #include "signals.h"
+#include "xconfig.h"
+#include "msg.h"
+#include "main.h"
+#include "xtimer.h"
+#include "child.h"
+#include "retry.h"
+#include "reconfig.h"
+#include "internals.h"
 
-PRIVATE void my_handler( int sig );
+static void my_handler( int sig );
+static void general_handler( int sig );
 
-PRIVATE void general_handler( int sig );
+typedef void sigfunc( int );
 
 #define SIGSET_NULL            ((sigset_t *)0)
 #define SIGVEC_NULL            ((struct sigvec *)0)
-#define SIGACTION_NULL            ((struct sigaction *)0)
+#define SIGACTION_NULL         ((struct sigaction *)0)
 
 #ifdef NO_POSIX_SIGS
 
@@ -49,7 +56,7 @@ typedef int sigset_t ;
 
 struct sigaction
 {
-   void         (*sa_handler)() ;
+   void         (*sa_handler)(int) ;
    sigset_t     sa_mask ;
    int          sa_flags ;
 } ;
@@ -155,22 +162,19 @@ static int nsig ;
  * if it failed because the signal number specified was invalid.
  * This allows us to determine the number of supported signals.
  */
-PRIVATE status_e handle_signal( int sig )
+static status_e handle_signal( int sig )
 {
    struct sigaction      sa ;
-   voidfunc              sig_handler ;
+   sigfunc              *sig_handler ;
 
    switch ( sig )
    {
-      case RECONFIG_SOFT_SIG:
       case RECONFIG_HARD_SIG:
+      case OLD_RECONFIG_HARD_SIG:
       case TERMINATION_SIG:
       case STATE_DUMP_SIG:
       case CONSISTENCY_CHECK_SIG:
       case SERVER_EXIT_SIG:
-#ifdef NO_TIMERS
-      case RETRY_SIG:
-#endif
       case QUIT_SIG:
          sig_handler = my_handler ;
          break ;
@@ -228,15 +232,17 @@ PRIVATE status_e handle_signal( int sig )
  * Install signal handlers for all signals that can be caught.
  * This implies that no core dumps are generated by default.
  */
-status_e signal_init()
+status_e signal_init(void)
 {
    int      sig ;
-   char    *func = "install_signal_handlers" ;
+   const char    *func = "install_signal_handlers" ;
 
    sigemptyset( &reset_sigs ) ;
-#ifndef NO_TIMERS
-   sigaddset( &reset_sigs, SIGALRM ) ;
-#endif
+
+   if ( pipe(signals_pending) != 0 ) {
+      msg( LOG_CRIT, func, "Failed to create signal pipe: %m" );
+      return( FAILED );
+   }
 
    for ( sig = 1 ;; sig++ )
       if ( handle_signal( sig ) == FAILED ) {
@@ -257,7 +263,7 @@ status_e signal_init()
 }
 
 
-void signal_wait()
+void signal_wait(void)
 {
    sigset_t masked_sigs ;
 
@@ -286,13 +292,13 @@ void signal_wait()
  * NOTE: We try to send a message to the log only once to avoid
  *         looping in this function (in case there is a bug in msg())
  */
-PRIVATE void bad_signal()
+static void bad_signal(void)
 {
    static time_t   interval_start ;
-   static int      interval_signal_count ;
-   static int      total_signal_count ;
+   static volatile int interval_signal_count ;
+   static volatile int total_signal_count ;
    time_t          current_time ;
-   char            *func = "bad_signal" ;
+   const char     *func = "bad_signal" ;
 
    total_signal_count++ ;
    if ( total_signal_count == MAX_SIGNAL_COUNT )
@@ -317,7 +323,7 @@ PRIVATE void bad_signal()
             interval_start = current_time ;
             interval_signal_count = 1 ;
             msg( LOG_ERR, func, "Resetting..." ) ;
-            longjmp( ps.rws.env, 1 ) ;
+            siglongjmp( ps.rws.env, 1 ) ;
             /* NOTREACHED */
          }
          msg( LOG_CRIT, func,
@@ -342,10 +348,16 @@ char *sig_name( int sig )
 #if defined(HAVE_STRSIGNAL)
    /* Use strsignal and remove the old sys_siglist stuff */
    if ( sig < NSIG )
-      return( strx_sprint( signame_buf, sizeof( signame_buf ),
+      return( strx_sprint( signame_buf, sizeof( signame_buf ) - 1,
                "%d (%s)", sig, strsignal(sig) ) ) ;
+#else
+#if defined(HAVE_SYS_SIGLIST)
+   if ( sig < NSIG )
+      return( strx_sprint( signame_buf, sizeof( signame_buf ) - 1,
+               "%d (%s)", sig, sys_siglist[sig] ) ) ;
 #endif
-   return( strx_sprint( signame_buf, sizeof( signame_buf ), "%d", sig ) ) ;
+#endif
+   return( strx_sprint( signame_buf, sizeof( signame_buf ) - 1, "%d", sig ) ) ;
 }
 
 
@@ -357,10 +369,10 @@ char *sig_name( int sig )
  * cause termination.
  */
 
-PRIVATE void general_handler( int sig )
+static void general_handler( int sig )
 {
    sigset_t badsigs ;
-   char *func = "general_handler" ;
+   const char *func = "general_handler" ;
 
    /*
     * Do this here to catch problems like SIGSEGV in msg()
@@ -386,49 +398,12 @@ PRIVATE void general_handler( int sig )
 
 
 /*
- * The job of this function is to set the flag that corresponds to the
- * received signal. No other action is taken.
+ * The job of this function is to write the signal recieved to the 
+ * pipe of pending signals, which is in the main select loop.
  */
-PRIVATE void my_handler( int sig )
+static void my_handler( int sig )
 {
-   /*
-    * Signals are listed most-frequent-first
-    */
-   switch( sig )
-   {
-      case SERVER_EXIT_SIG:
-         M_SET( ps.flags, CHILD_FLAG ) ;
-         break ;
-      
-      case RETRY_SIG:
-         M_SET( ps.flags, RETRY_FLAG ) ;
-         break ;
-      
-      case RECONFIG_SOFT_SIG:
-         M_SET( ps.flags, SOFT_RECONFIG_FLAG ) ;
-         break ;
-      
-      case RECONFIG_HARD_SIG:
-         M_SET( ps.flags, HARD_RECONFIG_FLAG ) ;
-         break ;
-      
-      case TERMINATION_SIG:
-         M_SET( ps.flags, TERMINATE_FLAG ) ;
-         break ;
-      
-      case STATE_DUMP_SIG:
-         M_SET( ps.flags, DUMP_FLAG ) ;
-         break ;
-      
-      case CONSISTENCY_CHECK_SIG:
-         M_SET( ps.flags, CONSISTENCY_FLAG ) ;
-         break ;
-      
-      case QUIT_SIG:
-         M_SET( ps.flags, QUIT_FLAG ) ;
-         break ;
-   }
-
+   write(signals_pending[1], &sig, sizeof(int));
 }
 
 
@@ -438,7 +413,7 @@ PRIVATE void my_handler( int sig )
  * This function is invoked from a forked process. That is why we
  * invoke _exit instead of exit (to avoid the possible stdio buffer flushes)
  */
-void signal_default_state()
+void signal_default_state(void)
 {
    int sig ;
    sigset_t empty ;
@@ -457,4 +432,35 @@ void signal_default_state()
    (void) sigprocmask( SIG_SETMASK, &empty, SIGSET_NULL ) ;
 }
 
+void check_pipe(void)
+{
+   int i, tmp;
+   const char *func = "check_pipe";
 
+   if( ioctl(signals_pending[0], FIONREAD, &i) != 0 ) {
+      msg(LOG_ERR, func, "Can't get the number of pending signals: %m");
+      return;
+   }
+   i /= sizeof(int);
+
+   while( --i >= 0 ) {
+      if( read(signals_pending[0], &tmp, sizeof(int)) != sizeof(int) ) {
+         msg(LOG_ERR, func, "Error retrieving pending signal: %m");
+         return;
+      }
+
+      if( debug.on ) {
+         msg(LOG_DEBUG, func, "Got signal %s", sig_name(tmp));
+      }
+
+      switch(tmp) {
+         case SERVER_EXIT_SIG:       child_exit();           break;
+         case RECONFIG_HARD_SIG:     hard_reconfig();        break;
+         case OLD_RECONFIG_HARD_SIG: hard_reconfig();        break;
+         case TERMINATION_SIG:       terminate_program();    break;
+         case STATE_DUMP_SIG:        dump_internal_state();  break;
+         case CONSISTENCY_CHECK_SIG: user_requested_check(); break;
+         case QUIT_SIG:              quit_program();         break;
+      }
+   }
+}
