@@ -5,7 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: tcpint.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
@@ -22,12 +21,12 @@ static char RCSid[] = "$Id: tcpint.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $"
 #include <sys/select.h>
 #endif
 
-#include "xconfig.h"
-#include "int.h"
-#include "access.h"
-#include "defs.h"
-#include "servicedefs.h"
+#include "tcpint.h"
+#include "intcommon.h"
+#include "msg.h"
 #include "log.h"
+#include "xconfig.h"
+#include "sconf.h"
 
 typedef enum { S_OK, S_SERVER_ERR, S_CLIENT_ERR } stream_status_e ;
 
@@ -39,9 +38,9 @@ struct istream_private
 #define SIP( p )                  ((struct istream_private *)(p))
 
 static struct istream_private istream ;
+static struct intercept_s stream_intercept_state ;
+static void si_mux(void) ;
 
-PRIVATE void si_mux() ;
-void si_exit() ;
 
 static struct intercept_ops istream_ops =
    {
@@ -49,12 +48,11 @@ static struct intercept_ops istream_ops =
       si_exit
    } ;
 
-static struct intercept stream_intercept_state ;
 
 
-struct intercept *si_init( struct server *serp )
+struct intercept_s *si_init( struct server *serp )
 {
-   struct intercept *ip = &stream_intercept_state ;
+   struct intercept_s *ip = &stream_intercept_state ;
 
    ip->int_socket_type = SOCK_STREAM ;
    ip->int_priv = (void *) &istream ;
@@ -65,15 +63,15 @@ struct intercept *si_init( struct server *serp )
    return( ip ) ;
 }
 
-PRIVATE status_e handle_io( psi_h iter, channel_s *chp, fd_set *maskp, stream_status_e (*iofunc)() );
-PRIVATE stream_status_e tcp_local_to_remote( channel_s *chp );
-PRIVATE stream_status_e tcp_remote_to_local( channel_s *chp );
-PRIVATE void connection_request( struct intercept *ip, channel_s **chpp );
+static status_e handle_io( psi_h iter, channel_s *chp, fd_set *maskp, stream_status_e (*iofunc)() );
+static stream_status_e tcp_local_to_remote( channel_s *chp );
+static stream_status_e tcp_remote_to_local( channel_s *chp );
+static void connection_request( struct intercept_s *ip, channel_s **chpp );
 
 /* Unfortunatly, this can't be private... */
-void si_exit()
+void si_exit(void)
 {
-   struct intercept *ip = &stream_intercept_state ;
+   struct intercept_s *ip = &stream_intercept_state ;
    
    if ( SIP( ip->int_priv )->accepted_connections == 0 )
       (void) accept( INT_REMOTE( ip ), SA( NULL ), INT_NULL ) ;
@@ -81,13 +79,13 @@ void si_exit()
 }
 
 
-PRIVATE void si_mux()
+static void si_mux(void)
 {
-   struct intercept   *ip = &stream_intercept_state ;
+   struct intercept_s   *ip = &stream_intercept_state ;
    fd_set                     socket_mask ;
    int                        mask_max ;
-   psi_h                        iter ;
-   char                        *func = "si_mux" ;
+   psi_h                      iter ;
+   const char                *func = "si_mux" ;
 
    FD_ZERO( &socket_mask ) ;
    FD_SET( INT_REMOTE( ip ), &socket_mask ) ;
@@ -160,12 +158,12 @@ PRIVATE void si_mux()
 }
 
 
-PRIVATE status_e handle_io( psi_h iter, 
+static status_e handle_io( psi_h iter, 
                             channel_s *chp, 
                             fd_set *maskp, 
                             stream_status_e (*iofunc)() )
 {
-   char *func = "handle_io" ;
+   const char *func = "handle_io" ;
 
    switch ( (*iofunc)( chp ) )
    {
@@ -177,12 +175,7 @@ PRIVATE status_e handle_io( psi_h iter,
          if ( debug.on )
             msg( LOG_DEBUG, func,
                "Closing channel to %s,%d using sockets %d(l),%d(r)",
-                  xntoa( SA( &chp->ch_from ) ),
-#ifdef INET6
-                     ntohs( chp->ch_from.sin6_port ),
-#else
-                     ntohs( chp->ch_from.sin_port ),
-#endif
+                  xaddrname( &chp->ch_from ), ntohs(xaddrport( &chp->ch_from )),
                         chp->ch_local_socket, chp->ch_remote_socket ) ;
 
          FD_CLR( chp->ch_local_socket, maskp ) ;
@@ -199,22 +192,20 @@ PRIVATE status_e handle_io( psi_h iter,
 }
 
 
-PRIVATE void connection_request( struct intercept *ip, channel_s **chpp )
+static void connection_request( struct intercept_s *ip, channel_s **chpp )
 {
-#ifdef INET6
-   struct sockaddr_in6  csin ;
-   uint16_t            *port = &csin.sin6_port;
-#else
-   struct sockaddr_in   csin ;
-   uint16_t            *port = &csin.sin_port;
-#endif
-   int                  sin_len = sizeof( csin ) ;
+   union xsockaddr      csin ;
+   int                  sin_len = 0;
    channel_s           *chp ;
    int                  sd ;
    bool_int             addr_checked ;
-   char                *func = "connection_request" ;
+   const char          *func = "connection_request" ;
 
    *chpp = NULL ;
+   if( SC_IPV4( SVC_CONF( SERVER_SERVICE( INT_SERVER( ip ) ) ) ) )
+      sin_len = sizeof(struct sockaddr_in);
+   if( SC_IPV6( SVC_CONF( SERVER_SERVICE( INT_SERVER( ip ) ) ) ) )
+      sin_len = sizeof(struct sockaddr_in6);
 
    if ( ( sd = accept( INT_REMOTE( ip ), SA( &csin ), &sin_len ) ) == -1 ) 
       return ;
@@ -223,7 +214,7 @@ PRIVATE void connection_request( struct intercept *ip, channel_s **chpp )
 
    if ( debug.on )
       msg( LOG_DEBUG, func, "connection request from %s,%d",
-         xntoa( SA( &csin ) ), ntohs( *port ) ) ;
+         xaddrname( &csin ), ntohs( xaddrport( &csin ) ) ) ;
 
    chp = int_lookupconn( ip, &csin, &addr_checked ) ;
    if ( chp == NULL )
@@ -275,17 +266,17 @@ PRIVATE void connection_request( struct intercept *ip, channel_s **chpp )
    else
       msg( LOG_ERR, func,
          "Received another connection request from %s,%d",
-            xntoa( SA(&csin) ), ntohs( *port ) ) ;
+            xaddrname( &csin ), ntohs( xaddrport( &csin ) ) ) ;
 }
 
 
-PRIVATE stream_status_e tcp_local_to_remote( channel_s *chp )
+static stream_status_e tcp_local_to_remote( channel_s *chp )
 {
    char  buf[ DATAGRAM_SIZE ] ;
    int   rcc, wcc ;
    char *p ;
    int   left ;
-   char *func = "tcp_local_to_remote" ;
+   const char *func = "tcp_local_to_remote" ;
 
    for ( ;; )
    {
@@ -332,13 +323,13 @@ PRIVATE stream_status_e tcp_local_to_remote( channel_s *chp )
 }
 
 
-PRIVATE stream_status_e tcp_remote_to_local( channel_s *chp )
+static stream_status_e tcp_remote_to_local( channel_s *chp )
 {
    char  buf[ DATAGRAM_SIZE ] ;
    int   rcc, wcc ;
    int   left ;
    char *p ;
-   char *func = "tcp_remote_to_local" ;
+   const char *func = "tcp_remote_to_local" ;
 
    for ( ;; )
    {
