@@ -5,7 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: time.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
@@ -13,12 +12,11 @@ static char RCSid[] = "$Id: time.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 #include <ctype.h>
 #include <time.h>
 
-#include "pset.h"
-
 #include "sio.h"
-#include "defs.h"
-#include "log.h"
-#include "parse.h"
+#include "timex.h"
+#include "msg.h"
+#include "util.h"
+
 
 #define IN_RANGE( val, low, high )     ( (low) <= (val) && (val) <= (high) )
 
@@ -29,7 +27,6 @@ struct time_interval
 } ;
 
 #define TIP( p )               ( (struct time_interval *) (p) )
-
 #define NEW_TI()               NEW( struct time_interval )
 #define FREE_TI( tip )         FREE( tip )
 
@@ -37,7 +34,7 @@ struct time_interval
 /*
  * Returns TRUE if the current time is within at least one of the intervals
  */
-bool_int ti_current_time_check( pset_h intervals )
+bool_int ti_current_time_check( const pset_h intervals )
 {
    time_t      current_time ;
    unsigned    u ;
@@ -60,33 +57,33 @@ bool_int ti_current_time_check( pset_h intervals )
 }
 
 
-PRIVATE char *get_num( int *nump, 
+static int get_num( int *nump, 
                        int min_val, 
                        int max_val, 
-                       char *s, 
+                       const char *s, 
                        char stop_char )
 {
-   char *p = s ;
-   char *func = "get_num" ;
+   const char *func = "get_num" ;
+   int i = 0;
 
-   for ( *nump = 0 ; isdigit( *p ) ; p++ )
+   for ( *nump = 0 ; isdigit( s[i] ) ; i++ )
    {
       *nump *= 10 ;
-      *nump += *p - '0' ;
+      *nump += s[i] - '0' ;
    }
 
-   if ( *p != stop_char )
+   if ( s[i] != stop_char )
    {
       parsemsg( LOG_ERR, func, "incorrect time interval" ) ;
-      return( NULL );
+      return( -1 );
    }
 
    if ( ! IN_RANGE( *nump, min_val, max_val ) )
    {
       parsemsg( LOG_ERR, func, "invalid time interval" ) ;
-      return( NULL ) ;
+      return( -1 ) ;
    }
-   return( p+1 ) ;
+   return( i ) ;
 }
 
 
@@ -96,25 +93,25 @@ PRIVATE char *get_num( int *nump,
  *    hour:min-hour:min
  * Example: 2:30-4:15
  */
-status_e ti_add( pset_h iset, char *interval_str )
+status_e ti_add( pset_h iset, const char *interval_str )
 {
    struct time_interval   *tip ;
    int                  hours ;
    int                  minutes ;
-   char                 *p ;
    int                  min_start ;
    int                  min_end ;
-   char                  *func = "add_interval" ;
+   int                  r ;
+   const char          *func = "add_interval" ;
 
-   if ( ( p = get_num( &hours, 0, 23, interval_str, ':' ) ) == NULL )
+   if ( ( r = get_num( &hours, 0, 23, interval_str, ':' ) ) == -1 )
       return( OK ) ;
-   if ( ( p = get_num( &minutes, 0, 59, p, '-' ) ) == NULL )
+   if ( ( r = get_num( &minutes, 0, 59, interval_str+r, '-' ) ) == -1 )
       return( OK ) ;
    min_start = hours * 60 + minutes ;
 
-   if ( ( p = get_num( &hours, 0, 23, p, ':' ) ) == NULL )
+   if ( ( r = get_num( &hours, 0, 23, interval_str+r, ':' ) ) == -1 )
       return( OK ) ;
-   if ( ( p = get_num( &minutes, 0, 59, p, NUL ) ) == NULL )
+   if ( ( r = get_num( &minutes, 0, 59, interval_str+r, NUL ) ) == -1 )
       return( OK ) ;
    min_end = hours * 60 + minutes ;
    if ( min_start >= min_end )
