@@ -5,8 +5,6 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: udpint.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
-
 #include "config.h"
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -21,11 +19,14 @@ static char RCSid[] = "$Id: udpint.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $"
 #include <sys/select.h>
 #endif
 
+#include "udpint.h"
+#include "intcommon.h"
+#include "util.h"
+#include "connection.h"
 #include "access.h"
-#include "defs.h"
-#include "int.h"
-#include "servicedefs.h"
 #include "log.h"
+#include "msg.h"
+#include "sconf.h"
 
 /*
  * Datagrams greater than this will be truncated
@@ -34,11 +35,7 @@ static char RCSid[] = "$Id: udpint.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $"
 
 struct packet
 {
-#ifdef INET6
-   struct sockaddr_in6   from ;
-#else
-   struct sockaddr_in   from ;
-#endif
+   union xsockaddr from ;
    char                  *data ;
    int                  size ;
 } ;
@@ -56,31 +53,25 @@ struct idgram_private
 
 static struct idgram_private idgram ;
 
-PRIVATE void di_mux() ;
-void di_exit() ;
+static void di_mux(void) ;
+static void udp_remote_to_local( struct intercept_s *ip, channel_s **chpp );
+static status_e udp_local_to_remote( channel_s *chp );
+static void send_data( int sd, char *buf, int len, union xsockaddr *addr );
+static status_e get_incoming_packet( struct intercept_s *ip, packet_s *pp );
 
-PRIVATE void udp_remote_to_local( struct intercept *ip, channel_s **chpp );
-PRIVATE status_e udp_local_to_remote( channel_s *chp );
-#ifdef INET6
-PRIVATE void send_data( int sd, char *buf, int len, struct sockaddr_in6 *addr );
-#else
-PRIVATE void send_data( int sd, char *buf, int len, struct sockaddr_in *addr );
-#endif
-PRIVATE status_e get_incoming_packet( struct intercept *ip, packet_s *pp );
-
-static struct intercept_ops idgram_ops =
+static const struct intercept_ops idgram_ops =
    {
       di_mux,
       di_exit
    } ;
 
 
-static struct intercept dgram_intercept_state ;
+static struct intercept_s dgram_intercept_state ;
 
 
-struct intercept *di_init( struct server *serp )
+struct intercept_s *di_init( struct server *serp )
 {
-   struct intercept *ip = &dgram_intercept_state ;
+   struct intercept_s *ip = &dgram_intercept_state ;
    
    ip->int_socket_type = SOCK_DGRAM ;
    ip->int_priv = (void *) &idgram ;
@@ -90,9 +81,9 @@ struct intercept *di_init( struct server *serp )
 }
 
 
-void di_exit()
+void di_exit(void)
 {
-   struct intercept *ip = &dgram_intercept_state ;
+   struct intercept_s *ip = &dgram_intercept_state ;
 
    if ( IDP( ip->int_priv )->received_packets == 0 )
       drain( INT_REMOTE( ip ) ) ;
@@ -103,9 +94,9 @@ void di_exit()
 /*
  * Returns only if there is an I/O error while communicating with the server
  */
-PRIVATE void di_mux()
+static void di_mux(void)
 {
-   struct intercept   *ip = &dgram_intercept_state ;
+   struct intercept_s   *ip = &dgram_intercept_state ;
    fd_set                     socket_mask ;
    int                        mask_max ;
 
@@ -161,12 +152,12 @@ PRIVATE void di_mux()
  * If this is a new connection, insert it in the connection table and
  * place its handle in *chpp.
  */
-PRIVATE void udp_remote_to_local( struct intercept *ip, channel_s **chpp )
+static void udp_remote_to_local( struct intercept_s *ip, channel_s **chpp )
 {
    char               buf[ MAX_DATAGRAM_SIZE ] ;
-   packet_s            packet ;
-   channel_s         *chp ;
-   bool_int            addr_checked ;
+   packet_s           packet ;
+   channel_s          *chp ;
+   bool_int           addr_checked ;
 
    *chpp = CHANNEL_NULL ;
 
@@ -223,7 +214,7 @@ PRIVATE void udp_remote_to_local( struct intercept *ip, channel_s **chpp )
 #endif
 
    send_data( chp->ch_local_socket,
-         packet.data, packet.size, SOCKADDRIN_NULL ) ;
+         packet.data, packet.size, NULL ) ;
 }
 
 
@@ -231,20 +222,16 @@ PRIVATE void udp_remote_to_local( struct intercept *ip, channel_s **chpp )
  * Send the data in buf to destination addr using the socket sd.
  * If addr is NULL, use the default socket destination
  */
-#ifdef INET6
-PRIVATE void send_data( int sd, char *buf, int len, struct sockaddr_in6 *addr )
-#else
-PRIVATE void send_data( int sd, char *buf, int len, struct sockaddr_in *addr )
-#endif
+static void send_data( int sd, char *buf, int len, union xsockaddr *addr )
 {
    char   *p ;
    int   left ;
    int   cc ;
-   char   *func = "send_data" ;
+   const char   *func = "send_data" ;
 
    for ( p = buf, left = len ; left > 0 ; left -= cc, p+= cc )
    {
-      if ( addr == SOCKADDRIN_NULL )
+      if ( addr == NULL )
          cc = send( sd, p, left, 0 ) ;
       else
          cc = sendto( sd, p, left, 0, SA( addr ), sizeof( *addr ) ) ;
@@ -265,10 +252,15 @@ PRIVATE void send_data( int sd, char *buf, int len, struct sockaddr_in *addr )
 }
 
 
-PRIVATE status_e get_incoming_packet( struct intercept *ip, packet_s *pp )
+static status_e get_incoming_packet( struct intercept_s *ip, packet_s *pp )
 {
-   int from_len ;
-   char *func = "get_incoming_packet" ;
+   int from_len = 0;
+   const char *func = "get_incoming_packet" ;
+
+   if( SC_IPV4( SVC_CONF( SERVER_SERVICE( INT_SERVER( ip ) ) ) ) )
+      from_len = sizeof( struct sockaddr_in );
+   if( SC_IPV6( SVC_CONF( SERVER_SERVICE( INT_SERVER( ip ) ) ) ) )
+      from_len = sizeof( struct sockaddr_in6 );
 
    for ( ;; )
    {
@@ -304,20 +296,18 @@ PRIVATE status_e get_incoming_packet( struct intercept *ip, packet_s *pp )
 #ifdef DEBUG_UDPINT
    if ( debug.on )
       msg( LOG_DEBUG, func, "Received %d bytes from address: %s,%d",
-         pp->size,
-            inet_ntoa( pp->from.sin_addr ), ntohs( pp->from.sin_port ) ) ;
+         pp->size, xaddrname( &pp->from ), ntohs( xaddrport(&pp->from) ) );
 #endif
 
    return( OK ) ;
 }
 
 
-
-PRIVATE status_e udp_local_to_remote( channel_s *chp )
+static status_e udp_local_to_remote( channel_s *chp )
 {
    char   buf[ MAX_DATAGRAM_SIZE ] ;
-   int   cc ;
-   char   *func = "udp_local_to_remote" ;
+   int    cc ;
+   const char   *func = "udp_local_to_remote" ;
 
    for ( ;; )
    {
@@ -340,7 +330,7 @@ PRIVATE status_e udp_local_to_remote( channel_s *chp )
 #ifdef DEBUG_UDPINT
    if ( debug.on )
       msg( LOG_DEBUG, func, "sending %d bytes to address %s,%d",
-         cc, inet_ntoa( chp->ch_from.sin_addr ), ntohs( chp->ch_from.sin_port ) ) ;
+         cc, xaddrname( &chp->ch_from ), ntohs( xaddrport(&chp->ch_from) ) ) ;
 #endif
 
    send_data( chp->ch_remote_socket, buf, cc, &chp->ch_from ) ;
