@@ -5,12 +5,12 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: util.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include "config.h"
 #include <sys/types.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <ctype.h>
 #if defined (HAVE_SYS_SOCKET_H)
 #include <sys/socket.h>
 #endif
@@ -37,36 +37,33 @@ static char RCSid[] = "$Id: util.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 #include <syslog.h>
 #include <errno.h>
 
-#include "pset.h"
 #include "sio.h"
-
-#include "defs.h"
-#include "xconfig.h"
-#include "log.h"
 #include "str.h"
+#include "util.h"
+#include "msg.h"
 
-void out_of_memory( char *func )
+void out_of_memory( const char *func )
 {
    msg( LOG_CRIT, func, ES_NOMEM ) ;
 }
 
 
-struct name_value *nv_find_value( struct name_value nv_array[], char *name )
+const struct name_value *nv_find_value( const struct name_value nv_array[], const char *name )
 {
-   struct name_value *nvp ;
+   int i;
 
-   for ( nvp = nv_array ; nvp->name ; nvp++ )
+   for ( i = 0 ; nv_array[i].name ; i++ )
    {
-      if ( EQ( name, nvp->name ) )
-         return( nvp ) ;
+      if ( EQ( name, nv_array[i].name ) )
+         return( &nv_array[i] ) ;
    }
    return( NULL ) ;
 }
 
 
-struct name_value *nv_find_name( struct name_value nv_array[], int value )
+const struct name_value *nv_find_name( const struct name_value nv_array[], int value )
 {
-   struct name_value *nvp ;
+   const struct name_value *nvp ;
 
    for ( nvp = nv_array ; nvp->name ; nvp++ )
    {
@@ -90,9 +87,9 @@ struct name_value *nv_find_name( struct name_value nv_array[], int value )
  * If the value field of the NULL entry is 0, then nv_get_name() will
  * return NULL.
  */
- char *nv_get_name( struct name_value nv_array[], int value )
+const char *nv_get_name( const struct name_value nv_array[], int value )
 {
-   struct name_value *nvp ;
+   const struct name_value *nvp ;
 
    for ( nvp = nv_array ; nvp->name ; nvp++ )
    {
@@ -108,7 +105,7 @@ char **argv_alloc( unsigned count )
 {
    unsigned argv_size = (count + 1) * sizeof( char *) ;
    char **argv ;
-   char *func = "new_argv" ;
+   const char *func = "new_argv" ;
 
    argv = (char **) malloc( argv_size ) ;
    if ( argv == NULL )
@@ -124,10 +121,10 @@ char **argv_alloc( unsigned count )
 /*
  * If size is 0, the pset holds strings
  */
-status_e copy_pset( pset_h from, pset_h *to, unsigned size )
+status_e copy_pset( const pset_h from, pset_h *to, unsigned size )
 {
    unsigned u ;
-   char *func = "copy_pset" ;
+   const char *func = "copy_pset" ;
 
    if ( *to == NULL )
    {
@@ -142,25 +139,25 @@ status_e copy_pset( pset_h from, pset_h *to, unsigned size )
    for ( u = 0 ; u < pset_count( from ) ; u++ )
    {
       char *p = (char *) pset_pointer( from, u ) ;
-      char *new ;
+      char *new_s ;
       
       if ( size == 0 )
-         new = new_string( p ) ;
+         new_s = new_string( p ) ;
       else
-         new = malloc( size ) ;
+         new_s = (char *)malloc( size ) ;
 
-      if ( new == NULL )
+      if ( new_s == NULL )
       {
          out_of_memory( func ) ;
          return( FAILED ) ;
       }
 
       if ( size != 0 )
-         (void) memcpy( new, p, (int) size ) ;
+         (void) memcpy( new_s, p, (int) size ) ;
 
-      if ( pset_add( *to, new ) == NULL )
+      if ( pset_add( *to, new_s ) == NULL )
       {
-         free( new ) ;
+         free( new_s ) ;
          out_of_memory( func ) ;
          return( FAILED ) ;
       }
@@ -172,11 +169,11 @@ status_e copy_pset( pset_h from, pset_h *to, unsigned size )
 /*
  * Disassociate from controlling terminal
  */
-void no_control_tty()
+void no_control_tty(void)
 {
 #if !defined(HAVE_SETSID)
    int fd ;
-   char *func = "no_control_tty" ;
+   const char *func = "no_control_tty" ;
 
    if ( ( fd = open( "/dev/tty", O_RDWR ) ) == -1 )
       msg( LOG_WARNING, func, "open of /dev/tty failed: %m" ) ;
@@ -196,14 +193,13 @@ void no_control_tty()
 /*
  * Write the whole buffer to the given file descriptor ignoring interrupts
  */
-status_e write_buf( int fd, char *buf, int len )
+status_e write_buf( int fd, const char *buf, int len )
 {
-   char *p ;
-   int cc ;
+   int cc, i ;
 
-   for ( p = buf ; len > 0 ; p += cc, len -= cc )
+   for ( i = 0 ; len > 0 ; i += cc, len -= cc )
    {
-      cc = write( fd, p, len ) ;
+      cc = write( fd, buf+i, len ) ;
       if ( cc == -1 )
       {
          if ( errno != EINTR )
@@ -215,7 +211,7 @@ status_e write_buf( int fd, char *buf, int len )
 }
 
 
-void tabprint( int fd, int tab_level, char *fmt, ...)
+void tabprint( int fd, int tab_level, const char *fmt, ...)
 {
    va_list ap ;
    int i ;
@@ -238,8 +234,42 @@ void drain( int sd )
    char cc ;
 
    cc = recv( sd, buf, sizeof( buf ), 0 ) ;
-   if( debug.on )
-      if ( cc == -1 )
-         msg( LOG_WARNING, "drain", "recv: %m" ) ;
 }
 
+/*
+ * Convert string to an int detecting errors.
+ */
+int parse_int(const char *str, int base, int term, int *res)
+{
+	unsigned char *endptr;
+	long strtol_res;
+
+/* SUSv2 says:
+ * "Because 0, LONG_MIN and LONG_MAX are returned on error and are also
+ * valid returns on success, an application wishing to check for error
+ * situations should set errno to 0, then call strtol(), then check errno." */
+	errno = 0;
+	strtol_res = strtol(str, (char **)&endptr, base);
+
+	if (errno == 0 && *str != NUL) {
+		/* Special case: -1 means allow trailing whitespace */
+		if (term == -1) {
+			while (*endptr != NUL && isspace(*endptr))
+				endptr++;
+			term = NUL;
+		}
+
+		if (*endptr == term) {
+			*res = strtol_res;
+			return 0;
+		}
+	}
+
+	*res = 0;
+	return -1;
+}
+
+int parse_base10(const char *str, int *res)
+{
+	return parse_int(str, 10, -1, res);
+}
