@@ -4,7 +4,7 @@
  * and conditions for redistribution.
  */
 /* This file contains OS dependant implementations of xgetloadavg().
- * xgetloadavg takes no arguements and simply returns the 1 minute
+ * xgetloadavg takes no arguments and simply returns the 1 minute
  * load average of the machine as a double.
  * Feel free to add implementations here, please update configure.in
  * to define HAVE_LOADAVG.  Defining this macro enables the option
@@ -13,14 +13,16 @@
  */
 
 #include "config.h"
+#ifdef HAVE_LOADAVG
 
 #ifdef linux
 #include <stdio.h>
+#include "xgetloadavg.h"
 
 #define LFILE "/proc/loadavg"
 #define MAX 32
 
-double xgetloadavg()
+double xgetloadavg(void)
 {
    FILE *fd;
    double ret = 0;
@@ -39,14 +41,14 @@ double xgetloadavg()
 
    return ret;
 }
-#endif
+#endif /* linux */
+
 
 #ifdef solaris
-#ifdef HAVE_KSTAT
-
+#ifdef HAVE_KSTAT_H
 #include <kstat.h>
 
-double xgetloadavg()
+double xgetloadavg(void)
 {
    kstat_ctl_t *kc = NULL;
    kstat_t     *ks = NULL;
@@ -85,11 +87,11 @@ double xgetloadavg()
 #endif /* HAVE_KSTAT */
 #endif /* solaris */
 
-#ifdef bsdi
 
+#if defined(bsdi) || defined(__APPLE__)
 #include <stdlib.h>
 
-double xgetloadavg()
+double xgetloadavg(void)
 {                  
    double loadavg[3];
 
@@ -102,13 +104,13 @@ double xgetloadavg()
       return loadavg[0];
    }
 }
-#endif /* bsdi */
+#endif /* bsdi || __APPLE__ */
+
 
 #ifdef __osf__
-
 #include <sys/table.h>
 
-double xgetloadavg()
+double xgetloadavg(void)
 {
   struct tbl_loadavg labuf;
 
@@ -124,7 +126,7 @@ double xgetloadavg()
 
   return (labuf.tl_avenrun.d[2]);
 }
-
 #endif /* __osf__ */
 
-      
+#endif /* HAVE_LOADAVG */      
+
